@@ -219,12 +219,12 @@ mx_bin_pick (ClutterActor       *self,
 /**
  * mx_bin_allocate_child:
  * @bin: An #MxBin
- * @box: The box to allocate the child within
+ * @box: The allocation box of the parent actor.
  * @flags: #ClutterAllocationFlags, usually provided by the.
  * clutter_actor_allocate function.
  *
- * Allocates the child of an #MxBin inside the given box. This function should
- * usually only be called by subclasses of #MxBin.
+ * Allocates the child of an #MxBin using the width and height from @box.
+ * This function should usually only be called by subclasses of #MxBin.
  *
  * This function can be used to allocate the child of an #MxBin if no special
  * allocation requirements are needed. It is similar to
@@ -284,7 +284,7 @@ mx_bin_get_preferred_width (ClutterActor *self,
 
   min_width = natural_width = padding.left + padding.right;
 
-  if (priv->child == NULL)
+  if (priv->child == NULL || !CLUTTER_ACTOR_IS_VISIBLE (priv->child))
     {
       if (min_width_p)
         *min_width_p = min_width;
@@ -323,7 +323,7 @@ mx_bin_get_preferred_height (ClutterActor *self,
 
   min_height = natural_height = padding.top + padding.bottom;
 
-  if (priv->child == NULL)
+  if (priv->child == NULL || !CLUTTER_ACTOR_IS_VISIBLE (priv->child))
     {
       if (min_height_p)
         *min_height_p = min_height;
