@@ -200,6 +200,7 @@ scrollable_set_adjustments (MxScrollable *scrollable,
         }
 
       priv->hadjustment = hadjustment;
+      g_object_notify (G_OBJECT (scrollable), "horizontal-adjustment");
     }
 
   if (vadjustment != priv->vadjustment)
@@ -221,6 +222,7 @@ scrollable_set_adjustments (MxScrollable *scrollable,
         }
 
       priv->vadjustment = vadjustment;
+      g_object_notify (G_OBJECT (scrollable), "vertical-adjustment");
     }
 }
 
@@ -367,8 +369,17 @@ mx_box_container_foreach (ClutterContainer *container,
                           gpointer          callback_data)
 {
   MxBoxLayoutPrivate *priv = MX_BOX_LAYOUT (container)->priv;
+  ClutterActor       *child;
+  GList              *list;
 
-  g_list_foreach (priv->children, (GFunc) callback, callback_data);
+  list = priv->children;
+  while (list)
+    {
+      child = list->data;
+      list  = list->next;
+
+      callback (child, callback_data);
+    }
 }
 
 /*
@@ -520,6 +531,8 @@ mx_box_layout_move_focus (MxFocusable      *focusable,
   if (!childlink)
     return NULL;
 
+  priv->last_focus = from;
+
   hint = mx_focus_hint_from_direction (direction);
 
   /* convert left/right and up/down into next/previous */
@@ -552,7 +565,6 @@ mx_box_layout_move_focus (MxFocusable      *focusable,
 
               if (focused)
                 {
-                  priv->last_focus = child;
                   update_adjustments (MX_BOX_LAYOUT (focusable), focused);
                   return focused;
                 }
@@ -572,7 +584,6 @@ mx_box_layout_move_focus (MxFocusable      *focusable,
 
               if (focused)
                 {
-                  priv->last_focus = child;
                   update_adjustments (MX_BOX_LAYOUT (focusable), focused);
                   return focused;
                 }
@@ -828,6 +839,9 @@ mx_box_layout_get_preferred_width (ClutterActor *actor,
   if (natural_width_p)
     *natural_width_p = 0;
 
+  if (for_height > 0)
+    for_height = MAX (0, for_height - padding.top - padding.bottom);
+
   for (l = priv->children; l; l = g_list_next (l))
     {
       gfloat child_min = 0, child_nat = 0;
@@ -904,6 +918,9 @@ mx_box_layout_get_preferred_height (ClutterActor *actor,
   if (natural_height_p)
     *natural_height_p = 0;
 
+  if (for_width > 0)
+    for_width = MAX (0, for_width - padding.left - padding.right);
+
 
   for (l = priv->children; l; l = g_list_next (l))
     {
@@ -991,15 +1008,14 @@ mx_box_layout_allocate (ClutterActor          *actor,
 
       if (CLUTTER_ACTOR_IS_VISIBLE (child))
         {
-          gboolean expand;
+          MxBoxLayoutChild *meta;
 
-          clutter_container_child_get ((ClutterContainer *) actor,
-                                       child,
-                                       "expand", &expand,
-                                       NULL);
+          meta = (MxBoxLayoutChild*)
+            clutter_container_get_child_meta ((ClutterContainer *) actor,
+                                              child);
           n_children++;
 
-          if (expand)
+          if (meta->expand)
             n_expand_children++;
 
         }
@@ -1150,19 +1166,13 @@ mx_box_layout_allocate (ClutterActor          *actor,
       ClutterActor *child = (ClutterActor*) l->data;
       ClutterActorBox child_box, old_child_box;
       gfloat child_nat, child_min;
-      gboolean xfill, yfill, expand;
-      MxAlign xalign, yalign;
+      MxBoxLayoutChild *meta;
 
       if (!CLUTTER_ACTOR_IS_VISIBLE (child))
         continue;
 
-      clutter_container_child_get ((ClutterContainer*) actor, child,
-                                   "x-fill", &xfill,
-                                   "y-fill", &yfill,
-                                   "x-align", &xalign,
-                                   "y-align", &yalign,
-                                   "expand", &expand,
-                                   NULL);
+      meta = (MxBoxLayoutChild*)
+        clutter_container_get_child_meta ((ClutterContainer *) actor, child);
 
       if (priv->orientation == MX_ORIENTATION_VERTICAL)
         {
@@ -1173,7 +1183,7 @@ mx_box_layout_allocate (ClutterActor          *actor,
 
           if (allocate_pref)
             {
-              if (expand)
+              if (meta->expand)
                 child_box.y2 = position + child_nat + (int)extra_space;
               else
                 child_box.y2 = position + child_nat;
@@ -1203,7 +1213,7 @@ mx_box_layout_allocate (ClutterActor          *actor,
 
           if (allocate_pref)
             {
-              if (expand)
+              if (meta->expand)
                 child_box.x2 = position + child_nat + (int)extra_space;
               else
                 child_box.x2 = position + child_nat;
@@ -1227,7 +1237,8 @@ mx_box_layout_allocate (ClutterActor          *actor,
 
       /* Adjust the box for alignment/fill */
       old_child_box = child_box;
-      mx_allocate_align_fill (child, &child_box, xalign, yalign, xfill, yfill);
+      mx_allocate_align_fill (child, &child_box, meta->x_align, meta->y_align,
+                              meta->x_fill, meta->y_fill);
 
       if (priv->is_animating)
         {
@@ -1306,6 +1317,28 @@ mx_box_layout_apply_transform (ClutterActor *a,
   cogl_matrix_translate (m, (int) -x, (int) -y, 0);
 }
 
+static gboolean
+mx_box_layout_get_paint_volume (ClutterActor       *actor,
+                                ClutterPaintVolume *volume)
+{
+  MxBoxLayoutPrivate *priv = MX_BOX_LAYOUT (actor)->priv;
+  ClutterVertex vertex;
+
+  if (!clutter_paint_volume_set_from_allocation (volume, actor))
+    return FALSE;
+
+  clutter_paint_volume_get_origin (volume, &vertex);
+
+  if (priv->hadjustment)
+    vertex.x += mx_adjustment_get_value (priv->hadjustment);
+
+  if (priv->vadjustment)
+    vertex.y += mx_adjustment_get_value (priv->vadjustment);
+
+  clutter_paint_volume_set_origin (volume, &vertex);
+
+  return TRUE;
+}
 
 static void
 mx_box_layout_paint (ClutterActor *actor)
@@ -1424,6 +1457,7 @@ mx_box_layout_class_init (MxBoxLayoutClass *klass)
   actor_class->get_preferred_width = mx_box_layout_get_preferred_width;
   actor_class->get_preferred_height = mx_box_layout_get_preferred_height;
   actor_class->apply_transform = mx_box_layout_apply_transform;
+  actor_class->get_paint_volume = mx_box_layout_get_paint_volume;
 
   actor_class->paint = mx_box_layout_paint;
   actor_class->pick = mx_box_layout_pick;
@@ -1527,6 +1561,12 @@ mx_box_layout_new (void)
   return g_object_new (MX_TYPE_BOX_LAYOUT, NULL);
 }
 
+ClutterActor *
+mx_box_layout_new_with_orientation (MxOrientation orientation)
+{
+  return g_object_new (MX_TYPE_BOX_LAYOUT, "orientation", orientation, NULL);
+}
+
 /**
  * mx_box_layout_set_orientation:
  * @box: A #MxBoxLayout
@@ -1815,6 +1855,8 @@ mx_box_layout_add_actor_with_properties (MxBoxLayout  *box,
  *   focused actor
  *
  * Enables or disables automatic scrolling to the focused actor.
+ *
+ * Since: 1.2
  */
 void
 mx_box_layout_set_scroll_to_focused (MxBoxLayout *box,
@@ -1839,6 +1881,8 @@ mx_box_layout_set_scroll_to_focused (MxBoxLayout *box,
  * Get the value of the #MxBoxLayout:scroll-to-focused property.
  *
  * Returns: #TRUE if automatically scrolling to the focused actor is enabled
+ *
+ * Since: 1.2
  */
 gboolean
 mx_box_layout_get_scroll_to_focused (MxBoxLayout *box)
