@@ -90,7 +90,6 @@ struct _MxButtonPrivate
   guint8            old_opacity;
 
   guint             is_pressed : 1;
-  guint             is_hover : 1;
   guint             is_toggle : 1;
   guint             is_toggled : 1;
 
@@ -169,8 +168,7 @@ mx_stylable_iface_init (MxStylableIface *iface)
 static MxFocusable*
 mx_button_accept_focus (MxFocusable *focusable, MxFocusHint hint)
 {
-  mx_stylable_set_style_pseudo_class (MX_STYLABLE (focusable), "focus");
-  MX_BUTTON (focusable)->priv->is_hover = TRUE;
+  mx_stylable_style_pseudo_class_add (MX_STYLABLE (focusable), "focus");
 
   clutter_actor_grab_key_focus (CLUTTER_ACTOR (focusable));
 
@@ -184,10 +182,7 @@ mx_button_move_focus (MxFocusable      *focusable,
 {
   /* check if focus is being moved from us */
   if (focusable == from)
-    {
-      mx_stylable_set_style_pseudo_class (MX_STYLABLE (focusable), "");
-      MX_BUTTON (focusable)->priv->is_hover = FALSE;
-    }
+    mx_stylable_style_pseudo_class_remove (MX_STYLABLE (focusable), "focus");
 
   return NULL;
 }
@@ -288,7 +283,7 @@ mx_button_push (MxButton           *button,
 
   //clutter_grab_pointer (CLUTTER_ACTOR (button));
 
-  mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "active");
+  mx_stylable_style_pseudo_class_add (MX_STYLABLE (button), "active");
 
   if (event)
     mx_widget_long_press_query (widget, event);
@@ -323,12 +318,7 @@ mx_button_pull (MxButton *button)
 
   mx_widget_long_press_cancel (widget);
 
-  if (priv->is_toggled)
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "checked");
-  else if (!priv->is_hover)
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), NULL);
-  else
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "hover");
+  mx_stylable_style_pseudo_class_remove (MX_STYLABLE (button), "active");
 }
 
 static gboolean
@@ -404,7 +394,6 @@ static gboolean
 mx_button_enter (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
 {
-  MxButton *button = MX_BUTTON (actor);
   MxWidget *widget = MX_WIDGET (actor);
 
   if (event->source != actor)
@@ -415,10 +404,7 @@ mx_button_enter (ClutterActor         *actor,
     return FALSE;
 
 
-  if (!button->priv->is_toggled)
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
-
-  button->priv->is_hover = 1;
+  mx_stylable_style_pseudo_class_add (MX_STYLABLE (widget), "hover");
 
   return FALSE;
 }
@@ -441,20 +427,16 @@ mx_button_leave (ClutterActor         *actor,
   if (mx_widget_get_disabled (MX_WIDGET (actor)))
     return FALSE;
 
-
-  button->priv->is_hover = 0;
-
   if (button->priv->is_pressed)
     {
       //clutter_ungrab_pointer ();
 
+      mx_widget_long_press_cancel (widget);
+      mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "active");
       button->priv->is_pressed = FALSE;
     }
 
-  if (button->priv->is_toggled)
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "checked");
-  else
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), NULL);
+  mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "hover");
 
   return FALSE;
 }
@@ -625,27 +607,6 @@ mx_button_unmap (ClutterActor *self)
     clutter_actor_unmap (priv->content_image);
 }
 
-static void
-mx_button_paint_background (MxWidget           *widget,
-                            ClutterActor       *background,
-                            const ClutterColor *color)
-{
-  MxButtonPrivate *priv;
-
-  priv = MX_BUTTON (widget)->priv;
-
-  if (priv->content_image)
-    {
-      clutter_actor_paint (priv->content_image);
-
-      return;
-    }
-
-  MX_WIDGET_CLASS (mx_button_parent_class)->paint_background (widget,
-                                                              background,
-                                                              color);
-}
-
 static void
 mx_button_allocate (ClutterActor           *actor,
                     const ClutterActorBox  *box,
@@ -728,7 +689,8 @@ mx_button_update_contents (MxButton *self)
 {
   MxButtonPrivate *priv = self->priv;
   ClutterActor *child;
-  gboolean icon_visible;
+  gboolean icon_visible, label_visible;
+  const gchar *text;
 
   /* If the icon doesn't have a name set, treat it as
    * not-visible.
@@ -738,6 +700,13 @@ mx_button_update_contents (MxButton *self)
   else
     icon_visible = FALSE;
 
+  text = clutter_text_get_text (CLUTTER_TEXT (priv->label));
+
+  if (priv->label_visible && text && (strcmp (text, "") != 0))
+    label_visible = TRUE;
+  else
+    label_visible = FALSE;
+
   /* replace any custom content */
   child = mx_bin_get_child (MX_BIN (self));
 
@@ -745,8 +714,7 @@ mx_button_update_contents (MxButton *self)
     mx_bin_set_child (MX_BIN (self), priv->hbox);
 
   /* Handle the simple cases first */
-  if (!icon_visible &&
-      !priv->label_visible)
+  if (!icon_visible && !label_visible)
     {
       clutter_actor_hide (priv->hbox);
       return;
@@ -755,8 +723,7 @@ mx_button_update_contents (MxButton *self)
   /* ensure the hbox is visible */
   clutter_actor_show (priv->hbox);
 
-  if (icon_visible &&
-      !priv->label_visible)
+  if (icon_visible && !label_visible)
     {
       clutter_actor_show (priv->icon);
       clutter_actor_hide (priv->label);
@@ -764,8 +731,7 @@ mx_button_update_contents (MxButton *self)
       return;
     }
 
-  if (!icon_visible &&
-      priv->label_visible)
+  if (!icon_visible && label_visible)
     {
       clutter_actor_hide (priv->icon);
       clutter_actor_show (priv->label);
@@ -840,7 +806,6 @@ mx_button_class_init (MxButtonClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
-  MxWidgetClass *widget_class = MX_WIDGET_CLASS (klass);
   GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (MxButtonPrivate));
@@ -864,8 +829,6 @@ mx_button_class_init (MxButtonClass *klass)
   actor_class->unmap = mx_button_unmap;
   actor_class->allocate = mx_button_allocate;
 
-  widget_class->paint_background = mx_button_paint_background;
-
   pspec = g_param_spec_string ("label",
                                "Label",
                                "Label of the button",
@@ -982,6 +945,8 @@ mx_button_init (MxButton *button)
                                   priv->icon, FALSE);
   mx_box_layout_child_set_x_fill (MX_BOX_LAYOUT (priv->hbox),
                                   priv->icon, FALSE);
+
+  mx_button_update_contents (button);
 }
 
 /**
@@ -1019,7 +984,7 @@ mx_button_new_with_label (const gchar *text)
  *
  * Returns: the text for the button. This must not be freed by the application
  */
-G_CONST_RETURN gchar *
+const gchar *
 mx_button_get_label (MxButton *button)
 {
   g_return_val_if_fail (MX_IS_BUTTON (button), NULL);
@@ -1128,12 +1093,9 @@ mx_button_set_toggled (MxButton *button,
       button->priv->is_toggled = toggled;
 
       if (toggled)
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "checked");
+        mx_stylable_style_pseudo_class_add (MX_STYLABLE (button), "checked");
       else
-      if (button->priv->is_hover)
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), "hover");
-      else
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (button), NULL);
+        mx_stylable_style_pseudo_class_remove (MX_STYLABLE (button), "checked");
 
       g_object_notify (G_OBJECT (button), "toggled");
     }
@@ -1146,6 +1108,8 @@ mx_button_set_toggled (MxButton *button,
  *
  * Sets @action as the action for @button. @Button will take its label and
  * icon from @action.
+ *
+ * Since: 1.2
  */
 void
 mx_button_set_action (MxButton *button,
@@ -1190,7 +1154,9 @@ mx_button_set_action (MxButton *button,
  *
  * Retrieves the #MxAction associated with @button.
  *
- * Returns: A #MxAction
+ * Returns: (transfer none): A #MxAction
+ *
+ * Since: 1.2
  */
 MxAction *
 mx_button_get_action (MxButton *button)
@@ -1206,6 +1172,8 @@ mx_button_get_action (MxButton *button)
  * @position: A #MxPosition
  *
  * Sets the icon position, relative to the text on the button.
+ *
+ * Since: 1.2
  */
 void
 mx_button_set_icon_position (MxButton   *button,
@@ -1233,6 +1201,8 @@ mx_button_set_icon_position (MxButton   *button,
  * Retrieves the icon's relative position to the text.
  *
  * Returns: A #MxPosition
+ *
+ * Since: 1.2
  */
 MxPosition
 mx_button_get_icon_position (MxButton *button)
@@ -1248,6 +1218,8 @@ mx_button_get_icon_position (MxButton *button)
  * @visible: %TRUE if the icon should be visible
  *
  * Sets the visibility of the icon associated with the button's action.
+ *
+ * Since: 1.2
  */
 void
 mx_button_set_icon_visible (MxButton *button,
@@ -1275,6 +1247,8 @@ mx_button_set_icon_visible (MxButton *button,
  * Retrieves the visibility of the icon associated with the button's action.
  *
  * Returns: %TRUE if the icon is visible, %FALSE otherwise
+ *
+ * Since: 1.2
  */
 gboolean
 mx_button_get_icon_visible (MxButton *button)
@@ -1289,6 +1263,8 @@ mx_button_get_icon_visible (MxButton *button)
  * @visible: %TRUE if the text should be visible
  *
  * Sets the visibility of the text associated with the button's action.
+ *
+ * Since: 1.2
  */
 void
 mx_button_set_label_visible (MxButton *button,
@@ -1317,6 +1293,8 @@ mx_button_set_label_visible (MxButton *button,
  * Retrieves the visibility of the text associated with the button's action.
  *
  * Returns: %TRUE if the text is visible, %FALSE otherwise
+ *
+ * Since: 1.2
  */
 gboolean
 mx_button_get_label_visible (MxButton *button)
@@ -1334,8 +1312,10 @@ mx_button_get_label_visible (MxButton *button)
  *
  * Returns: the icon-name. This must not be freed by the application. %NULL if
  *   no icon has been set
+ *
+ * Since: 1.2
  */
-G_CONST_RETURN gchar *
+const gchar *
 mx_button_get_icon_name (MxButton *button)
 {
   g_return_val_if_fail (MX_IS_BUTTON (button), NULL);
@@ -1352,6 +1332,8 @@ mx_button_get_icon_name (MxButton *button)
  * Sets the icon-name used to display an icon on the button. Setting %NULL
  * will remove the icon name, or resort to the icon-name set in the current
  * style. Setting an icon name overrides any icon set in the style.
+ *
+ * Since: 1.2
  */
 void
 mx_button_set_icon_name (MxButton    *button,
@@ -1380,6 +1362,8 @@ mx_button_set_icon_name (MxButton    *button,
  * Retrieves the icon-size being used for the displayed icon inside the button.
  *
  * Returns: The icon-size being used for the button icon, in pixels
+ *
+ * Since: 1.2
  */
 guint
 mx_button_get_icon_size (MxButton *button)
@@ -1397,6 +1381,8 @@ mx_button_get_icon_size (MxButton *button)
  * Sets the icon-size to use for the icon displayed inside the button. This will
  * override the icon-size set in the style. Setting a value of %0 resets to the
  * size from the style.
+ *
+ * Since: 1.2
  */
 void
 mx_button_set_icon_size (MxButton *button,
