@@ -217,8 +217,7 @@ mx_combo_box_unmap (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_combo_box_parent_class)->unmap (actor);
 
-  if (priv->label)
-    clutter_actor_unmap (priv->label);
+  clutter_actor_unmap (priv->label);
 
   if (priv->icon)
     clutter_actor_unmap (priv->icon);
@@ -420,7 +419,8 @@ mx_combo_box_allocate (ClutterActor          *actor,
       if (height >= nat_marker_h)
         {
           marker_h = nat_marker_h;
-          clutter_actor_get_preferred_width (priv->marker, -1, NULL, &marker_w);
+          clutter_actor_get_preferred_width (priv->marker, -1, NULL,
+                                             &marker_w);
         }
       else
         {
@@ -455,6 +455,8 @@ mx_combo_box_allocate (ClutterActor          *actor,
   childbox.x2 = (box->x2 - box->x1);
   childbox.y1 = (box->y2 - box->y1);
 
+  childbox.y2 = childbox.y1 + nat_menu_h;
+
   stage = clutter_actor_get_stage (actor);
   if (stage != NULL)
     {
@@ -466,14 +468,40 @@ mx_combo_box_allocate (ClutterActor          *actor,
       clutter_actor_apply_transform_to_point (actor, &point, &point);
 
       /* If the menu would appear off the stage, flip it around. */
-      if ((point.x < 0) || (point.x >= stage_w) ||
-          (point.y < 0) || (point.y >= stage_h))
+      if ((point.y < 0) || (point.y >= stage_h))
         {
           childbox.y1 = -nat_menu_h;
+          point.y = -nat_menu_h;
+          clutter_actor_apply_transform_to_point (actor, &point, &point);
+          /* if the menu would still appear out of the stage, force
+           * it to appear on the top of the stage.
+           */
+          if (point.y < 0)
+            {
+              gfloat xactor, yactor;
+              clutter_actor_get_transformed_position (actor, &xactor,
+                                                      &yactor);
+              childbox.y1 = -yactor;
+            }
+        }
+
+      point.y = childbox.y1 + nat_menu_h;
+      clutter_actor_apply_transform_to_point (actor, &point, &point);
+      if (point.y >= stage_h)
+        {
+          gfloat xactor, yactor;
+          clutter_actor_get_transformed_position (actor, &xactor, &yactor);
+          /*
+           * clamp so that the menu doesn't appear out of the screen
+           */
+          clutter_actor_transform_stage_point (actor, xactor, stage_h,
+                                               NULL, &childbox.y2);
+        }
+      else
+        {
+          childbox.y2 = childbox.y1 + nat_menu_h;
         }
     }
-
-  childbox.y2 = childbox.y1 + nat_menu_h;
   clutter_actor_allocate (menu, &childbox, flags);
 }
 
@@ -1017,7 +1045,7 @@ mx_combo_box_set_index (MxComboBox *box,
   if (!item)
     {
       box->priv->index = -1;
-      clutter_text_set_text ((ClutterText*) box->priv->label, "");
+      clutter_text_set_text ((ClutterText*) box->priv->label, NULL);
       return;
     }
 
