@@ -32,6 +32,8 @@
  *
  * It also allows actions to be added to it, which will be represented as
  * buttons, using #MxButton.
+ *
+ * Since: 1.2
  */
 
 #include "mx-dialog.h"
@@ -83,25 +85,7 @@ struct _MxDialogPrivate
   GList         *actions;
 };
 
-#ifdef COGL_HAS_GLES2
-
-#define GLES2_VARS \
-  "precision mediump float;\n" \
-  "varying vec2 tex_coord;\n" \
-  "varying vec4 frag_color;\n"
-#define TEX_COORD "tex_coord"
-#define COLOR_VAR "frag_color"
-
-#else
-
-#define GLES2_VARS ""
-#define TEX_COORD "gl_TexCoord[0]"
-#define COLOR_VAR "gl_Color"
-
-#endif
-
 static gchar *blur_shader =
-  GLES2_VARS
   "uniform sampler2D tex;\n"
   "uniform float x_step, y_step;\n"
 
@@ -117,18 +101,18 @@ static gchar *blur_shader =
   "      for (v = -1.0; v <= 1.0; v++)\n"
   "        {\n"
   "          color += texture2D(tex, \n"
-  "              vec2(" TEX_COORD ".s + u * x_step, \n"
-  "                   " TEX_COORD ".t + v * y_step));\n"
+  "              vec2(cogl_tex_coord_in[0].s + u * x_step, \n"
+  "                   cogl_tex_coord_in[0].t + v * y_step));\n"
   "          count ++;\n"
   "        }\n"
 
   "    color = color / float (count);\n"
-  "    gl_FragColor = color * "COLOR_VAR";\n"
+  "    cogl_color_out = color * cogl_color_in;\n"
   "  }\n";
 #else
   "vec4 get_rgba_rel(sampler2D tex, float dx, float dy)\n"
   "{\n"
-  "  return texture2D (tex, " TEX_COORD ".st \n"
+  "  return texture2D (tex, cogl_tex_coord_in[0].st \n"
   "                         + vec2(dx, dy));\n"
   "}\n"
 
@@ -146,7 +130,7 @@ static gchar *blur_shader =
   "    color += get_rgba_rel (tex,  x_step,  0.0);\n"
   "    color += get_rgba_rel (tex,  x_step,  y_step);\n"
   "    color = color / 9.0;\n"
-  "    gl_FragColor = color * "COLOR_VAR";\n"
+  "    cogl_color_out = color * cogl_color_in;\n"
   "  }\n";
 #endif
 
@@ -686,8 +670,6 @@ mx_dialog_paint (ClutterActor *actor)
       clutter_actor_paint (priv->blur);
     }
 
-  mx_widget_paint_background (MX_WIDGET (actor));
-
   cogl_translate (width/2, height/2, 0);
   cogl_scale (priv->zoom, priv->zoom, 1.f);
   cogl_rotate (priv->angle, 0, 0, 1);
@@ -1024,6 +1006,8 @@ mx_dialog_init (MxDialog *self)
  * Creates a new #MxDialog.
  *
  * Returns: A newly allocated #MxDialog
+ *
+ * Since: 1.2
  */
 ClutterActor *
 mx_dialog_new (void)
@@ -1038,6 +1022,8 @@ mx_dialog_new (void)
  *
  * Sets the parent of the #MxDialog. This is the actor over which the
  * modal frame will appear when clutter_actor_show() is called.
+ *
+ * Since: 1.2
  */
 void
 mx_dialog_set_transient_parent (MxDialog *dialog,
@@ -1051,6 +1037,18 @@ mx_dialog_set_transient_parent (MxDialog *dialog,
   clutter_actor_pop_internal (actor);
 }
 
+static void
+mx_dialog_steal_focus (MxDialog *self)
+{
+  ClutterActor *stage = clutter_actor_get_stage (CLUTTER_ACTOR (self));
+  if (stage)
+    {
+      MxFocusManager *manager =
+        mx_focus_manager_get_for_stage (CLUTTER_STAGE (stage));
+      mx_focus_manager_push_focus (manager, MX_FOCUSABLE (self));
+    }
+}
+
 static void
 mx_dialog_show (ClutterActor *self)
 {
@@ -1059,7 +1057,6 @@ mx_dialog_show (ClutterActor *self)
 
   if (!priv->visible)
     {
-      ClutterActor *stage;
       ClutterActor *parent = clutter_actor_get_parent (self);
 
       if (!parent)
@@ -1076,6 +1073,9 @@ mx_dialog_show (ClutterActor *self)
             CLUTTER_TIMELINE_BACKWARD : CLUTTER_TIMELINE_FORWARD;
           clutter_timeline_set_direction (priv->timeline, direction);
 
+          CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_VISIBLE);
+          mx_dialog_steal_focus (dialog);
+
           return;
         }
 
@@ -1119,14 +1119,7 @@ mx_dialog_show (ClutterActor *self)
       clutter_alpha_set_mode (priv->alpha, CLUTTER_EASE_OUT_QUAD);
       clutter_timeline_start (priv->timeline);
 
-      /* Steal focus */
-      stage = clutter_actor_get_stage (self);
-      if (stage)
-        {
-          MxFocusManager *manager =
-            mx_focus_manager_get_for_stage (CLUTTER_STAGE (stage));
-          mx_focus_manager_push_focus (manager, MX_FOCUSABLE (self));
-        }
+      mx_dialog_steal_focus (dialog);
     }
 }
 
@@ -1183,6 +1176,8 @@ mx_dialog_hide (ClutterActor *self)
  * @action: A #MxAction
  *
  * Adds an #MxButton that represents @action to the button area of @dialog
+ *
+ * Since: 1.2
  */
 void
 mx_dialog_add_action (MxDialog *dialog,
@@ -1215,6 +1210,8 @@ mx_dialog_add_action (MxDialog *dialog,
  * @action: A #MxAction
  *
  * Removes the button associated with @action from the button area of @dialog
+ *
+ * Since: 1.2
  */
 void
 mx_dialog_remove_action (MxDialog *dialog,
@@ -1261,8 +1258,11 @@ mx_dialog_remove_action (MxDialog *dialog,
  *
  * Retrieves a list of actions added to @dialog.
  *
- * Returns: A newly allocated #GList of #MxAction objects. The actions
- *          in the list are owned by the dialog.
+ * Returns: (transfer container) (element-type Mx.Action): A newly allocated
+ *   #GList of #MxAction objects. The actions in the list are owned by the
+ *   dialog.
+ *
+ * Since: 1.2
  */
 GList *
 mx_dialog_get_actions (MxDialog *dialog)
