@@ -66,6 +66,7 @@
 #include "mx-clipboard.h"
 #include "mx-focusable.h"
 #include "mx-private.h"
+#include "mx-tooltip.h"
 
 #ifdef HAVE_X11
 /* for pointer cursor support */
@@ -76,6 +77,8 @@
 
 #define HAS_FOCUS(actor) (clutter_actor_get_stage (actor) && clutter_stage_get_key_focus ((ClutterStage *) clutter_actor_get_stage (actor)) == actor)
 
+#define MX_ENTRY_TOOLTIP_DELAY 500
+
 /* properties */
 enum
 {
@@ -84,7 +87,10 @@ enum
   PROP_CLUTTER_TEXT,
   PROP_HINT_TEXT,
   PROP_TEXT,
-  PROP_PASSWORD_CHAR
+  PROP_PASSWORD_CHAR,
+  PROP_ICON_HIGHLIGHT_SUFFIX,
+  PROP_PRIMARY_ICON_TOOLTIP_TEXT,
+  PROP_SECONDARY_ICON_TOOLTIP_TEXT
 };
 
 /* signals */
@@ -106,18 +112,34 @@ struct _MxEntryPrivate
   gchar        *hint;
 
   ClutterActor *primary_icon;
+  ClutterActor *primary_icon_highlight;
+  MxTooltip    *primary_icon_tooltip;
+
   ClutterActor *secondary_icon;
+  ClutterActor *secondary_icon_highlight;
+  MxTooltip    *secondary_icon_tooltip;
+
+  gchar  *primary_icon_filename;
+  gchar  *secondary_icon_filename;
+
+  gchar  *icon_highlight_suffix;
 
   gfloat        spacing;
 
-  gboolean hint_visible;
   gunichar password_char;
 
-  gboolean  pause_undo;
   GQueue   *undo_history;
   gulong    undo_timeout_source;
 
-  gboolean scrolling;
+  guint hint_visible : 1;
+  guint pause_undo : 1;
+  guint scrolling : 1;
+  guint unicode_input_mode : 1;
+  guint pointer_in_entry : 1;
+
+  GString *preedit_string;
+
+  guint tooltip_timeout;
 };
 
 static guint entry_signals[LAST_SIGNAL] = { 0, };
@@ -153,6 +175,21 @@ mx_entry_set_property (GObject      *gobject,
       mx_entry_set_password_char (entry, g_value_get_uint (value));
       break;
 
+    case PROP_ICON_HIGHLIGHT_SUFFIX:
+      mx_entry_set_icon_highlight_suffix (entry,
+                                          g_value_get_string (value));
+      break;
+
+    case PROP_PRIMARY_ICON_TOOLTIP_TEXT:
+      mx_entry_set_primary_icon_tooltip_text (entry,
+                                              g_value_get_string (value));
+      break;
+
+    case PROP_SECONDARY_ICON_TOOLTIP_TEXT:
+      mx_entry_set_secondary_icon_tooltip_text (entry,
+                                                g_value_get_string (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -186,6 +223,22 @@ mx_entry_get_property (GObject    *gobject,
       g_value_set_uint (value, priv->password_char);
       break;
 
+    case PROP_ICON_HIGHLIGHT_SUFFIX:
+      g_value_set_string (value, priv->icon_highlight_suffix);
+      break;
+
+    case PROP_PRIMARY_ICON_TOOLTIP_TEXT:
+      if (priv->primary_icon_tooltip)
+        g_value_set_string (value,
+                            mx_tooltip_get_text (priv->primary_icon_tooltip));
+      break;
+
+    case PROP_SECONDARY_ICON_TOOLTIP_TEXT:
+      if (priv->secondary_icon_tooltip)
+        g_value_set_string (value,
+                            mx_tooltip_get_text (priv->secondary_icon_tooltip));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -199,10 +252,46 @@ mx_entry_dispose (GObject *object)
 
   if (priv->entry)
     {
-      clutter_actor_unparent (priv->entry);
+      clutter_actor_destroy (priv->entry);
       priv->entry = NULL;
     }
 
+  if (priv->primary_icon)
+    {
+      clutter_actor_destroy (priv->primary_icon);
+      priv->primary_icon = NULL;
+    }
+
+  if (priv->secondary_icon)
+    {
+      clutter_actor_destroy (priv->secondary_icon);
+      priv->secondary_icon = NULL;
+    }
+
+  if (priv->primary_icon_highlight)
+    {
+      clutter_actor_destroy (priv->primary_icon_highlight);
+      priv->primary_icon_highlight = NULL;
+    }
+
+  if (priv->secondary_icon_highlight)
+    {
+      clutter_actor_destroy (priv->secondary_icon_highlight);
+      priv->secondary_icon_highlight = NULL;
+    }
+
+  if (priv->primary_icon_tooltip)
+    {
+      clutter_actor_destroy (CLUTTER_ACTOR (priv->primary_icon_tooltip));
+      priv->primary_icon_tooltip = NULL;
+    }
+
+  if (priv->secondary_icon_tooltip)
+    {
+      clutter_actor_destroy (CLUTTER_ACTOR (priv->secondary_icon_tooltip));
+      priv->secondary_icon_tooltip = NULL;
+    }
+
   G_OBJECT_CLASS (mx_entry_parent_class)->dispose (object);
 }
 
@@ -227,6 +316,36 @@ mx_entry_finalize (GObject *object)
       priv->undo_timeout_source = 0;
     }
 
+  if (priv->tooltip_timeout)
+    {
+      g_source_remove (priv->tooltip_timeout);
+      priv->tooltip_timeout = 0;
+    }
+
+  if (priv->preedit_string)
+    {
+      g_string_free (priv->preedit_string, TRUE);
+      priv->preedit_string = NULL;
+    }
+
+  if (priv->primary_icon_filename)
+    {
+      g_free (priv->primary_icon_filename);
+      priv->primary_icon_filename = NULL;
+    }
+
+  if (priv->secondary_icon_filename)
+    {
+      g_free (priv->secondary_icon_filename);
+      priv->secondary_icon_filename = NULL;
+    }
+
+  if (priv->icon_highlight_suffix)
+    {
+      g_free (priv->icon_highlight_suffix);
+      priv->icon_highlight_suffix = NULL;
+    }
+
   G_OBJECT_CLASS (mx_entry_parent_class)->finalize (object);
 }
 
@@ -240,6 +359,8 @@ mx_stylable_iface_init (MxStylableIface *iface)
       GParamSpec *pspec;
       static const ClutterColor default_color
         = { 0x0, 0x9c, 0xcf, 0xff };
+      static const ClutterColor white
+        = { 0xff, 0xff, 0xff, 0xff };
 
       is_initialized = TRUE;
 
@@ -256,6 +377,13 @@ mx_stylable_iface_init (MxStylableIface *iface)
                                         &default_color,
                                         G_PARAM_READWRITE);
       mx_stylable_iface_install_property (iface, MX_TYPE_ENTRY, pspec);
+
+      pspec = clutter_param_spec_color ("selected-text-color",
+                                        "Selected Text Color",
+                                        "Color of the selected text",
+                                        &white,
+                                        G_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_ENTRY, pspec);
     }
 }
 
@@ -279,10 +407,12 @@ mx_entry_style_changed (MxWidget *self)
   MxEntryPrivate *priv = MX_ENTRY_PRIV (self);
   ClutterColor *caret_color = NULL;
   ClutterColor *selection_background_color = NULL;
+  ClutterColor *selected_text_color = NULL;
 
   mx_stylable_get (MX_STYLABLE (self),
                    "caret-color", &caret_color,
                    "selection-background-color", &selection_background_color,
+                   "selected-text-color", &selected_text_color,
                    NULL);
 
   if (caret_color)
@@ -298,6 +428,13 @@ mx_entry_style_changed (MxWidget *self)
       clutter_color_free (selection_background_color);
     }
 
+  if (selected_text_color)
+    {
+      clutter_text_set_selected_text_color (CLUTTER_TEXT (priv->entry),
+                                            selected_text_color);
+      clutter_color_free (selected_text_color);
+    }
+
   mx_stylable_apply_clutter_text_attributes (MX_STYLABLE (self),
                                              CLUTTER_TEXT (priv->entry));
 }
@@ -447,6 +584,17 @@ mx_entry_allocate (ClutterActor          *actor,
 
       /* reduce the size for the entry */
       child_box.x1 += icon_w + priv->spacing;
+
+      if (priv->primary_icon_tooltip)
+        mx_tooltip_set_tip_area_from_actor (priv->primary_icon_tooltip,
+                                            priv->primary_icon);
+
+      if (priv->primary_icon_highlight)
+        {
+          clutter_actor_allocate (priv->primary_icon_highlight,
+                                  &icon_box,
+                                  flags);
+        }
     }
 
   if (priv->secondary_icon)
@@ -468,6 +616,17 @@ mx_entry_allocate (ClutterActor          *actor,
 
       /* reduce the size for the entry */
       child_box.x2 -= icon_w - priv->spacing;
+
+      if (priv->secondary_icon_tooltip)
+        mx_tooltip_set_tip_area_from_actor (priv->secondary_icon_tooltip,
+                                            priv->secondary_icon);
+
+      if (priv->secondary_icon_highlight)
+        {
+          clutter_actor_allocate (priv->secondary_icon_highlight,
+                                  &icon_box,
+                                  flags);
+        }
     }
 
   clutter_actor_get_preferred_height (priv->entry, child_box.x2 - child_box.x1,
@@ -483,6 +642,13 @@ mx_entry_allocate (ClutterActor          *actor,
   child_box.y2 = child_box.y1 + entry_h;
 
   clutter_actor_allocate (priv->entry, &child_box, flags);
+
+ if (priv->secondary_icon_tooltip)
+   clutter_actor_allocate_preferred_size (CLUTTER_ACTOR (priv->secondary_icon_tooltip), flags);
+
+ if (priv->primary_icon_tooltip)
+   clutter_actor_allocate_preferred_size (CLUTTER_ACTOR (priv->primary_icon_tooltip), flags);
+
 }
 
 static void
@@ -551,9 +717,20 @@ mx_entry_paint (ClutterActor *actor)
   if (priv->primary_icon)
     clutter_actor_paint (priv->primary_icon);
 
+  if (priv->primary_icon_highlight)
+    clutter_actor_paint (priv->primary_icon_highlight);
+
+  if (priv->primary_icon_tooltip)
+    clutter_actor_paint (CLUTTER_ACTOR (priv->primary_icon_tooltip));
+
   if (priv->secondary_icon)
     clutter_actor_paint (priv->secondary_icon);
 
+  if (priv->secondary_icon_highlight)
+    clutter_actor_paint (priv->secondary_icon_highlight);
+
+  if (priv->secondary_icon_tooltip)
+    clutter_actor_paint (CLUTTER_ACTOR (priv->secondary_icon_tooltip));
 
   /* draw a shadow if the entry is not focused and the text is scrolling */
   stage = (ClutterStage *) clutter_actor_get_stage (priv->entry);
@@ -626,8 +803,20 @@ mx_entry_map (ClutterActor *actor)
   if (priv->primary_icon)
     clutter_actor_map (priv->primary_icon);
 
+  if (priv->primary_icon_highlight)
+    clutter_actor_map (priv->primary_icon_highlight);
+
   if (priv->secondary_icon)
     clutter_actor_map (priv->secondary_icon);
+
+  if (priv->secondary_icon_highlight)
+    clutter_actor_map (priv->secondary_icon_highlight);
+
+  if (priv->secondary_icon_tooltip)
+    clutter_actor_map (CLUTTER_ACTOR (priv->secondary_icon_tooltip));
+
+  if (priv->primary_icon_tooltip)
+    clutter_actor_map (CLUTTER_ACTOR (priv->primary_icon_tooltip));
 }
 
 static void
@@ -642,8 +831,20 @@ mx_entry_unmap (ClutterActor *actor)
   if (priv->primary_icon)
     clutter_actor_unmap (priv->primary_icon);
 
+  if (priv->primary_icon_highlight)
+    clutter_actor_unmap (priv->primary_icon_highlight);
+
+  if (priv->primary_icon_tooltip)
+    clutter_actor_unmap (CLUTTER_ACTOR (priv->primary_icon_tooltip));
+
   if (priv->secondary_icon)
     clutter_actor_unmap (priv->secondary_icon);
+
+  if (priv->secondary_icon_highlight)
+    clutter_actor_unmap (priv->secondary_icon_highlight);
+
+  if (priv->secondary_icon_tooltip)
+    clutter_actor_unmap (CLUTTER_ACTOR (priv->secondary_icon_tooltip));
 }
 
 static void
@@ -665,6 +866,87 @@ mx_entry_clipboard_callback (MxClipboard *clipboard,
   clutter_text_insert_text (ctext, text, cursor_pos);
 }
 
+static void
+mx_entry_update_preedit (MxEntry *entry)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
+  list = pango_attr_list_new ();
+  pango_attr_list_insert (list, attr);
+  clutter_text_set_preedit_string (CLUTTER_TEXT (entry->priv->entry),
+                                   entry->priv->preedit_string->str,
+                                   list, 0);
+  pango_attr_list_unref (list);
+}
+
+static gboolean
+mx_entry_captured_event (ClutterActor *actor,
+                         ClutterEvent *event)
+{
+  MxEntryPrivate *priv = MX_ENTRY_PRIV (actor);
+  ClutterKeyEvent *kevent;
+
+  /* handle key events during unicode input mode */
+  if (event->type == CLUTTER_KEY_PRESS && priv->unicode_input_mode)
+    {
+      kevent = (ClutterKeyEvent*) event;
+
+      if (kevent->keyval == CLUTTER_KEY_space
+          || kevent->keyval == CLUTTER_KEY_Return
+          || kevent->keyval == CLUTTER_KEY_Escape)
+        {
+          gunichar unichar;
+
+          if (sscanf (priv->preedit_string->str + 1, "%x", &unichar) == 1)
+            {
+              clutter_text_set_preedit_string (CLUTTER_TEXT (priv->entry), NULL,
+                                               NULL, 0);
+              if (kevent->keyval != CLUTTER_KEY_Escape)
+                clutter_text_insert_unichar (CLUTTER_TEXT (priv->entry),
+                                             unichar);
+              g_string_free (priv->preedit_string, TRUE);
+              priv->preedit_string = NULL;
+            }
+
+          priv->unicode_input_mode = FALSE;
+
+          return TRUE;
+        }
+      else if (kevent->keyval == CLUTTER_KEY_BackSpace)
+        {
+          g_string_truncate (priv->preedit_string,
+                             MAX (1, priv->preedit_string->len - 1));
+          mx_entry_update_preedit (MX_ENTRY (actor));
+        }
+      else
+        {
+          if (g_unichar_isxdigit (kevent->unicode_value))
+            {
+              g_string_append_c (priv->preedit_string, kevent->unicode_value);
+              mx_entry_update_preedit (MX_ENTRY (actor));
+            }
+        }
+
+      return TRUE;
+    }
+
+  /* button press should cancel unicode input mode */
+  if (event->type == CLUTTER_BUTTON_PRESS && priv->unicode_input_mode)
+    {
+      clutter_text_set_preedit_string (CLUTTER_TEXT (priv->entry), NULL,
+                                       NULL, 0);
+      g_string_free (priv->preedit_string, TRUE);
+      priv->preedit_string = NULL;
+      priv->unicode_input_mode = FALSE;
+    }
+
+
+  return FALSE;
+}
+
+
 static gboolean
 mx_entry_key_press_event (ClutterActor    *actor,
                           ClutterKeyEvent *event)
@@ -753,6 +1035,18 @@ mx_entry_key_press_event (ClutterActor    *actor,
       return TRUE;
     }
 
+  /* unicode input mode */
+  if (event->modifier_state & CLUTTER_CONTROL_MASK
+      && event->modifier_state & CLUTTER_SHIFT_MASK
+      && event->keyval == CLUTTER_KEY_U)
+  {
+    priv->unicode_input_mode = TRUE;
+
+    priv->preedit_string = g_string_new ("u");
+    mx_entry_update_preedit (MX_ENTRY (actor));
+  }
+
+
   return FALSE;
 }
 
@@ -766,32 +1060,38 @@ mx_entry_key_focus_in (ClutterActor *actor)
   clutter_actor_grab_key_focus (priv->entry);
 }
 
+#ifdef HAVE_X11
+static void
+mx_entry_set_cursor (MxEntry  *entry,
+                     gboolean  use_ibeam)
+{
+  Display *dpy;
+  ClutterActor *stage, *actor = CLUTTER_ACTOR (entry);
+  Window wid;
+  static Cursor ibeam = None;
+
+  dpy = clutter_x11_get_default_display ();
+  stage = clutter_actor_get_stage (actor);
+  wid = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
+
+  if (ibeam == None)
+    ibeam = XCreateFontCursor (dpy, XC_xterm);
+
+  if (use_ibeam)
+    XDefineCursor (dpy, wid, ibeam);
+  else
+    XUndefineCursor (dpy, wid);
+}
+#endif
+
 static gboolean
 mx_entry_swallow_crossing_event (ClutterActor         *actor,
                                  ClutterCrossingEvent *event)
 {
 
 #ifdef HAVE_X11
-  if (event->source == MX_ENTRY (actor)->priv->entry
-      && event->related != NULL)
-    {
-      Display *dpy;
-      ClutterActor *stage;
-      Window wid;
-      static Cursor ibeam = None;
-
-      dpy = clutter_x11_get_default_display ();
-      stage = clutter_actor_get_stage (actor);
-      wid = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
-
-      if (ibeam == None)
-        ibeam = XCreateFontCursor (dpy, XC_xterm);
-
-      if (event->type == CLUTTER_ENTER)
-        XDefineCursor (dpy, wid, ibeam);
-      else
-        XUndefineCursor (dpy, wid);
-    }
+  if (event->source == MX_ENTRY (actor)->priv->entry && event->related != NULL)
+    mx_entry_set_cursor (MX_ENTRY (actor), (event->type == CLUTTER_ENTER));
 #endif
 
   /* swallow enter and leave events, since the pseudo-class must not be set to
@@ -837,6 +1137,7 @@ mx_entry_class_init (MxEntryClass *klass)
   actor_class->button_press_event = mx_entry_swallow_button_event;
   actor_class->button_release_event = mx_entry_swallow_button_event;
 
+  actor_class->captured_event = mx_entry_captured_event;
   actor_class->key_press_event = mx_entry_key_press_event;
   actor_class->key_focus_in = mx_entry_key_focus_in;
 
@@ -866,6 +1167,29 @@ mx_entry_class_init (MxEntryClass *klass)
                                 0, G_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_PASSWORD_CHAR, pspec);
 
+  pspec = g_param_spec_string ("icon-highlight-suffix",
+                               "Icon highlight suffix",
+                               "The filename suffix for the highligh icon",
+                               NULL, G_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_ICON_HIGHLIGHT_SUFFIX,
+                                   pspec);
+
+  pspec = g_param_spec_string ("primary-icon-tooltip-text",
+                               "Tooltip for the primary icon",
+                               "The tooltip text for the primary icon",
+                               NULL, G_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_PRIMARY_ICON_TOOLTIP_TEXT,
+                                   pspec);
+
+  pspec = g_param_spec_string ("secondary-icon-tooltip-text",
+                               "Tooltip for the secondary icon",
+                               "The tooltip text for the secondary icon",
+                               NULL, G_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class,
+                                   PROP_SECONDARY_ICON_TOOLTIP_TEXT,
+                                   pspec);
+
   /* signals */
   /**
    * MxEntry::primary-icon-clicked:
@@ -983,6 +1307,40 @@ clutter_text_changed_cb (ClutterText *text,
   mx_entry_store_undo_history (text, entry);
 }
 
+
+static gboolean
+entry_event (ClutterActor *actor,
+             ClutterEvent *event,
+             MxEntry      *entry)
+{
+#ifdef HAVE_X11
+  /* ensure the cursor is in the correct state when the mouse button is
+   * released */
+  if (event->type == CLUTTER_BUTTON_RELEASE)
+    mx_entry_set_cursor (entry, entry->priv->pointer_in_entry);
+#endif
+
+  /* don't track enter and leave events for actors other than the entry */
+  if (event->any.source != actor)
+    return FALSE;
+
+  switch (event->type)
+    {
+    case CLUTTER_LEAVE:
+      entry->priv->pointer_in_entry = FALSE;
+      break;
+
+    case CLUTTER_ENTER:
+      entry->priv->pointer_in_entry = TRUE;
+      break;
+
+    default:
+      break;
+    }
+
+  return FALSE;
+}
+
 static void
 mx_entry_init (MxEntry *entry)
 {
@@ -1013,10 +1371,14 @@ mx_entry_init (MxEntry *entry)
   g_signal_connect (priv->entry, "key-press-event",
                     G_CALLBACK (mx_entry_store_undo_on_keypress), entry);
 
+  g_signal_connect (priv->entry, "event", G_CALLBACK (entry_event), entry);
 
   priv->spacing = 6.0f;
 
+  clutter_actor_push_internal (CLUTTER_ACTOR (entry));
   clutter_actor_set_parent (priv->entry, CLUTTER_ACTOR (entry));
+  clutter_actor_pop_internal (CLUTTER_ACTOR (entry));
+
   clutter_actor_set_reactive ((ClutterActor *) entry, TRUE);
 
   /* set cursor hidden until we receive focus */
@@ -1068,7 +1430,7 @@ mx_entry_new_with_text (const gchar *text)
  *
  * Returns: the text for the entry. This must not be freed by the application
  */
-G_CONST_RETURN gchar *
+const gchar *
 mx_entry_get_text (MxEntry *entry)
 {
   g_return_val_if_fail (MX_IS_ENTRY (entry), NULL);
@@ -1088,13 +1450,19 @@ mx_entry_get_text (MxEntry *entry)
  */
 void
 mx_entry_set_text (MxEntry     *entry,
-                   const gchar *text)
+                   const gchar *entry_text)
 {
   MxEntryPrivate *priv;
   gunichar password_char;
+  const gchar *text;
 
   g_return_if_fail (MX_IS_ENTRY (entry));
 
+  if (entry_text)
+    text = entry_text;
+  else
+    text = "";
+
   priv = entry->priv;
 
   /* set a hint if we are blanking the entry */
@@ -1188,7 +1556,7 @@ mx_entry_set_hint_text (MxEntry     *entry,
  * Returns: the current value of the hint property. This string is owned by the
  * #MxEntry and should not be freed or modified.
  */
-G_CONST_RETURN
+const
 gchar *
 mx_entry_get_hint_text (MxEntry *entry)
 {
@@ -1248,13 +1616,195 @@ _mx_entry_icon_press_cb (ClutterActor       *actor,
   MxEntryPrivate *priv = entry->priv;
 
   if (actor == priv->primary_icon)
-    g_signal_emit (entry, entry_signals[PRIMARY_ICON_CLICKED], 0);
+    {
+      g_signal_emit (entry, entry_signals[PRIMARY_ICON_CLICKED], 0);
+      if (priv->primary_icon_tooltip)
+        mx_tooltip_hide (MX_TOOLTIP (priv->primary_icon_tooltip));
+    }
   else
-    g_signal_emit (entry, entry_signals[SECONDARY_ICON_CLICKED], 0);
+    {
+      g_signal_emit (entry, entry_signals[SECONDARY_ICON_CLICKED], 0);
+      if (priv->secondary_icon_tooltip)
+        mx_tooltip_hide (MX_TOOLTIP (priv->secondary_icon_tooltip));
+    }
 
   return FALSE;
 }
 
+
+static gboolean _mx_entry_tooltip_timeout_cb (gpointer data)
+{
+  if (!CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR (data)))
+    mx_tooltip_show (MX_TOOLTIP (data));
+  else
+    mx_tooltip_hide (MX_TOOLTIP (data));
+
+  return FALSE;
+}
+
+
+static guint
+_mx_entry_which_icon (ClutterActor *actor, MxEntry *entry)
+{
+  MxEntryPrivate *priv = entry->priv;
+
+  if (actor == priv->primary_icon)
+    return 1;
+  else if (actor == priv->secondary_icon)
+    return 2;
+
+  return 0;
+}
+
+static gchar *
+_mx_entry_make_highlight_filename (MxEntry     *entry,
+                                   const gchar *original_filename)
+{
+  MxEntryPrivate *priv = entry->priv;
+
+  gchar *extension, *highlight_icon_path, *filepath;
+
+  extension = g_strrstr (original_filename, ".");
+  filepath = g_strndup (original_filename, (strlen (original_filename)
+                                            - strlen (extension)));
+
+  highlight_icon_path = g_strconcat (filepath,
+                                     priv->icon_highlight_suffix,
+                                     extension,
+                                     NULL);
+
+  g_free (filepath);
+
+  if (!g_file_test (highlight_icon_path, G_FILE_TEST_EXISTS))
+    {
+      g_free (highlight_icon_path);
+      highlight_icon_path = NULL;
+    }
+
+  return highlight_icon_path;
+}
+
+static void
+_mx_entry_switch_to_highlight_icon (ClutterActor *icon,
+                                    ClutterActor *icon_highlight,
+                                    gboolean highlighted)
+{
+  /* Switches round the highlight icon to visible or the normal icon
+   * depending on highlighted
+   */
+  guint icon_opacity;
+  guint highlight_icon_opacity;
+
+  if (!icon_highlight)
+    return;
+
+  icon_opacity = !highlighted*255;
+  highlight_icon_opacity = highlighted*255;
+
+  if (clutter_actor_get_opacity (icon) != icon_opacity)
+    clutter_actor_set_opacity (icon, icon_opacity);
+  if(clutter_actor_get_opacity (icon_highlight) != highlight_icon_opacity)
+    clutter_actor_set_opacity (icon_highlight, highlight_icon_opacity);
+}
+
+
+static gboolean
+_mx_entry_icon_motion_event_cb (ClutterActor *actor,
+                                ClutterEvent *event,
+                                MxEntry      *entry)
+{
+  MxEntryPrivate *priv = entry->priv;
+  MxTooltip *target_tooltip = NULL;
+  guint icon_position;
+
+  icon_position = _mx_entry_which_icon (actor, entry);
+
+  if (icon_position  == 1)
+    {
+      target_tooltip = priv->primary_icon_tooltip;
+
+      _mx_entry_switch_to_highlight_icon (priv->primary_icon,
+                                          priv->primary_icon_highlight,
+                                          TRUE);
+    }
+  else if (icon_position == 2)
+    {
+      target_tooltip = priv->secondary_icon_tooltip;
+
+
+      _mx_entry_switch_to_highlight_icon (priv->secondary_icon,
+                                          priv->secondary_icon_highlight,
+                                          TRUE);
+    }
+
+  /* Show the tooltip if it's not already visible */
+
+  if (target_tooltip && !CLUTTER_ACTOR_IS_VISIBLE (target_tooltip))
+    {
+      if (mx_tooltip_is_in_browse_mode ())
+        {
+          mx_tooltip_show (target_tooltip);
+
+          if (priv->tooltip_timeout)
+            {
+              g_source_remove (priv->tooltip_timeout);
+              priv->tooltip_timeout = 0;
+            }
+        }
+      else
+        {
+          /* Not in browse mode so add a small delay before showing tooltip */
+          if (!priv->tooltip_timeout)
+            priv->tooltip_timeout =
+              clutter_threads_add_timeout (MX_ENTRY_TOOLTIP_DELAY,
+                                           _mx_entry_tooltip_timeout_cb,
+                                           target_tooltip);
+        }
+    }
+  return FALSE;
+}
+
+static gboolean
+_mx_entry_icon_leave_cb (ClutterActor *actor,
+                         ClutterEvent *event,
+                         MxEntry      *entry)
+{
+  MxEntryPrivate *priv = entry->priv;
+  MxTooltip *target_tooltip = NULL;
+
+  guint icon_position;
+
+  icon_position = _mx_entry_which_icon (actor, entry);
+
+  if (icon_position == 1)
+    {
+      target_tooltip = priv->primary_icon_tooltip;
+
+      _mx_entry_switch_to_highlight_icon (priv->primary_icon,
+                                          priv->primary_icon_highlight,
+                                          FALSE);
+    }
+  else if (icon_position == 2)
+    {
+      target_tooltip = priv->secondary_icon_tooltip;
+
+      _mx_entry_switch_to_highlight_icon (priv->secondary_icon,
+                                          priv->secondary_icon_highlight,
+                                          FALSE);
+    }
+
+  if (priv->tooltip_timeout)
+    {
+      g_source_remove (priv->tooltip_timeout);
+      priv->tooltip_timeout = 0;
+    }
+
+  mx_tooltip_hide (target_tooltip);
+
+  return FALSE;
+}
+
+
 static void
 _mx_entry_set_icon_from_file (MxEntry       *entry,
                               ClutterActor **icon,
@@ -1265,6 +1815,12 @@ _mx_entry_set_icon_from_file (MxEntry       *entry,
       g_signal_handlers_disconnect_by_func (*icon,
                                             _mx_entry_icon_press_cb,
                                             entry);
+      g_signal_handlers_disconnect_by_func (*icon,
+                                            _mx_entry_icon_motion_event_cb,
+                                            entry);
+      g_signal_handlers_disconnect_by_func (*icon,
+                                            _mx_entry_icon_leave_cb,
+                                            entry);
       clutter_actor_unparent (*icon);
       *icon = NULL;
     }
@@ -1283,14 +1839,64 @@ _mx_entry_set_icon_from_file (MxEntry       *entry,
           return;
 
       clutter_actor_set_reactive (*icon, TRUE);
+
+      clutter_actor_push_internal (CLUTTER_ACTOR (entry));
       clutter_actor_set_parent (*icon, CLUTTER_ACTOR (entry));
+      clutter_actor_pop_internal (CLUTTER_ACTOR (entry));
+
       g_signal_connect (*icon, "button-release-event",
                         G_CALLBACK (_mx_entry_icon_press_cb), entry);
+
+      g_signal_connect (*icon, "motion-event",
+                        G_CALLBACK (_mx_entry_icon_motion_event_cb),
+                        entry);
+
+      g_signal_connect (*icon, "leave-event",
+                        G_CALLBACK (_mx_entry_icon_leave_cb),
+                        entry);
     }
 
   clutter_actor_queue_relayout (CLUTTER_ACTOR (entry));
 }
 
+
+static void
+_mx_entry_create_highlight_icon (MxEntry *entry, guint icon_position)
+{
+  MxEntryPrivate *priv = entry->priv;
+
+  ClutterActor **target = NULL;
+  const gchar *filename = NULL;
+  gchar *highlight_filename;
+
+  if (!priv->icon_highlight_suffix)
+    return;
+
+  if (icon_position == 1)
+    {
+      target = &priv->primary_icon_highlight;
+      filename = priv->primary_icon_filename;
+    }
+  else if (icon_position == 2)
+    {
+      target = &priv->secondary_icon_highlight;
+      filename = priv->secondary_icon_filename;
+    }
+
+  if (!filename)
+    return;
+
+  highlight_filename = _mx_entry_make_highlight_filename (entry, filename);
+
+  if (highlight_filename)
+    {
+      _mx_entry_set_icon_from_file (entry, target, highlight_filename);
+
+      clutter_actor_set_opacity (*target, 0x0);
+      g_free (highlight_filename);
+    }
+}
+
 /**
  * mx_entry_set_primary_icon_from_file:
  * @entry: a #MxEntry
@@ -1308,8 +1914,13 @@ mx_entry_set_primary_icon_from_file (MxEntry     *entry,
 
   priv = entry->priv;
 
-  _mx_entry_set_icon_from_file (entry, &priv->primary_icon, filename);
+  if (priv->primary_icon_filename)
+    g_free (priv->primary_icon_filename);
 
+  priv->primary_icon_filename = g_strdup (filename);
+
+  _mx_entry_set_icon_from_file (entry, &priv->primary_icon, filename);
+  _mx_entry_create_highlight_icon (entry, 1);
 }
 
 /**
@@ -1329,7 +1940,131 @@ mx_entry_set_secondary_icon_from_file (MxEntry     *entry,
 
   priv = entry->priv;
 
-  _mx_entry_set_icon_from_file (entry, &priv->secondary_icon, filename);
+  if (priv->secondary_icon_filename)
+    g_free (priv->secondary_icon_filename);
 
+  priv->secondary_icon_filename = g_strdup (filename);
+
+  _mx_entry_set_icon_from_file (entry, &priv->secondary_icon, filename);
+  _mx_entry_create_highlight_icon (entry, 2);
 }
 
+/**
+ * mx_entry_set_secondary_icon_tooltip:
+ * @entry: a #MxEntry
+ * @text: the secondary icon tooltip
+ *
+ * Set the secondary icon tooltip text
+*/
+void
+mx_entry_set_secondary_icon_tooltip_text (MxEntry     *entry,
+                                          const gchar *text)
+{
+  MxEntryPrivate *priv;
+
+  g_return_if_fail (MX_IS_ENTRY (entry));
+
+  priv = entry->priv;
+
+  if (!priv->secondary_icon_tooltip)
+    {
+      priv->secondary_icon_tooltip = g_object_new (MX_TYPE_TOOLTIP,
+                                                  "text", text,
+                                                  NULL);
+
+      mx_tooltip_set_text (priv->secondary_icon_tooltip, text);
+      clutter_actor_set_parent (CLUTTER_ACTOR (priv->secondary_icon_tooltip),
+                                CLUTTER_ACTOR (entry));
+    }
+  else
+    {
+      mx_tooltip_set_text (priv->secondary_icon_tooltip, text);
+    }
+}
+
+/**
+ * mx_entry_set_primary_icon_tooltip:
+ * @entry: a #MxEntry
+ * @text: the primary icon tooltip
+ *
+ * Set the primary icon tooltip text
+*/
+void
+mx_entry_set_primary_icon_tooltip_text (MxEntry *entry,
+                                          const gchar *text)
+{
+  MxEntryPrivate *priv;
+
+  g_return_if_fail (MX_IS_ENTRY (entry));
+
+  priv = entry->priv;
+
+  if (!priv->primary_icon_tooltip)
+    {
+      priv->primary_icon_tooltip = g_object_new (MX_TYPE_TOOLTIP,
+                                                  "text", text,
+                                                  NULL);
+
+      mx_tooltip_set_text (priv->primary_icon_tooltip, text);
+      clutter_actor_set_parent (CLUTTER_ACTOR (priv->primary_icon_tooltip),
+                                CLUTTER_ACTOR (entry));
+
+
+    }
+  else
+    {
+      mx_tooltip_set_text (priv->primary_icon_tooltip, text);
+    }
+}
+
+/**
+ * mx_entry_get_icon_highlight_suffix:
+ * @entry: a #MxEntry
+ *
+ * Get the suffix appended to the filename to use for the highlighted version
+ * of the icon.
+ *
+ * Returns: the highlight filename suffix. This string is owned by the
+ * #MxEntry and should not be freed or modified.
+*/
+const gchar *mx_entry_get_icon_highlight_suffix (MxEntry *entry)
+{
+  MxEntryPrivate *priv;
+
+  g_return_val_if_fail (MX_IS_ENTRY (entry), NULL);
+
+  priv = entry->priv;
+
+  return priv->icon_highlight_suffix;
+}
+
+/**
+ * mx_entry_set_icon_highlight_suffix:
+ * @entry: a #MxEntry
+ * @suffix: the suffix to append to the filename for the highlight version
+ *
+ * Sets the suffix appended to the filename to use for the highlighted version
+ * of the icon. e.g. if you have set your primay icon to "primary-icon.png"
+ * and the suffix to "-highlight" #MxEntry will look for
+ * "primary-icon-highlight.png"
+*/
+void
+mx_entry_set_icon_highlight_suffix (MxEntry *entry, const gchar* suffix)
+{
+  MxEntryPrivate *priv;
+
+  g_return_if_fail (MX_IS_ENTRY (entry));
+
+  priv = entry->priv;
+
+  if (g_strcmp0 (priv->icon_highlight_suffix, suffix) == 0)
+      return;
+
+  if (priv->icon_highlight_suffix)
+    g_free (priv->icon_highlight_suffix);
+
+  priv->icon_highlight_suffix = g_strdup (suffix);
+
+  _mx_entry_create_highlight_icon (entry, 1);
+  _mx_entry_create_highlight_icon (entry, 2);
+}
