@@ -41,15 +41,15 @@
  * This layout is particularly flexible, with the following configuration possibilities:
  *
  * <itemizedlist>
- *   <listitem><para>Column and row spacing are controllable 
+ *   <listitem><para>Column and row spacing are controllable
  *   (#MxGrid:column_spacing and #MxGrid:row_spacing)</para></listitem>
- *   <listitem><para>Column and row sizes can be made consistent, regardless of the size 
+ *   <listitem><para>Column and row sizes can be made consistent, regardless of the size
  *   of the contained actors (#MxGrid:homogenous_columns and #MxGrid:homogenous_rows)
  *   </para></listitem>
  *   <listitem><para>Prefer to pack children vertically first,
  *   rather than horizontally (#MxGrid:orientation)</para></listitem>
- *   <listitem><para>Specify the maximum number of rows or columns 
- *   to allow in the layout, to prevent it from being excessively stretched 
+ *   <listitem><para>Specify the maximum number of rows or columns
+ *   to allow in the layout, to prevent it from being excessively stretched
  *   (#MxGrid:max_stride).</para></listitem>
  * </itemizedlist>
  *
@@ -57,9 +57,9 @@
  *
  * <figure id="mx-grid">
  *   <title>MxGrid flowing across multiple rows</title>
- *   <para>An #MxGrid containing 9 child actors; 
- *   #MxGrid:orientation is set to the default (MX_ORIENTATION_HORIZONTAL, 
- *   i.e. lay out horizontally first); #MxGrid:max_stride has not been set 
+ *   <para>An #MxGrid containing 9 child actors;
+ *   #MxGrid:orientation is set to the default (MX_ORIENTATION_HORIZONTAL,
+ *   i.e. lay out horizontally first); #MxGrid:max_stride has not been set
  *   (so there's no maximum row size); #MxGrid:column_spacing and #MxGrid:row_spacing have
  *   been set so that there is spacing between cells vertically and horizontally.</para>
  *   <graphic fileref="MxGrid-3x3.png" format="PNG"/>
@@ -74,12 +74,12 @@
  *
  * <figure id="mx-grid-two-rows">
  *   <title>MxGrid flowing onto two rows</title>
- *   <para>The same #MxGrid with 9 children wrapping onto two rows: notice 
- *   how the "odd" rectangle is on the end of a row, rather than at the 
- *   bottom of a column. This is because preference 
- *   is being given to packing onto the end of rows, rather than columns, 
- *   because #MxGrid:orientation is set to MX_ORIENTATION_HORIZONTAL. Even though 
- *   there is room for the rectangle at the bottom of the column, the 
+ *   <para>The same #MxGrid with 9 children wrapping onto two rows: notice
+ *   how the "odd" rectangle is on the end of a row, rather than at the
+ *   bottom of a column. This is because preference
+ *   is being given to packing onto the end of rows, rather than columns,
+ *   because #MxGrid:orientation is set to MX_ORIENTATION_HORIZONTAL. Even though
+ *   there is room for the rectangle at the bottom of the column, the
  *   layout prefers to place children onto the end of a row if there is room.</para>
  *   <graphic fileref="MxGrid-2rows-row-major.png" format="PNG"/>
  * </figure>
@@ -166,7 +166,9 @@ mx_grid_do_allocate (ClutterActor          *self,
                      ClutterAllocationFlags flags,
                      gboolean               calculate_extents_only,
                      gfloat                *actual_width,
-                     gfloat                *actual_height);
+                     gfloat                *actual_height,
+                     gfloat                *min_width,
+                     gfloat                *min_height);
 
 static void scrollable_interface_init (MxScrollableIface *iface);
 static void mx_box_focusable_iface_init (MxFocusableIface *iface);
@@ -269,6 +271,7 @@ scrollable_set_adjustments (MxScrollable *scrollable,
         }
 
       priv->hadjustment = hadjustment;
+      g_object_notify (G_OBJECT (scrollable), "horizontal-adjustment");
     }
 
   if (vadjustment != priv->vadjustment)
@@ -290,6 +293,7 @@ scrollable_set_adjustments (MxScrollable *scrollable,
         }
 
       priv->vadjustment = vadjustment;
+      g_object_notify (G_OBJECT (scrollable), "vertical-adjustment");
     }
 }
 
@@ -380,6 +384,29 @@ mx_grid_apply_transform (ClutterActor *a,
   cogl_matrix_translate (m , (int) -x, (int) -y, 0);
 }
 
+static gboolean
+mx_grid_get_paint_volume (ClutterActor       *actor,
+                          ClutterPaintVolume *volume)
+{
+  MxGridPrivate *priv = MX_GRID (actor)->priv;
+  ClutterVertex vertex;
+
+  if (!clutter_paint_volume_set_from_allocation (volume, actor))
+    return FALSE;
+
+  clutter_paint_volume_get_origin (volume, &vertex);
+
+  if (priv->hadjustment)
+    vertex.x += mx_adjustment_get_value (priv->hadjustment);
+
+  if (priv->vadjustment)
+    vertex.y += mx_adjustment_get_value (priv->vadjustment);
+
+  clutter_paint_volume_set_origin (volume, &vertex);
+
+  return TRUE;
+}
+
 static void
 mx_grid_class_init (MxGridClass *klass)
 {
@@ -400,6 +427,7 @@ mx_grid_class_init (MxGridClass *klass)
   actor_class->get_preferred_height = mx_grid_get_preferred_height;
   actor_class->allocate             = mx_grid_allocate;
   actor_class->apply_transform      = mx_grid_apply_transform;
+  actor_class->get_paint_volume     = mx_grid_get_paint_volume;
 
   g_type_class_add_private (klass, sizeof (MxGridPrivate));
 
@@ -561,6 +589,8 @@ mx_grid_move_focus (MxFocusable      *focusable,
   if (!childlink)
     return NULL;
 
+  priv->last_focus = from;
+
   /* find the next widget to focus */
   if (direction == MX_FOCUS_DIRECTION_NEXT)
     {
@@ -575,15 +605,11 @@ mx_grid_move_focus (MxFocusable      *focusable,
 
               if (focused)
                 {
-                  priv->last_focus = (MxFocusable *)l->data;
                   update_adjustments (MX_GRID (focusable), focused);
                   return focused;
                 }
             }
         }
-
-      /* no next widgets to focus */
-      return NULL;
     }
   else if (direction == MX_FOCUS_DIRECTION_PREVIOUS)
     {
@@ -598,7 +624,6 @@ mx_grid_move_focus (MxFocusable      *focusable,
 
               if (focused)
                 {
-                  priv->last_focus = (MxFocusable *)l->data;
                   update_adjustments (MX_GRID (focusable), focused);
                   return focused;
                 }
@@ -606,6 +631,7 @@ mx_grid_move_focus (MxFocusable      *focusable,
         }
     }
 
+
   return NULL;
 }
 
@@ -688,11 +714,6 @@ mx_grid_init (MxGrid *self)
 static void
 mx_grid_dispose (GObject *object)
 {
-  MxGrid *self = (MxGrid *) object;
-  MxGridPrivate *priv;
-
-  priv = self->priv;
-
   /* Destroy all of the children. This will cause them to be removed
      from the container and unparented */
   clutter_container_foreach (CLUTTER_CONTAINER (object),
@@ -942,10 +963,6 @@ mx_grid_get_property (GObject    *object,
   MxAdjustment *adjustment;
   MxGrid *grid = MX_GRID (object);
 
-  MxGridPrivate *priv;
-
-  priv = MX_GRID_GET_PRIVATE (object);
-
   switch (prop_id)
     {
     case PROP_HOMOGENOUS_ROWS:
@@ -1245,7 +1262,7 @@ mx_grid_get_preferred_width (ClutterActor *self,
                              gfloat       *min_width_p,
                              gfloat       *natural_width_p)
 {
-  gfloat actual_width, actual_height;
+  gfloat actual_width, min_width;
   ClutterActorBox box;
 
   box.x1 = 0;
@@ -1254,10 +1271,10 @@ mx_grid_get_preferred_width (ClutterActor *self,
   box.y2 = for_height;
 
   mx_grid_do_allocate (self, &box, FALSE,
-                       TRUE, &actual_width, &actual_height);
+                       TRUE, &actual_width, NULL, &min_width, NULL);
 
   if (min_width_p)
-    *min_width_p = actual_width;
+    *min_width_p = min_width;
   if (natural_width_p)
     *natural_width_p = actual_width;
 }
@@ -1268,7 +1285,7 @@ mx_grid_get_preferred_height (ClutterActor *self,
                               gfloat       *min_height_p,
                               gfloat       *natural_height_p)
 {
-  gfloat actual_width, actual_height;
+  gfloat actual_height, min_height;
   ClutterActorBox box;
 
   box.x1 = 0;
@@ -1277,10 +1294,10 @@ mx_grid_get_preferred_height (ClutterActor *self,
   box.y2 = G_MAXFLOAT;
 
   mx_grid_do_allocate (self, &box, FALSE,
-                       TRUE, &actual_width, &actual_height);
+                       TRUE, NULL, &actual_height, NULL, &min_height);
 
   if (min_height_p)
-    *min_height_p = actual_height;
+    *min_height_p = min_height;
   if (natural_height_p)
     *natural_height_p = actual_height;
 }
@@ -1294,19 +1311,16 @@ compute_row_height (GList         *siblings,
   GList *l;
 
   gboolean homogenous_a;
-  gboolean homogenous_b;
   gfloat gap;
 
   if (priv->orientation == MX_ORIENTATION_VERTICAL)
     {
-      homogenous_b = priv->homogenous_columns;
       homogenous_a = priv->homogenous_rows;
       gap          = priv->row_spacing;
     }
   else
     {
       homogenous_a = priv->homogenous_columns;
-      homogenous_b = priv->homogenous_rows;
       gap          = priv->column_spacing;
     }
 
@@ -1357,19 +1371,16 @@ compute_row_start (GList         *siblings,
   GList *l;
 
   gboolean homogenous_a;
-  gboolean homogenous_b;
   gfloat gap;
 
   if (priv->orientation == MX_ORIENTATION_VERTICAL)
     {
-      homogenous_b = priv->homogenous_columns;
       homogenous_a = priv->homogenous_rows;
       gap          = priv->row_spacing;
     }
   else
     {
       homogenous_a = priv->homogenous_columns;
-      homogenous_b = priv->homogenous_rows;
       gap          = priv->column_spacing;
     }
 
@@ -1409,7 +1420,9 @@ mx_grid_do_allocate (ClutterActor          *self,
                      ClutterAllocationFlags flags,
                      gboolean               calculate_extents_only,
                      gfloat                *actual_width,
-                     gfloat                *actual_height)
+                     gfloat                *actual_height,
+                     gfloat                *min_width,
+                     gfloat                *min_height)
 {
   MxGrid *layout = (MxGrid *) self;
   MxGridPrivate *priv = layout->priv;
@@ -1435,6 +1448,12 @@ mx_grid_do_allocate (ClutterActor          *self,
   if (actual_height)
     *actual_height = 0;
 
+  if (min_width)
+    *min_width = 0;
+
+  if (min_height)
+    *min_height = 0;
+
   current_a = current_b = next_b = 0;
 
   GList *iter;
@@ -1501,13 +1520,15 @@ mx_grid_do_allocate (ClutterActor          *self,
       ClutterActor *child = iter->data;
       gfloat natural_a;
       gfloat natural_b;
+      gfloat min_a;
+      gfloat min_b;
 
       if (!CLUTTER_ACTOR_IS_VISIBLE (child))
         continue;
 
       /* each child will get as much space as they require */
       clutter_actor_get_preferred_size (CLUTTER_ACTOR (child),
-                                        NULL, NULL,
+                                        &min_a, &min_b,
                                         &natural_a, &natural_b);
 
       /* swap axes around if column is major */
@@ -1516,6 +1537,10 @@ mx_grid_do_allocate (ClutterActor          *self,
           gfloat temp = natural_a;
           natural_a = natural_b;
           natural_b = temp;
+
+          temp = min_a;
+          min_a = min_b;
+          min_b = temp;
         }
 
       /* if the child is overflowing, or the max-stride has been reached,
@@ -1545,6 +1570,7 @@ mx_grid_do_allocate (ClutterActor          *self,
       {
         gfloat row_height;
         ClutterActorBox child_box;
+        ClutterActorBox min_child_box;
 
         if (homogenous_b)
           {
@@ -1571,6 +1597,10 @@ mx_grid_do_allocate (ClutterActor          *self,
         child_box.y1 = current_b + (row_height-natural_b) * balign;
         child_box.y2 = child_box.y1 + natural_b;
 
+        min_child_box.x1 = 0;
+        min_child_box.y1 = 0;
+        min_child_box.x2 = min_a;
+        min_child_box.y2 = min_b;
 
         if (priv->orientation == MX_ORIENTATION_VERTICAL)
           {
@@ -1581,6 +1611,14 @@ mx_grid_do_allocate (ClutterActor          *self,
             temp = child_box.x2;
             child_box.x2 = child_box.y2;
             child_box.y2 = temp;
+
+            temp = min_child_box.x1;
+            min_child_box.x1 = min_child_box.y1;
+            min_child_box.y1 = temp;
+
+            temp = min_child_box.x2;
+            min_child_box.x2 = min_child_box.y2;
+            min_child_box.y2 = temp;
           }
 
         /* account for padding and pixel-align */
@@ -1602,6 +1640,18 @@ mx_grid_do_allocate (ClutterActor          *self,
         if (actual_height && (child_box.y2 + padding.bottom) > *actual_height)
           *actual_height = child_box.y2 + padding.bottom;
 
+        if (min_width &&
+            padding.left + min_child_box.x2 + padding.right > *min_width)
+          {
+            *min_width = padding.left + min_child_box.x2 + padding.right;
+          }
+
+        if (min_height &&
+            padding.top + min_child_box.y2 + padding.bottom > *min_height)
+          {
+            *min_height = padding.left + min_child_box.y2 + padding.right;
+          }
+
         if (homogenous_a)
           {
             current_a += priv->max_extent_a + agap;
@@ -1639,12 +1689,7 @@ mx_grid_allocate (ClutterActor          *self,
       gfloat height;
 
       /* get preferred height for this width */
-      mx_grid_do_allocate (self,
-                           box,
-                           flags,
-                           TRUE,
-                           NULL,
-                           &height);
+      mx_grid_do_allocate (self, box, flags, TRUE, NULL, &height, NULL, NULL);
       /* set our allocated height to be the preferred height, since we will be
        * scrolling
        */
@@ -1675,12 +1720,7 @@ mx_grid_allocate (ClutterActor          *self,
       gfloat width;
 
       /* get preferred width for this height */
-      mx_grid_do_allocate (self,
-                           box,
-                           flags,
-                           TRUE,
-                           &width,
-                           NULL);
+      mx_grid_do_allocate (self, box, flags, TRUE, &width, NULL, NULL, NULL);
       /* set our allocated height to be the preferred height, since we will be
        * scrolling
        */
@@ -1707,12 +1747,8 @@ mx_grid_allocate (ClutterActor          *self,
     }
 
 
-  mx_grid_do_allocate (self,
-                       &alloc_box,
-                       flags,
-                       FALSE,
-                       NULL,
-                       NULL);
+  mx_grid_do_allocate (self, &alloc_box, flags, FALSE, NULL, NULL,
+      NULL, NULL);
 }
 
 
