@@ -67,7 +67,7 @@ typedef struct
 {
   MxImage   *parent;
 
-  GMutex         *mutex;
+  GMutex          mutex;
   guint           complete  : 1;
   guint           cancelled : 1;
   guint           upscale   : 1;
@@ -162,8 +162,6 @@ mx_image_error_quark (void)
 static void
 mx_image_async_data_free (MxImageAsyncData *data)
 {
-  g_mutex_free (data->mutex);
-
   if (data->free_func)
     data->free_func (data->buffer);
 
@@ -187,7 +185,7 @@ mx_image_async_data_new (MxImage *parent)
   MxImageAsyncData *data = g_new0 (MxImageAsyncData, 1);
 
   data->parent = parent;
-  data->mutex = g_mutex_new ();
+  g_mutex_init (&data->mutex);
   data->width = -1;
   data->height = -1;
   data->upscale = parent->priv->upscale;
@@ -1146,7 +1144,7 @@ mx_image_set_from_data_internal (MxImage          *image,
 /**
  * mx_image_set_from_data:
  * @image: An #MxImage
- * @data: (array): Image data
+ * @data: Image data
  * @pixel_format: The #CoglPixelFormat of the buffer
  * @width: Width in pixels of image data.
  * @height: Height in pixels of image data
@@ -1287,8 +1285,8 @@ mx_image_load_complete_cb (gpointer task_data)
    * the mutex, and freeing a locked mutex results in undefined behaviour
    * (well, it crashes on Linux with an assert in pthreads...)
    */
-  g_mutex_lock (data->mutex);
-  g_mutex_unlock (data->mutex);
+  g_mutex_lock (&data->mutex);
+  g_mutex_unlock (&data->mutex);
 
   /* Reset the idle handler id so we don't try to remove it when we free
    * the data later on.
@@ -1500,7 +1498,7 @@ mx_image_async_cb (gpointer task_data,
   gboolean scaled;
   MxImageAsyncData *data = task_data;
 
-  g_mutex_lock (data->mutex);
+  g_mutex_lock (&data->mutex);
 
   /* Check if the task has been cancelled and bail out - leave to the main
    * thread to free the data.
@@ -1510,7 +1508,7 @@ mx_image_async_cb (gpointer task_data,
       data->idle_handler =
         clutter_threads_add_idle_full (G_PRIORITY_HIGH_IDLE,
                                        mx_image_load_complete_cb, data, NULL);
-      g_mutex_unlock (data->mutex);
+      g_mutex_unlock (&data->mutex);
 
       return;
     }
@@ -1535,7 +1533,7 @@ mx_image_async_cb (gpointer task_data,
     clutter_threads_add_idle_full (G_PRIORITY_HIGH_IDLE,
                                    mx_image_load_complete_cb, data, NULL);
 
-  g_mutex_unlock (data->mutex);
+  g_mutex_unlock (&data->mutex);
 }
 
 static gboolean
@@ -1597,7 +1595,7 @@ mx_image_set_async (MxImage         *image,
     {
       MxImageAsyncData *old_data = priv->async_load_data;
 
-      if (!g_mutex_trylock (old_data->mutex))
+      if (!g_mutex_trylock (&old_data->mutex))
         {
           /* The thread is busy, cancel it and start a new one */
           old_data->cancelled = TRUE;
@@ -1608,7 +1606,7 @@ mx_image_set_async (MxImage         *image,
             {
               /* The load finished, cancel the upload */
               old_data->cancelled = TRUE;
-              g_mutex_unlock (old_data->mutex);
+              g_mutex_unlock (&old_data->mutex);
             }
           else
             {
@@ -1621,7 +1619,7 @@ mx_image_set_async (MxImage         *image,
               old_data->width = width;
               old_data->height = height;
               old_data->cancelled = FALSE;
-              g_mutex_unlock (old_data->mutex);
+              g_mutex_unlock (&old_data->mutex);
 
               data = old_data;
             }
@@ -1872,10 +1870,9 @@ mx_image_set_from_cogl_texture (MxImage    *image,
 /**
  * mx_image_set_from_buffer:
  * @image: An #MxImage
- * @buffer: (array length=buffer_size) (transfer full): A buffer
- *   pointing to encoded image data
+ * @buffer: A buffer pointing to encoded image data
  * @buffer_size: The size of @buffer, in bytes
- * @buffer_free_func: (allow-none): A function to free @buffer, or %NULL
+ * @buffer_free_func: A function to free @buffer, or %NULL
  * @error: Return location for a #GError, or #NULL
  *
  * Set the image data from unencoded image data, stored in memory. In case of
@@ -1901,10 +1898,9 @@ mx_image_set_from_buffer (MxImage         *image,
 /**
  * mx_image_set_from_buffer_at_size:
  * @image: An #MxImage
- * @buffer: (array length=buffer_size) (transfer full): A buffer
- *   pointing to encoded image data
+ * @buffer: A buffer pointing to encoded image data
  * @buffer_size: The size of @buffer, in bytes
- * @buffer_free_func: (allow-none): A function to free @buffer, or %NULL
+ * @buffer_free_func: A function to free @buffer, or %NULL
  * @width: Width to scale the image to, or -1
  * @height: Height to scale the image to, or -1
  * @error: Return location for a #GError, or #NULL
