@@ -33,6 +33,8 @@
  *
  * #MxKineticScrollView also implements #MxScrollable itself, allowing it to
  * be embedded in an #MxScrollView to provide scroll-bars.
+ *
+ * Since: 1.2
  */
 
 #include "mx-kinetic-scroll-view.h"
@@ -68,6 +70,8 @@ struct _MxKineticScrollViewPrivate
   guint                  in_drag      : 1;
   guint                  hmoving      : 1;
   guint                  vmoving      : 1;
+  guint                  hclamping    : 1;
+  guint                  vclamping    : 1;
   guint32                button;
 
   /* Mouse motion event information */
@@ -81,6 +85,15 @@ struct _MxKineticScrollViewPrivate
   gdouble                decel_rate;
   gdouble                overshoot;
   gdouble                accumulated_delta;
+  gdouble                acceleration_factor;
+
+  MxScrollPolicy         scroll_policy;
+
+  guint                  clamp_duration;
+  gulong                 clamp_mode;
+  gboolean               clamp_to_center;
+
+  MxKineticScrollViewState state;
 };
 
 enum {
@@ -92,9 +105,19 @@ enum {
   PROP_VADJUST,
   PROP_BUTTON,
   PROP_USE_CAPTURED,
-  PROP_OVERSHOOT
+  PROP_OVERSHOOT,
+  PROP_SCROLL_POLICY,
+  PROP_ACCELERATION_FACTOR,
+  PROP_CLAMP_DURATION,
+  PROP_CLAMP_MODE,
+  PROP_STATE,
+  PROP_CLAMP_TO_CENTER,
 };
 
+static gboolean button_release (MxKineticScrollView *scroll,
+                                gint                 x,
+                                gint                 y);
+
 /* MxScrollableIface implementation */
 
 static void
@@ -186,6 +209,30 @@ mx_kinetic_scroll_view_get_property (GObject    *object,
       g_value_set_double (value, priv->overshoot);
       break;
 
+    case PROP_SCROLL_POLICY:
+      g_value_set_enum (value, priv->scroll_policy);
+      break;
+
+    case PROP_ACCELERATION_FACTOR :
+      g_value_set_double (value, priv->acceleration_factor);
+      break;
+
+    case PROP_CLAMP_DURATION :
+      g_value_set_uint (value, priv->clamp_duration);
+      break;
+
+    case PROP_CLAMP_MODE :
+      g_value_set_ulong (value, priv->clamp_mode);
+      break;
+
+    case PROP_STATE :
+      g_value_set_enum (value, priv->state);
+      break;
+
+    case PROP_CLAMP_TO_CENTER :
+      g_value_set_boolean (value, priv->clamp_to_center);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -243,6 +290,30 @@ mx_kinetic_scroll_view_set_property (GObject      *object,
       mx_kinetic_scroll_view_set_overshoot (self, g_value_get_double (value));
       break;
 
+    case PROP_SCROLL_POLICY:
+      mx_kinetic_scroll_view_set_scroll_policy (self, g_value_get_enum (value));
+      break;
+
+    case PROP_ACCELERATION_FACTOR :
+      mx_kinetic_scroll_view_set_acceleration_factor (self,
+          g_value_get_double (value));
+      break;
+
+    case PROP_CLAMP_DURATION :
+      mx_kinetic_scroll_view_set_clamp_duration (self,
+          g_value_get_uint (value));
+      break;
+
+    case PROP_CLAMP_MODE :
+      mx_kinetic_scroll_view_set_clamp_mode (self,
+          g_value_get_ulong (value));
+      break;
+
+    case PROP_CLAMP_TO_CENTER :
+      mx_kinetic_scroll_view_set_clamp_to_center (self,
+          g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -279,10 +350,12 @@ mx_kinetic_scroll_view_get_preferred_width (ClutterActor *actor,
                                             gfloat       *min_width_p,
                                             gfloat       *nat_width_p)
 {
+  MxKineticScrollViewPrivate *priv = MX_KINETIC_SCROLL_VIEW (actor)->priv;
+
   CLUTTER_ACTOR_CLASS (mx_kinetic_scroll_view_parent_class)->
     get_preferred_width (actor, for_height, NULL, nat_width_p);
 
-  if (min_width_p)
+  if (min_width_p && priv->scroll_policy != MX_SCROLL_POLICY_VERTICAL)
     {
       MxPadding padding;
 
@@ -297,10 +370,12 @@ mx_kinetic_scroll_view_get_preferred_height (ClutterActor *actor,
                                              gfloat       *min_height_p,
                                              gfloat       *nat_height_p)
 {
+  MxKineticScrollViewPrivate *priv = MX_KINETIC_SCROLL_VIEW (actor)->priv;
+
   CLUTTER_ACTOR_CLASS (mx_kinetic_scroll_view_parent_class)->
     get_preferred_height (actor, for_width, NULL, nat_height_p);
 
-  if (min_height_p)
+  if (min_height_p && priv->scroll_policy != MX_SCROLL_POLICY_HORIZONTAL)
     {
       MxPadding padding;
 
@@ -345,7 +420,7 @@ mx_kinetic_scroll_view_class_init (MxKineticScrollViewClass *klass)
   pspec = g_param_spec_double ("deceleration",
                                "Deceleration",
                                "Rate at which the view will decelerate in.",
-                               1.1, G_MAXDOUBLE, 1.1,
+                               1.01, G_MAXDOUBLE, 1.1,
                                MX_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_DECELERATION, pspec);
 
@@ -380,6 +455,50 @@ mx_kinetic_scroll_view_class_init (MxKineticScrollViewClass *klass)
                                MX_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_OVERSHOOT, pspec);
 
+  pspec = g_param_spec_enum ("scroll-policy",
+                             "Scroll Policy",
+                             "The scroll policy",
+                             MX_TYPE_SCROLL_POLICY,
+                             MX_SCROLL_POLICY_BOTH,
+                             MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_SCROLL_POLICY, pspec);
+
+  pspec = g_param_spec_double ("acceleration-factor",
+                               "Initial acceleration factor",
+                               "Factor applied to the initial acceleration.",
+                               0.0, G_MAXDOUBLE, 1.0,
+                               MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_ACCELERATION_FACTOR, pspec);
+
+  pspec = g_param_spec_uint ("clamp-duration",
+                             "Clamp duration",
+                             "Duration of the adjustment clamp animation.",
+                             0, G_MAXUINT, 250,
+                             MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_CLAMP_DURATION, pspec);
+
+  pspec = g_param_spec_ulong ("clamp-mode",
+                              "Clamp mode",
+                              "Animation mode to use for the clamp animation.",
+                              0, G_MAXULONG, CLUTTER_EASE_OUT_QUAD,
+                              MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_CLAMP_MODE, pspec);
+
+  pspec = g_param_spec_enum ("state",
+                             "State",
+                             "State of the scrolling",
+                             MX_TYPE_KINETIC_SCROLL_VIEW_STATE,
+                             MX_KINETIC_SCROLL_VIEW_STATE_IDLE,
+                             MX_PARAM_READABLE);
+  g_object_class_install_property (object_class, PROP_STATE, pspec);
+
+  pspec = g_param_spec_boolean ("clamp-to-center",
+                                "Clamp to center",
+                                "Whether to clamp to step increments based on the center of the page.",
+                                FALSE,
+                                MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_CLAMP_TO_CENTER, pspec);
+
   /* MxScrollable properties */
   g_object_class_override_property (object_class,
                                     PROP_HADJUST,
@@ -390,20 +509,59 @@ mx_kinetic_scroll_view_class_init (MxKineticScrollViewClass *klass)
                                     "vertical-adjustment");
 }
 
+static void
+set_state (MxKineticScrollView *scroll, MxKineticScrollViewState state)
+{
+  MxKineticScrollViewPrivate *priv = scroll->priv;
+
+  priv->state = state;
+  g_object_notify (G_OBJECT (scroll), "state");
+}
+
 static gboolean
-motion_event_cb (ClutterActor        *stage,
+motion_event_cb (ClutterActor        *actor,
                  ClutterMotionEvent  *event,
                  MxKineticScrollView *scroll)
 {
   gfloat x, y;
 
   MxKineticScrollViewPrivate *priv = scroll->priv;
-  ClutterActor *actor = CLUTTER_ACTOR (scroll);
 
   if (event->type != CLUTTER_MOTION)
     return FALSE;
 
-  if (clutter_actor_transform_stage_point (actor,
+  /* For various reasons, it's possible we may not get the button release
+   * event - in this case, check here if the button is still down and
+   * handle it manually.
+   *
+   * We need to do this, or we can end up just stealing events forever.
+   */
+  switch (priv->button)
+    {
+    default:
+    case 1:
+      if (!(event->modifier_state & CLUTTER_BUTTON1_MASK))
+        return button_release (scroll, event->x, event->y);
+      break;
+    case 2:
+      if (!(event->modifier_state & CLUTTER_BUTTON2_MASK))
+        return button_release (scroll, event->x, event->y);
+      break;
+    case 3:
+      if (!(event->modifier_state & CLUTTER_BUTTON3_MASK))
+        return button_release (scroll, event->x, event->y);
+      break;
+    case 4:
+      if (!(event->modifier_state & CLUTTER_BUTTON4_MASK))
+        return button_release (scroll, event->x, event->y);
+      break;
+    case 5:
+      if (!(event->modifier_state & CLUTTER_BUTTON5_MASK))
+        return button_release (scroll, event->x, event->y);
+      break;
+    }
+
+  if (clutter_actor_transform_stage_point (CLUTTER_ACTOR (scroll),
                                            event->x,
                                            event->y,
                                            &x, &y))
@@ -415,6 +573,7 @@ motion_event_cb (ClutterActor        *stage,
       if (!priv->in_drag)
         {
           guint threshold;
+          gboolean threshold_passed;
           MxSettings *settings = mx_settings_get_default ();
 
           g_object_get (G_OBJECT (settings),
@@ -422,11 +581,34 @@ motion_event_cb (ClutterActor        *stage,
           motion = &g_array_index (priv->motion_buffer,
                                    MxKineticScrollViewMotion, 0);
 
-          if ((ABS (motion->x - x) >= threshold) ||
-              (ABS (motion->y - y) >= threshold))
+          if ((ABS (motion->y - y) >= threshold) &&
+              (priv->scroll_policy == MX_SCROLL_POLICY_VERTICAL ||
+               priv->scroll_policy == MX_SCROLL_POLICY_BOTH))
+           threshold_passed = TRUE;
+          else if ((ABS (motion->x - x) >= threshold) &&
+              (priv->scroll_policy == MX_SCROLL_POLICY_HORIZONTAL ||
+               priv->scroll_policy == MX_SCROLL_POLICY_BOTH))
+           threshold_passed = TRUE;
+          else
+           threshold_passed = FALSE;
+
+          if (threshold_passed)
             {
-              clutter_set_motion_events_enabled (TRUE);
+              clutter_set_motion_events_enabled (FALSE);
               priv->in_drag = TRUE;
+
+              set_state (scroll, MX_KINETIC_SCROLL_VIEW_STATE_PANNING);
+
+              if (!priv->use_captured)
+                {
+                  g_signal_handlers_disconnect_by_func (actor,
+                                                        motion_event_cb,
+                                                        scroll);
+                  g_signal_connect (clutter_actor_get_stage (actor),
+                                    "captured-event",
+                                    G_CALLBACK (motion_event_cb),
+                                    scroll);
+                }
             }
           else
             return FALSE;
@@ -475,44 +657,92 @@ motion_event_cb (ClutterActor        *stage,
   return TRUE;
 }
 
+static void
+interpolation_completed_cb (MxAdjustment *adj, MxKineticScrollView *scroll)
+{
+  MxKineticScrollViewPrivate *priv = scroll->priv;
+  MxAdjustment *hadj, *vadj;
+
+  g_signal_handlers_disconnect_by_func (adj, interpolation_completed_cb,
+                                        scroll);
+
+  mx_scrollable_get_adjustments (MX_SCROLLABLE (scroll), &hadj, &vadj);
+  if (adj == hadj)
+    priv->hclamping = FALSE;
+  else
+    priv->vclamping = FALSE;
+
+  if (!priv->hclamping && !priv->vclamping && \
+      priv->state == MX_KINETIC_SCROLL_VIEW_STATE_CLAMPING)
+    set_state (scroll, MX_KINETIC_SCROLL_VIEW_STATE_IDLE);
+}
+
+static void
+clamp_adjustment (MxKineticScrollView *scroll,
+                  MxAdjustment        *adj,
+                  guint                duration)
+{
+  MxKineticScrollViewPrivate *priv = scroll->priv;
+  gdouble d, value, lower, upper, step_increment, page_size;
+
+  /* Snap to the nearest step increment on hadjustment */
+  mx_adjustment_get_values (adj, &value, &lower, &upper,
+                            &step_increment, NULL, &page_size);
+
+  d = (rint ((value - lower) / step_increment) *
+      step_increment) + lower;
+
+  if (priv->clamp_to_center)
+    {
+      gdouble offset = page_size / 2;
+      offset -= step_increment / 2;
+      offset = step_increment - (int) offset % (int) step_increment;
+
+      if (offset > step_increment / 2)
+        offset = - (step_increment - offset);
+
+      d += offset;
+    }
+
+  if (mx_adjustment_get_clamp_value (adj))
+    d = CLAMP (d, lower, upper - page_size);
+
+  g_signal_connect (adj, "interpolation-completed",
+                    G_CALLBACK (interpolation_completed_cb), scroll);
+  mx_adjustment_interpolate (adj, d, duration, priv->clamp_mode);
+}
+
 static void
 clamp_adjustments (MxKineticScrollView *scroll,
                    guint                duration,
                    gboolean             horizontal,
                    gboolean             vertical)
 {
+  MxKineticScrollViewPrivate *priv = scroll->priv;
   ClutterActor *child = mx_bin_get_child (MX_BIN (scroll));
 
   if (child)
     {
-      gdouble d, value, lower, upper, step_increment, page_size;
       MxAdjustment *hadj, *vadj;
 
-      mx_scrollable_get_adjustments (MX_SCROLLABLE (child),
-                                     &hadj, &vadj);
+      mx_scrollable_get_adjustments (MX_SCROLLABLE (child), &hadj, &vadj);
 
       if (horizontal && hadj)
         {
-          /* Snap to the nearest step increment on hadjustment */
-          mx_adjustment_get_values (hadj, &value, &lower, &upper,
-                                    &step_increment, NULL, &page_size);
-          d = (rint ((value - lower) / step_increment) *
-              step_increment) + lower;
-          d = CLAMP (d, lower, upper - page_size);
-          mx_adjustment_interpolate (hadj, d, duration, CLUTTER_EASE_OUT_QUAD);
+          priv->hclamping = TRUE;
+          clamp_adjustment (scroll, hadj, duration);
         }
 
       if (vertical && vadj)
         {
-          /* Snap to the nearest step increment on vadjustment */
-          mx_adjustment_get_values (vadj, &value, &lower, &upper,
-                                    &step_increment, NULL, &page_size);
-          d = (rint ((value - lower) / step_increment) *
-              step_increment) + lower;
-          d = CLAMP (d, lower, upper - page_size);
-          mx_adjustment_interpolate (vadj, d, duration, CLUTTER_EASE_OUT_QUAD);
-        }
+          priv->vclamping = TRUE;
+          clamp_adjustment (scroll, vadj, duration);
+        };
     }
+
+
+  if (priv->hclamping && priv->vclamping)
+    set_state (scroll, MX_KINETIC_SCROLL_VIEW_STATE_CLAMPING);
 }
 
 static void
@@ -520,9 +750,10 @@ deceleration_completed_cb (ClutterTimeline     *timeline,
                            MxKineticScrollView *scroll)
 {
   MxKineticScrollViewPrivate *priv = scroll->priv;
+  guint duration;
 
-  clamp_adjustments (scroll, (priv->overshoot > 0.0) ? 250 : 10,
-                     priv->hmoving, priv->vmoving);
+  duration = (priv->overshoot > 0.0) ? priv->clamp_duration : 10;
+  clamp_adjustments (scroll, duration, priv->hmoving, priv->vmoving);
 
   g_object_unref (timeline);
   priv->deceleration_timeline = NULL;
@@ -556,7 +787,7 @@ deceleration_new_frame_cb (ClutterTimeline     *timeline,
 
           if (hadjust)
             {
-              if (ABS (priv->dx) > 0.1)
+              if (ABS (priv->dx) > 5)
                 {
                   hvalue = priv->dx + mx_adjustment_get_value (hadjust);
                   mx_adjustment_set_value (hadjust, hvalue);
@@ -575,16 +806,19 @@ deceleration_new_frame_cb (ClutterTimeline     *timeline,
                 }
               else if (priv->hmoving)
                 {
+                  guint duration;
+
                   priv->hmoving = FALSE;
-                  clamp_adjustments (scroll,
-                                     (priv->overshoot > 0.0) ? 250 : 10,
-                                     TRUE, FALSE);
+
+                  duration = (priv->overshoot > 0.0) ?
+                                priv->clamp_duration : 10;
+                  clamp_adjustments (scroll, duration, TRUE, FALSE);
                 }
             }
 
           if (vadjust)
             {
-              if (ABS (priv->dy) > 0.1)
+              if (ABS (priv->dy) > 5)
                 {
                   vvalue = priv->dy + mx_adjustment_get_value (vadjust);
                   mx_adjustment_set_value (vadjust, vvalue);
@@ -603,10 +837,13 @@ deceleration_new_frame_cb (ClutterTimeline     *timeline,
                 }
               else if (priv->vmoving)
                 {
+                  guint duration;
+
                   priv->vmoving = FALSE;
-                  clamp_adjustments (scroll,
-                                     (priv->overshoot > 0.0) ? 250 : 10,
-                                     FALSE, TRUE);
+
+                  duration = (priv->overshoot > 0.0) ?
+                                priv->clamp_duration : 10;
+                  clamp_adjustments (scroll, duration, FALSE, TRUE);
                 }
             }
 
@@ -627,14 +864,28 @@ button_release_event_cb (ClutterActor        *stage,
                          MxKineticScrollView *scroll)
 {
   MxKineticScrollViewPrivate *priv = scroll->priv;
-  ClutterActor *actor = CLUTTER_ACTOR (scroll);
-  ClutterActor *child = mx_bin_get_child (MX_BIN (scroll));
-  gboolean decelerating = FALSE;
 
   if ((event->type != CLUTTER_BUTTON_RELEASE) ||
       (event->button != priv->button))
     return FALSE;
 
+  return button_release (scroll, event->x, event->y);
+}
+
+static gboolean
+button_release (MxKineticScrollView *scroll,
+                gint                 x_pos,
+                gint                 y_pos)
+{
+  ClutterActor *actor = CLUTTER_ACTOR (scroll);
+  ClutterActor *stage = clutter_actor_get_stage (actor);
+  ClutterActor *child = mx_bin_get_child (MX_BIN (scroll));
+  MxKineticScrollViewPrivate *priv = scroll->priv;
+  gboolean decelerating = FALSE;
+
+  g_signal_handlers_disconnect_by_func (scroll,
+                                        motion_event_cb,
+                                        scroll);
   g_signal_handlers_disconnect_by_func (stage,
                                         motion_event_cb,
                                         scroll);
@@ -651,7 +902,7 @@ button_release_event_cb (ClutterActor        *stage,
     {
       gfloat event_x, event_y;
 
-      if (clutter_actor_transform_stage_point (actor, event->x, event->y,
+      if (clutter_actor_transform_stage_point (actor, x_pos, y_pos,
                                                &event_x, &event_y))
         {
           gdouble value, lower, upper, step_increment, page_size,
@@ -699,8 +950,8 @@ button_release_event_cb (ClutterActor        *stage,
           frac = (time_diff/1000.0) / (1000.0/60.0);
 
           /* See how many units to move in 1/60th of a second */
-          priv->dx = (x_origin - event_x) / frac;
-          priv->dy = (y_origin - event_y) / frac;
+          priv->dx = (x_origin - event_x) / frac * priv->acceleration_factor;
+          priv->dy = (y_origin - event_y) / frac * priv->acceleration_factor;
 
           /* If the delta is too low for the equations to work,
            * bump the values up a bit.
@@ -812,6 +1063,7 @@ button_release_event_cb (ClutterActor        *stage,
               priv->hmoving = priv->vmoving = TRUE;
               clutter_timeline_start (priv->deceleration_timeline);
               decelerating = TRUE;
+              set_state (scroll, MX_KINETIC_SCROLL_VIEW_STATE_SCROLLING);
             }
         }
     }
@@ -820,7 +1072,7 @@ button_release_event_cb (ClutterActor        *stage,
   priv->last_motion = 0;
 
   if (!decelerating)
-    clamp_adjustments (scroll, 250, TRUE, TRUE);
+    clamp_adjustments (scroll, priv->clamp_duration, TRUE, TRUE);
 
   return TRUE;
 }
@@ -857,12 +1109,21 @@ button_press_event_cb (ClutterActor        *actor,
               clutter_timeline_stop (priv->deceleration_timeline);
               g_object_unref (priv->deceleration_timeline);
               priv->deceleration_timeline = NULL;
+
+              clamp_adjustments (scroll, priv->clamp_duration, priv->hmoving,
+                                 priv->vmoving);
             }
 
-          g_signal_connect (stage,
-                            "captured-event",
-                            G_CALLBACK (motion_event_cb),
-                            scroll);
+          if (priv->use_captured)
+            g_signal_connect (stage,
+                              "captured-event",
+                              G_CALLBACK (motion_event_cb),
+                              scroll);
+          else
+            g_signal_connect (scroll,
+                              "motion-event",
+                              G_CALLBACK (motion_event_cb),
+                              scroll);
           g_signal_connect (stage,
                             "captured-event",
                             G_CALLBACK (button_release_event_cb),
@@ -931,6 +1192,10 @@ mx_kinetic_scroll_view_init (MxKineticScrollView *self)
   g_array_set_size (priv->motion_buffer, 3);
   priv->decel_rate = 1.1f;
   priv->button = 1;
+  priv->scroll_policy = MX_SCROLL_POLICY_BOTH;
+  priv->acceleration_factor = 1.0;
+  priv->clamp_duration = 250;
+  priv->clamp_mode = CLUTTER_EASE_OUT_QUAD;
 
   clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
   g_signal_connect (self, "button-press-event",
@@ -949,6 +1214,8 @@ mx_kinetic_scroll_view_init (MxKineticScrollView *self)
  * Creates a new #MxKineticScrollView.
  *
  * Returns: a newly allocated #MxKineticScrollView
+ *
+ * Since: 1.2
  */
 ClutterActor *
 mx_kinetic_scroll_view_new ()
@@ -961,6 +1228,8 @@ mx_kinetic_scroll_view_new ()
  * @scroll: A #MxKineticScrollView
  *
  * Stops any current movement due to kinetic scrolling.
+ *
+ * Since: 1.2
  */
 void
 mx_kinetic_scroll_view_stop (MxKineticScrollView *scroll)
@@ -987,6 +1256,8 @@ mx_kinetic_scroll_view_stop (MxKineticScrollView *scroll)
  * Sets the deceleration rate when a drag is finished on the kinetic
  * scroll-view. This is the value that the momentum is divided by
  * every 60th of a second.
+ *
+ * Since: 1.2
  */
 void
 mx_kinetic_scroll_view_set_deceleration (MxKineticScrollView *scroll,
@@ -995,7 +1266,7 @@ mx_kinetic_scroll_view_set_deceleration (MxKineticScrollView *scroll,
   MxKineticScrollViewPrivate *priv;
 
   g_return_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll));
-  g_return_if_fail (rate >= 1.1);
+  g_return_if_fail (rate >= 1.01);
 
   priv = scroll->priv;
 
@@ -1013,11 +1284,13 @@ mx_kinetic_scroll_view_set_deceleration (MxKineticScrollView *scroll,
  * Retrieves the deceleration rate of the kinetic scroll-view.
  *
  * Returns: The deceleration rate of the kinetic scroll-view
+ *
+ * Since: 1.2
  */
 gdouble
 mx_kinetic_scroll_view_get_deceleration (MxKineticScrollView *scroll)
 {
-  g_return_val_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll), 0.0);
+  g_return_val_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll), 1.01);
   return scroll->priv->decel_rate;
 }
 
@@ -1054,6 +1327,8 @@ mx_kinetic_scroll_view_get_buffer_size (MxKineticScrollView *scroll)
  *
  * Sets the mouse button number used to initiate drag events on the kinetic
  * scroll-view.
+ *
+ * Since: 1.2
  */
 void
 mx_kinetic_scroll_view_set_mouse_button (MxKineticScrollView *scroll,
@@ -1080,6 +1355,8 @@ mx_kinetic_scroll_view_set_mouse_button (MxKineticScrollView *scroll,
  *
  * Returns: The mouse button number used to initiate drag events on the
  *          kinetic scroll-view
+ *
+ * Since: 1.2
  */
 guint32
 mx_kinetic_scroll_view_get_mouse_button (MxKineticScrollView *scroll)
@@ -1096,6 +1373,8 @@ mx_kinetic_scroll_view_get_mouse_button (MxKineticScrollView *scroll)
  * Sets whether to use captured events to initiate drag events. This can be
  * used to block events that would initiate scrolling from reaching the child
  * actor.
+ *
+ * Since: 1.2
  */
 void
 mx_kinetic_scroll_view_set_use_captured (MxKineticScrollView *scroll,
@@ -1130,6 +1409,8 @@ mx_kinetic_scroll_view_set_use_captured (MxKineticScrollView *scroll,
  * Gets the #MxKineticScrollView:use-captured property.
  *
  * Returns: %TRUE if captured-events should be used to initiate scrolling
+ *
+ * Since: 1.2
  */
 gboolean
 mx_kinetic_scroll_view_get_use_captured (MxKineticScrollView *scroll)
@@ -1150,6 +1431,8 @@ mx_kinetic_scroll_view_get_use_captured (MxKineticScrollView *scroll)
  * adjustments.
  *
  * See mx_kinetic_scroll_view_set_deceleration()
+ *
+ * Since: 1.2
  */
 void
 mx_kinetic_scroll_view_set_overshoot (MxKineticScrollView *scroll,
@@ -1173,6 +1456,8 @@ mx_kinetic_scroll_view_set_overshoot (MxKineticScrollView *scroll,
  *
  * Retrieves the deceleration rate multiplier used when the scroll-view is
  * scrolling beyond its boundaries.
+ *
+ * Since: 1.2
  */
 gdouble
 mx_kinetic_scroll_view_get_overshoot (MxKineticScrollView *scroll)
@@ -1180,3 +1465,218 @@ mx_kinetic_scroll_view_get_overshoot (MxKineticScrollView *scroll)
   g_return_val_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll), 0.0);
   return scroll->priv->overshoot;
 }
+
+/**
+ * mx_kinetic_scroll_view_set_scroll_policy:
+ * @scroll: A #MxKineticScrollView
+ * @policy: A #MxScrollPolicy
+ *
+ * Sets the scrolling policy for the kinetic scroll-view. This controls the
+ * possible axes of movement, and can affect the minimum size of the widget.
+ */
+void
+mx_kinetic_scroll_view_set_scroll_policy (MxKineticScrollView  *scroll,
+                                          MxScrollPolicy policy)
+{
+  MxKineticScrollViewPrivate *priv;
+
+  g_return_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll));
+
+  priv = scroll->priv;
+
+  if (priv->scroll_policy != policy)
+    {
+      priv->scroll_policy = policy;
+
+      g_object_notify (G_OBJECT (scroll), "scroll-policy");
+    }
+}
+
+/**
+ * mx_kinetic_scroll_view_get_scroll_policy:
+ * @scroll: A #MxKineticScrollView
+ *
+ * Retrieves the scrolling policy of the kinetic scroll-view.
+ *
+ * Returns: A #MxScrollPolicy
+ */
+MxScrollPolicy
+mx_kinetic_scroll_view_get_scroll_policy (MxKineticScrollView *scroll)
+{
+  g_return_val_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll), 0);
+
+  return scroll->priv->scroll_policy;
+}
+
+/**
+ * mx_kinetic_scroll_view_set_acceleration_factor:
+ * @scroll: A #MxKineticScrollView
+ * @acceleration_factor: The acceleration factor
+ *
+ * Factor applied to the initial momentum.
+ *
+ * Since: 1.4
+ */
+void
+mx_kinetic_scroll_view_set_acceleration_factor (MxKineticScrollView *scroll,
+                                                gdouble              acceleration_factor)
+{
+  MxKineticScrollViewPrivate *priv;
+
+  g_return_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll));
+  g_return_if_fail (acceleration_factor >= 0.0);
+
+  priv = scroll->priv;
+
+  if (priv->acceleration_factor != acceleration_factor)
+    {
+      priv->acceleration_factor = acceleration_factor;
+      g_object_notify (G_OBJECT (scroll), "acceleration-factor");
+    }
+}
+
+/**
+ * mx_kinetic_scroll_view_get_acceleration_factor:
+ * @scroll: A #MxKineticScrollView
+ *
+ * Retrieves the initial acceleration factor of the kinetic scroll-view.
+ *
+ * Returns: The initial acceleration factor of the kinetic scroll-view
+ *
+ * Since: 1.4
+ */
+gdouble
+mx_kinetic_scroll_view_get_acceleration_factor (MxKineticScrollView *scroll)
+{
+  g_return_val_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll), 1.0);
+  return scroll->priv->acceleration_factor;
+}
+
+/**
+ * mx_kinetic_scroll_view_set_clamp_duration:
+ * @scroll: A #MxKineticScrollView
+ * @clamp_duration: Clamp duration
+ *
+ * Duration of the adjustment clamp animation.
+ *
+ * Since: 1.4
+ */
+void
+mx_kinetic_scroll_view_set_clamp_duration (MxKineticScrollView *scroll,
+                                           guint                clamp_duration)
+{
+  MxKineticScrollViewPrivate *priv;
+
+  g_return_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll));
+
+  priv = scroll->priv;
+
+  if (priv->clamp_duration != clamp_duration)
+    {
+      priv->clamp_duration = clamp_duration;
+      g_object_notify (G_OBJECT (scroll), "clamp-duration");
+    }
+}
+
+/**
+ * mx_kinetic_scroll_view_get_clamp_duration:
+ * @scroll: A #MxKineticScrollView
+ *
+ * Retrieves the duration of the adjustment clamp animation.
+ *
+ * Returns: Clamp duration
+ *
+ * Since: 1.4
+ */
+guint
+mx_kinetic_scroll_view_get_clamp_duration (MxKineticScrollView *scroll)
+{
+  g_return_val_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll), 250);
+  return scroll->priv->clamp_duration;
+}
+
+/**
+ * mx_kinetic_scroll_view_set_clamp_mode:
+ * @scroll: A #MxKineticScrollView
+ * @clamp_mode: Clamp mode
+ *
+ * Animation mode to use for the adjustment clamp animation.
+ *
+ * Since: 1.4
+ */
+void
+mx_kinetic_scroll_view_set_clamp_mode (MxKineticScrollView *scroll,
+                                       gulong               clamp_mode)
+{
+  MxKineticScrollViewPrivate *priv;
+
+  g_return_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll));
+
+  priv = scroll->priv;
+
+  if (priv->clamp_mode != clamp_mode)
+    {
+      priv->clamp_mode = clamp_mode;
+      g_object_notify (G_OBJECT (scroll), "clamp-mode");
+    }
+}
+
+/**
+ * mx_kinetic_scroll_view_get_clamp_mode:
+ * @scroll: A #MxKineticScrollView
+ *
+ * Retrieves the animation mode to use for the adjustment clamp animation.
+ *
+ * Returns: Clamp mode
+ *
+ * Since: 1.4
+ */
+gulong
+mx_kinetic_scroll_view_get_clamp_mode (MxKineticScrollView *scroll)
+{
+  g_return_val_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll), CLUTTER_EASE_OUT_QUAD);
+  return scroll->priv->clamp_mode;
+}
+
+/**
+ * mx_kinetic_scroll_view_set_clamp_to_center:
+ * @scroll: A #MxKineticScrollView
+ * @clamp_to_center: Clamp to center
+ *
+ * Set whether to clamp to step increments based on the center of the page.
+ *
+ * Since: 1.4
+ */
+void
+mx_kinetic_scroll_view_set_clamp_to_center (MxKineticScrollView *scroll,
+                                            gboolean             clamp_to_center)
+{
+  MxKineticScrollViewPrivate *priv;
+
+  g_return_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll));
+
+  priv = scroll->priv;
+
+  if (priv->clamp_to_center != clamp_to_center)
+    {
+      priv->clamp_to_center = clamp_to_center;
+      g_object_notify (G_OBJECT (scroll), "clamp-to-center");
+    }
+}
+
+/**
+ * mx_kinetic_scroll_view_get_clamp_to_center:
+ * @scroll: A #MxKineticScrollView
+ *
+ * Retrieves whether to clamp to step increments based on the center of the page.
+ *
+ * Returns: Clamp to center
+ *
+ * Since: 1.4
+ */
+gboolean
+mx_kinetic_scroll_view_get_clamp_to_center (MxKineticScrollView *scroll)
+{
+  g_return_val_if_fail (MX_IS_KINETIC_SCROLL_VIEW (scroll), FALSE);
+  return scroll->priv->clamp_to_center;
+}
