@@ -28,7 +28,7 @@
  *
  * #MxListView is a box layout container driven by a #ClutterModel. Children
  * are created for each row in the model, either by creating actors from the
- * supplied #ClutterActor derived type, or from a #MxItemFactor.
+ * supplied #ClutterActor derived type, or from a #MxItemFactory.
  *
  * Data is set on the children by mapping columns in the model to object
  * properties on the children.
@@ -72,6 +72,8 @@ struct _MxListViewPrivate
   gulong         row_changed;
   gulong         row_removed;
   gulong         sort_changed;
+
+  guint          is_frozen : 1;
 };
 
 /* gobject implementations */
@@ -141,6 +143,13 @@ mx_list_view_dispose (GObject *object)
   G_OBJECT_CLASS (mx_list_view_parent_class)->dispose (object);
 }
 
+static void
+free_attribute (AttributeData *data)
+{
+  g_free (data->name);
+  g_free (data);
+}
+
 static void
 mx_list_view_finalize (GObject *object)
 {
@@ -148,7 +157,7 @@ mx_list_view_finalize (GObject *object)
 
   if (priv->attributes)
     {
-      g_slist_foreach (priv->attributes, (GFunc) g_free, NULL);
+      g_slist_foreach (priv->attributes, (GFunc) free_attribute, NULL);
       g_slist_free (priv->attributes);
       priv->attributes = NULL;
     }
@@ -221,6 +230,9 @@ model_changed_cb (ClutterModel *model,
   if (!priv->item_type && !priv->factory)
     return;
 
+  if (priv->is_frozen)
+    return;
+
   if (priv->item_type)
     {
       /* check the item-type is an descendant of ClutterActor */
@@ -327,6 +339,9 @@ row_removed_cb (ClutterModel     *model,
   GList *l;
   ClutterActor *child;
 
+  if (list_view->priv->is_frozen)
+    return;
+
   children = clutter_container_get_children (CLUTTER_CONTAINER (list_view));
   l = g_list_nth (children, clutter_model_iter_get_row (iter));
   child = (ClutterActor *) l->data;
@@ -393,7 +408,7 @@ mx_list_view_set_item_type (MxListView *list_view,
  *
  * Get the model currently used by the #MxListView
  *
- * Returns: the current #ClutterModel
+ * Returns: (transfer none): the current #ClutterModel
  */
 ClutterModel*
 mx_list_view_get_model (MxListView *list_view)
@@ -517,7 +532,7 @@ mx_list_view_add_attribute (MxListView  *list_view,
  * @list_view: An #MxListView
  *
  * Freeze the view. This means that the view will not act on changes to the
- * model until it is thawed. Call mx_list_view_thaw() to thaw the view
+ * model until it is thawed. Call #mx_list_view_thaw to thaw the view.
  */
 void
 mx_list_view_freeze (MxListView *list_view)
@@ -528,17 +543,7 @@ mx_list_view_freeze (MxListView *list_view)
 
   priv = list_view->priv;
 
-  g_signal_handlers_block_by_func (priv->model,
-                                   model_changed_cb,
-                                   list_view);
-
-  g_signal_handlers_block_by_func (priv->model,
-                                   row_removed_cb,
-                                   list_view);
-
-  g_signal_handlers_block_by_func (priv->model,
-                                   row_changed_cb,
-                                   list_view);
+  priv->is_frozen = TRUE;
 }
 
 /**
@@ -557,17 +562,7 @@ mx_list_view_thaw (MxListView *list_view)
 
   priv = list_view->priv;
 
-  g_signal_handlers_unblock_by_func (priv->model,
-                                     model_changed_cb,
-                                     list_view);
-
-  g_signal_handlers_unblock_by_func (priv->model,
-                                     row_removed_cb,
-                                     list_view);
-
-  g_signal_handlers_unblock_by_func (priv->model,
-                                     row_changed_cb,
-                                     list_view);
+  priv->is_frozen = FALSE;
 
   /* Repopulate */
   model_changed_cb (priv->model, list_view);
@@ -612,7 +607,7 @@ mx_list_view_set_factory (MxListView    *list_view,
  *
  * Gets the #MxItemFactory used for creating new list items.
  *
- * Returns: A #MxItemFactory.
+ * Returns: (transfer none): A #MxItemFactory.
  */
 MxItemFactory *
 mx_list_view_get_factory (MxListView *list_view)
