@@ -28,16 +28,16 @@
 
 #include "mx-menu.h"
 #include "mx-label.h"
-#include "mx-box-layout.h"
-#include "mx-icon.h"
-#include "mx-icon-theme.h"
+#include "mx-button.h"
 #include "mx-stylable.h"
+#include "mx-focusable.h"
+#include "mx-focus-manager.h"
 
-static void mx_stylable_iface_init (MxStylableIface *iface);
+static void mx_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxMenu, mx_menu, MX_TYPE_FLOATING_WIDGET,
-                         G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
-                                                mx_stylable_iface_init))
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                mx_focusable_iface_init))
 
 #define MENU_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MX_TYPE_MENU, MxMenuPrivate))
@@ -50,12 +50,21 @@ typedef struct
 
 struct _MxMenuPrivate
 {
-  gint     spacing;
   GArray  *children;
   gboolean transition_out;
 
   ClutterActor *stage;
   gulong captured_event_handler;
+
+  gulong internal_focus_push : 1;
+
+  gulong scrolling_mode : 1;
+  gint id_offset;
+  gint last_shown_id;
+  ClutterActor *up_button;
+  ClutterActor *down_button;
+  gulong up_source;
+  gulong down_source;
 };
 
 enum
@@ -71,25 +80,142 @@ static gboolean mx_menu_captured_event_handler (ClutterActor *actor,
                                                 ClutterEvent *event,
                                                 ClutterActor *menu);
 
-static void
-mx_stylable_iface_init (MxStylableIface *iface)
+/* MxFocusable Interface */
+
+static MxFocusable*
+mx_menu_move_focus (MxFocusable      *focusable,
+                    MxFocusDirection  direction,
+                    MxFocusable      *from)
 {
-  static gboolean is_initialised = FALSE;
+  MxMenuPrivate *priv = MX_MENU (focusable)->priv;
+  MxFocusable *result;
+  MxMenuChild *child;
+  gint i, start;
 
-  if (!is_initialised)
+  /* find the current focused child */
+  for (i = 0; i < priv->children->len; i++)
     {
-      GParamSpec *pspec;
-
-      is_initialised = TRUE;
-
-      pspec = g_param_spec_int ("x-mx-spacing",
-                                "Spacing",
-                                "Spacing to use between elements inside the "
-                                "menu.",
-                                0, G_MAXINT, 8,
-                                G_PARAM_READWRITE);
-      mx_stylable_iface_install_property (iface, MX_TYPE_MENU, pspec);
+      child = &g_array_index (priv->children, MxMenuChild, i);
+      if ((MxFocusable*) child->box == from)
+        break;
+      else
+        child = NULL;
     }
+
+  if (!child)
+    return NULL;
+
+  start = i;
+
+  switch (direction)
+    {
+    case MX_FOCUS_DIRECTION_UP:
+      if (i == 0)
+        {
+          i = priv->children->len - 1;
+          gint nb_elts = priv->last_shown_id - priv->id_offset;
+          priv->id_offset = i - nb_elts;
+          clutter_actor_queue_redraw (CLUTTER_ACTOR(focusable));
+        }
+      else
+        {
+          i--;
+          if (i < priv->id_offset)
+            {
+              priv->id_offset--;
+              clutter_actor_queue_redraw (CLUTTER_ACTOR(focusable));
+            }
+        }
+
+      while (i >= 0)
+        {
+          if (i == start)
+            break;
+
+          child = &g_array_index (priv->children, MxMenuChild, i);
+
+          result = mx_focusable_accept_focus (MX_FOCUSABLE (child->box), 0);
+
+          if (result)
+            return result;
+
+          /* loop */
+          if (i == 0)
+            i = priv->children->len;
+
+          i--;
+        }
+
+
+    case MX_FOCUS_DIRECTION_DOWN:
+      if (i == priv->children->len - 1)
+        {
+          priv->id_offset = 0;
+          i = 0;
+          clutter_actor_queue_redraw (CLUTTER_ACTOR(focusable));
+        }
+      else
+        {
+          i++;
+          if (i > priv->last_shown_id)
+            {
+              priv->id_offset++;
+              clutter_actor_queue_redraw (CLUTTER_ACTOR(focusable));
+            }
+        }
+
+      while (i < priv->children->len)
+        {
+          if (i == start)
+            break;
+
+          child = &g_array_index (priv->children, MxMenuChild, i);
+
+          result = mx_focusable_accept_focus (MX_FOCUSABLE (child->box), 0);
+
+          if (result)
+            return result;
+
+          /* loop */
+          if (i == priv->children->len - 1)
+            i = -1;
+
+          i++;
+        }
+
+    case MX_FOCUS_DIRECTION_OUT:
+      if (priv->internal_focus_push)
+        {
+          /* do nothing if this notification was caused internally */
+          priv->internal_focus_push = FALSE;
+          return NULL;
+        }
+
+    default:
+      break;
+    }
+
+  clutter_actor_hide (CLUTTER_ACTOR (focusable));
+  return NULL;
+}
+
+static MxFocusable*
+mx_menu_accept_focus (MxFocusable *focusable,
+                      MxFocusHint  hint)
+{
+  MxMenuPrivate *priv = MX_MENU (focusable)->priv;
+  MxMenuChild *child;
+
+  child = &g_array_index (priv->children, MxMenuChild, 0);
+
+  return mx_focusable_accept_focus (MX_FOCUSABLE (child->box), 0);
+}
+
+static void
+mx_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->move_focus = mx_menu_move_focus;
+  iface->accept_focus = mx_menu_accept_focus;
 }
 
 static void
@@ -245,12 +371,54 @@ mx_menu_allocate (ClutterActor           *actor,
   ClutterActorBox child_box;
   MxMenuPrivate *priv = MX_MENU (actor)->priv;
 
+  gfloat available_h = box->y2-box->y1;
+
+  /*
+   * first of all, we have to check if the allocated height is our
+   * natural height...
+   */
+  gfloat menu_nat_h;
+  mx_menu_get_preferred_height (actor, box->x2-box->x1, NULL, &menu_nat_h);
+  if (available_h < menu_nat_h)
+    {
+      /*
+       * ...if not, we have to draw 2 buttons in order to let the
+       * user be able to navigate in the whole menu.
+       */
+      priv->scrolling_mode = TRUE;
+    }
+  else
+    {
+      priv->scrolling_mode = FALSE;
+    }
+
   /* Allocate children */
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
   child_box.x1 = padding.left;
   child_box.y1 = padding.top;
   child_box.x2 = box->x2 - box->x1 - padding.right;
-  for (i = 0; i < priv->children->len; i++)
+
+  gfloat down_but_height;
+  clutter_actor_get_preferred_height (priv->down_button,
+                                      child_box.x2 - child_box.x1,
+                                      NULL,
+                                      &down_but_height);
+
+  if (priv->scrolling_mode)
+    {
+      clutter_actor_get_preferred_height (priv->up_button,
+                                          child_box.x2 - child_box.x1,
+                                          NULL,
+                                          &child_box.y2);
+      child_box.y2 += child_box.y1;
+      clutter_actor_allocate (priv->up_button, &child_box, flags);
+      child_box.y1 = child_box.y2 + 1;
+
+      available_h -= down_but_height;
+    }
+
+
+  for (i = priv->id_offset; i < priv->children->len; i++)
     {
       gfloat natural_height;
 
@@ -262,12 +430,26 @@ mx_menu_allocate (ClutterActor           *actor,
                                           NULL,
                                           &natural_height);
       child_box.y2 = child_box.y1 + natural_height;
+      if (child_box.y2 >= available_h)
+        {
+          priv->last_shown_id = i-1;
+          break;
+        }
 
       clutter_actor_allocate (CLUTTER_ACTOR (child->box), &child_box, flags);
 
       child_box.y1 = child_box.y2 + 1;
     }
+    if (priv->children->len == i)
+      {
+        priv->last_shown_id = i-1;
+      }
 
+    if (priv->scrolling_mode)
+      {
+        child_box.y2 = child_box.y1 + down_but_height;
+        clutter_actor_allocate (priv->down_button, &child_box, flags);
+      }
   /* Chain up and allocate background */
   CLUTTER_ACTOR_CLASS (mx_menu_parent_class)->allocate (actor, box, flags);
 }
@@ -282,12 +464,18 @@ mx_menu_floating_paint (ClutterActor *menu)
   MX_FLOATING_WIDGET_CLASS (mx_menu_parent_class)->floating_paint (menu);
 
   /* Paint children */
-  for (i = 0; i < priv->children->len; i++)
+  for (i = priv->id_offset; i <= priv->last_shown_id; i++)
     {
       MxMenuChild *child = &g_array_index (priv->children, MxMenuChild,
                                             i);
       clutter_actor_paint (CLUTTER_ACTOR (child->box));
     }
+
+  if(priv->scrolling_mode)
+    {
+      clutter_actor_paint (priv->up_button);
+      clutter_actor_paint (priv->down_button);
+    }
 }
 
 static void
@@ -302,7 +490,7 @@ mx_menu_floating_pick (ClutterActor       *menu,
   MX_FLOATING_WIDGET_CLASS (mx_menu_parent_class)->floating_pick (menu, color);
 
   /* pick children */
-  for (i = 0; i < priv->children->len; i++)
+  for (i = priv->id_offset; i <= priv->last_shown_id; i++)
     {
       MxMenuChild *child = &g_array_index (priv->children, MxMenuChild, i);
 
@@ -311,6 +499,11 @@ mx_menu_floating_pick (ClutterActor       *menu,
           clutter_actor_paint (CLUTTER_ACTOR (child->box));
         }
     }
+  if(priv->scrolling_mode)
+    {
+      clutter_actor_paint(priv->up_button);
+      clutter_actor_paint(priv->down_button);
+    }
 }
 
 static void
@@ -328,6 +521,8 @@ mx_menu_map (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_menu_parent_class)->map (actor);
 
+  clutter_actor_map (priv->up_button);
+  clutter_actor_map (priv->down_button);
   for (i = 0; i < priv->children->len; i++)
     {
       MxMenuChild *child = &g_array_index (priv->children, MxMenuChild,
@@ -355,6 +550,8 @@ mx_menu_unmap (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_menu_parent_class)->unmap (actor);
 
+  clutter_actor_unmap (priv->up_button);
+  clutter_actor_unmap (priv->down_button);
   for (i = 0; i < priv->children->len; i++)
     {
       MxMenuChild *child = &g_array_index (priv->children, MxMenuChild,
@@ -388,6 +585,22 @@ mx_menu_event (ClutterActor *actor,
     case CLUTTER_BUTTON_RELEASE:
     case CLUTTER_SCROLL:
       return TRUE;
+
+    case CLUTTER_KEY_PRESS:
+    case CLUTTER_KEY_RELEASE:
+      /* hide the menu if the escape key was pressed */
+      if (((ClutterKeyEvent*) event)->keyval == CLUTTER_KEY_Escape
+          && CLUTTER_ACTOR_IS_VISIBLE (actor))
+        {
+          clutter_actor_set_reactive (actor, FALSE);
+          clutter_actor_animate (actor, CLUTTER_LINEAR, 250,
+                                 "opacity", (guchar) 0,
+                                 "signal-swapped::completed",
+                                 clutter_actor_hide,
+                                 actor,
+                                 NULL);
+        }
+
     default:
       return FALSE;
     }
@@ -417,6 +630,8 @@ mx_menu_captured_event_handler (ClutterActor *actor,
       if (source == (ClutterActor*) child->box)
         return FALSE;
     }
+  if (source == priv->up_button || source == priv->down_button)
+    return FALSE;
 
   /* hide the menu if the user clicks outside the menu */
   if (event->type == CLUTTER_BUTTON_PRESS)
@@ -445,6 +660,7 @@ static void
 mx_menu_show (ClutterActor *actor)
 {
   ClutterAnimation *animation = NULL;
+  ClutterStage *stage;
 
   /* set reactive and opacity, since these may have been set by the fade-out
    * animation (e.g. from captured_event_handler or button_release_cb) */
@@ -457,6 +673,14 @@ mx_menu_show (ClutterActor *actor)
 
   /* chain up to run show after re-setting properties above */
   CLUTTER_ACTOR_CLASS (mx_menu_parent_class)->show (actor);
+
+  clutter_actor_grab_key_focus (actor);
+
+  stage = (ClutterStage*) clutter_actor_get_stage (actor);
+
+  mx_focus_manager_push_focus (mx_focus_manager_get_for_stage (stage),
+                               MX_FOCUSABLE (actor));
+
 }
 
 static void
@@ -470,21 +694,7 @@ mx_menu_style_changed (MxMenu              *menu,
                        MxStyleChangedFlags  flags)
 {
   MxMenuPrivate *priv = menu->priv;
-  gint i, spacing;
-
-  mx_stylable_get (MX_STYLABLE (menu),
-                   "x-mx-spacing", &spacing,
-                   NULL);
-
-  if (priv->spacing != spacing)
-    {
-      priv->spacing = spacing;
-      for (i = 0; i < priv->children->len; i++)
-        {
-          MxMenuChild *child = &g_array_index (priv->children, MxMenuChild, i);
-          mx_box_layout_set_spacing (MX_BOX_LAYOUT (child->box), priv->spacing);
-        }
-    }
+  gint i;
 
   for (i = 0; i < priv->children->len; i++)
     {
@@ -532,6 +742,73 @@ mx_menu_class_init (MxMenuClass *klass)
                   G_TYPE_NONE, 1, MX_TYPE_ACTION);
 }
 
+static gboolean
+_up_callback (MxMenu *self)
+{
+  MxMenuPrivate *priv = self->priv;
+  if (0 < priv->id_offset)
+    {
+      priv->id_offset--;
+      clutter_actor_queue_relayout (CLUTTER_ACTOR(self));
+    }
+  return TRUE;
+}
+
+static gboolean
+mx_menu_up_enter (ClutterActor         *actor,
+                  ClutterCrossingEvent *event,
+                  MxMenu               *self)
+{
+  MxMenuPrivate *priv = self->priv;
+  priv->up_source = g_timeout_add (250, (GSourceFunc)_up_callback, self);
+  return FALSE;
+}
+
+static gboolean
+mx_menu_up_leave (ClutterActor         *actor,
+                  ClutterCrossingEvent *event,
+                  MxMenu               *self)
+{
+  MxMenuPrivate *priv = self->priv;
+  g_source_remove (priv->up_source);
+  return FALSE;
+}
+
+static gboolean
+_down_callback (MxMenu *self)
+{
+  MxMenuPrivate *priv = self->priv;
+  gint last_id_offset = (priv->children->len - 1) -
+    (priv->last_shown_id - priv->id_offset);
+
+  if(last_id_offset > priv->id_offset)
+  {
+    priv->id_offset++;
+    clutter_actor_queue_relayout (CLUTTER_ACTOR(self));
+  }
+  return TRUE;
+}
+
+static gboolean
+mx_menu_down_enter (ClutterActor         *actor,
+                    ClutterCrossingEvent *event,
+                    MxMenu               *self)
+{
+  MxMenuPrivate *priv = self->priv;
+  priv->down_source = g_timeout_add (250, (GSourceFunc)_down_callback, self);
+  return FALSE;
+}
+
+static gboolean
+mx_menu_down_leave (ClutterActor         *actor,
+                    ClutterCrossingEvent *event,
+                    MxMenu               *self)
+{
+  MxMenuPrivate *priv = self->priv;
+  g_source_remove (priv->down_source);
+  return FALSE;
+}
+
 static void
 mx_menu_init (MxMenu *self)
 {
@@ -545,6 +822,23 @@ mx_menu_init (MxMenu *self)
 
   g_signal_connect (self, "style-changed", G_CALLBACK (mx_menu_style_changed),
                     NULL);
+  priv->id_offset = 0;
+  priv->last_shown_id = 0;
+
+  priv->up_button = mx_button_new_with_label("/\\");
+  clutter_actor_set_parent (CLUTTER_ACTOR (priv->up_button),
+                            CLUTTER_ACTOR (self));
+  g_signal_connect (priv->up_button, "enter-event",
+                    G_CALLBACK(mx_menu_up_enter), self);
+  g_signal_connect (priv->up_button, "leave-event",
+                    G_CALLBACK(mx_menu_up_leave), self);
+  priv->down_button = mx_button_new_with_label ("\\/");
+  clutter_actor_set_parent (CLUTTER_ACTOR (priv->down_button),
+                            CLUTTER_ACTOR (self));
+  g_signal_connect (priv->down_button, "enter-event",
+                    G_CALLBACK(mx_menu_down_enter), self);
+  g_signal_connect (priv->down_button, "leave-event",
+                    G_CALLBACK(mx_menu_down_leave), self);
 }
 
 /**
@@ -561,13 +855,12 @@ mx_menu_new (void)
 }
 
 static void
-mx_menu_button_release_cb (MxBoxLayout  *box,
-                           ClutterEvent *event,
+mx_menu_button_clicked_cb (ClutterActor *box,
                            MxAction     *action)
 {
   MxMenu *menu;
 
-  menu = MX_MENU (clutter_actor_get_parent (CLUTTER_ACTOR (box)));
+  menu = MX_MENU (clutter_actor_get_parent (box));
 
   /* set the menu unreactive to prevent other items being hilighted */
   clutter_actor_set_reactive ((ClutterActor*) menu, FALSE);
@@ -586,7 +879,28 @@ mx_menu_button_release_cb (MxBoxLayout  *box,
 
   g_object_unref (action);
   g_object_unref (menu);
+}
 
+static gboolean
+mx_menu_button_enter_event_cb (ClutterActor *box,
+                               ClutterEvent *event,
+                               gpointer      user_data)
+{
+  MxMenuPrivate *priv = MX_MENU (user_data)->priv;
+  ClutterStage *stage;
+
+  /* each menu item grabs focus when hovered */
+
+  stage = (ClutterStage *) clutter_actor_get_stage (box);
+
+  /* ensure the menu is not closed when focus is pushed to another actor */
+  priv->internal_focus_push = TRUE;
+
+  mx_focus_manager_push_focus (mx_focus_manager_get_for_stage (stage),
+                               MX_FOCUSABLE (box));
+
+  /* prevent the hover pseudo-class from being applied */
+  return TRUE;
 }
 
 /**
@@ -602,7 +916,6 @@ mx_menu_add_action (MxMenu   *menu,
                     MxAction *action)
 {
   MxMenuChild child;
-  ClutterActor *label, *icon;
 
   g_return_if_fail (MX_IS_MENU (menu));
   g_return_if_fail (MX_IS_ACTION (action));
@@ -611,37 +924,16 @@ mx_menu_add_action (MxMenu   *menu,
 
   child.action = g_object_ref_sink (action);
   /* TODO: Connect to notify signals in case action properties change */
-  child.box = g_object_new (MX_TYPE_BOX_LAYOUT,
-                            "reactive", TRUE,
-                            "spacing", priv->spacing,
+  child.box = g_object_new (MX_TYPE_BUTTON,
+                            "action", child.action,
+                            "x-align", MX_ALIGN_START,
                             NULL);
+  mx_button_set_action (MX_BUTTON (child.box), child.action);
 
-  if (mx_action_get_icon (action))
-    {
-      MxIconTheme *theme = mx_icon_theme_get_default ();
-
-      if (mx_icon_theme_has_icon (theme, mx_action_get_icon (action)))
-        {
-          icon = mx_icon_new ();
-          mx_icon_set_icon_name (MX_ICON (icon), mx_action_get_icon (action));
-
-          clutter_container_add_actor (CLUTTER_CONTAINER (child.box), icon);
-          clutter_container_child_set (CLUTTER_CONTAINER (child.box), icon,
-                                       "y-fill", FALSE,
-                                       NULL);
-        }
-    }
-
-  label = mx_label_new_with_text (mx_action_get_display_name (action));
-
-  clutter_container_add_actor (CLUTTER_CONTAINER (child.box), label);
-
-  clutter_container_child_set (CLUTTER_CONTAINER (child.box), label,
-                               "y-fill", FALSE,
-                               NULL);
-
-  g_signal_connect (child.box, "button-release-event",
-                    G_CALLBACK (mx_menu_button_release_cb), action);
+  g_signal_connect (child.box, "clicked",
+                    G_CALLBACK (mx_menu_button_clicked_cb), action);
+  g_signal_connect (child.box, "enter-event",
+                    G_CALLBACK (mx_menu_button_enter_event_cb), menu);
   clutter_actor_set_parent (CLUTTER_ACTOR (child.box),
                             CLUTTER_ACTOR (menu));
 
