@@ -47,7 +47,7 @@
 
 static void mx_stylable_iface_init (MxStylableIface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (MxScrollBar, mx_scroll_bar, MX_TYPE_BIN,
+G_DEFINE_TYPE_WITH_CODE (MxScrollBar, mx_scroll_bar, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
                                                 mx_stylable_iface_init))
 
@@ -250,14 +250,9 @@ mx_scroll_bar_unmap (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_scroll_bar_parent_class)->unmap (actor);
 
-  if (priv->bw_stepper)
-    clutter_actor_unmap (priv->bw_stepper);
-
-  if (priv->fw_stepper)
-    clutter_actor_unmap (priv->fw_stepper);
-
-  if (priv->trough)
-    clutter_actor_unmap (priv->trough);
+  clutter_actor_unmap (priv->bw_stepper);
+  clutter_actor_unmap (priv->fw_stepper);
+  clutter_actor_unmap (priv->trough);
 
   if (priv->handle)
     clutter_actor_unmap (priv->handle);
