@@ -33,9 +33,9 @@
  *
  * <figure id="mx-scroll-view">
  *   <title>#MxScrollView around an #MxBoxLayout</title>
- *   <para>An example of an #MxScrollView wrapped around an #MxBoxLayout 
- *   actor (which implements #MxScrollable). The #MxBoxLayout contains 
- *   nine #ClutterRectangle instances, but the stage is too small for 
+ *   <para>An example of an #MxScrollView wrapped around an #MxBoxLayout
+ *   actor (which implements #MxScrollable). The #MxBoxLayout contains
+ *   nine #ClutterRectangle instances, but the stage is too small for
  *   all of them to be visible. The #MxScrollView adds the appropriate
  *   horizontal scroll, which makes it possible to scroll to the end
  *   of the row of rectangles.</para>
@@ -60,8 +60,6 @@
 static void clutter_container_iface_init (ClutterContainerIface *iface);
 static void mx_stylable_iface_init (MxStylableIface *iface);
 
-static ClutterContainerIface *mx_scroll_view_parent_iface = NULL;
-
 G_DEFINE_TYPE_WITH_CODE (MxScrollView, mx_scroll_view, MX_TYPE_BIN,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
                                                 clutter_container_iface_init)
@@ -673,7 +671,8 @@ mx_scroll_view_class_init (MxScrollViewClass *klass)
 
   pspec = g_param_spec_boolean ("enable-gestures",
                                 "Enable Gestures",
-                                "Enable use of pointer gestures for scrolling",
+                                "Enable use of pointer gestures for scrolling "
+                                "if Mx was built with ClutterGesture support",
                                 FALSE,
                                 MX_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_ENABLE_GESTURES, pspec);
@@ -894,8 +893,8 @@ mx_scroll_view_init (MxScrollView *self)
 }
 
 static void
-mx_scroll_view_add (ClutterContainer *container,
-                    ClutterActor     *actor)
+mx_scroll_view_actor_added (ClutterContainer *container,
+                            ClutterActor     *actor)
 {
   MxScrollView *self = MX_SCROLL_VIEW (container);
   MxScrollViewPrivate *priv = self->priv;
@@ -904,9 +903,6 @@ mx_scroll_view_add (ClutterContainer *container,
     {
       priv->child = actor;
 
-      /* chain up to MxBin::add() */
-      mx_scroll_view_parent_iface->add (container, actor);
-
       /* Get adjustments for scroll-bars */
       g_signal_connect (actor, "notify::horizontal-adjustment",
                         G_CALLBACK (child_hadjustment_notify_cb),
@@ -927,8 +923,8 @@ mx_scroll_view_add (ClutterContainer *container,
 }
 
 static void
-mx_scroll_view_remove (ClutterContainer *container,
-                       ClutterActor     *actor)
+mx_scroll_view_actor_removed (ClutterContainer *container,
+                              ClutterActor     *actor)
 {
   MxScrollViewPrivate *priv = MX_SCROLL_VIEW (container)->priv;
 
@@ -936,9 +932,6 @@ mx_scroll_view_remove (ClutterContainer *container,
     {
       g_object_ref (priv->child);
 
-      /* chain up to MxBin::remove() */
-      mx_scroll_view_parent_iface->remove (container, actor);
-
       g_signal_handlers_disconnect_by_func (priv->child,
                                             child_hadjustment_notify_cb,
                                             container);
@@ -972,15 +965,10 @@ mx_scroll_view_foreach_with_internals (ClutterContainer *container,
 static void
 clutter_container_iface_init (ClutterContainerIface *iface)
 {
-  /* store a pointer to the MxBin implementation of
-   * ClutterContainer so that we can chain up when
-   * overriding the methods
-   */
-  mx_scroll_view_parent_iface = g_type_interface_peek_parent (iface);
-
-  iface->add = mx_scroll_view_add;
-  iface->remove = mx_scroll_view_remove;
   iface->foreach_with_internals = mx_scroll_view_foreach_with_internals;
+
+  iface->actor_added = mx_scroll_view_actor_added;
+  iface->actor_removed = mx_scroll_view_actor_removed;
 }
 
 /**
@@ -1005,7 +993,7 @@ mx_scroll_view_set_enable_mouse_scrolling (MxScrollView *scroll,
 
   g_return_if_fail (MX_IS_SCROLL_VIEW (scroll));
 
-  priv = MX_SCROLL_VIEW (scroll)->priv;
+  priv = scroll->priv;
 
   if (priv->mouse_scroll != enabled)
     {
@@ -1026,7 +1014,7 @@ mx_scroll_view_get_enable_mouse_scrolling (MxScrollView *scroll)
 
   g_return_val_if_fail (MX_IS_SCROLL_VIEW (scroll), FALSE);
 
-  priv = MX_SCROLL_VIEW (scroll)->priv;
+  priv = scroll->priv;
 
   return priv->mouse_scroll;
 }
@@ -1039,7 +1027,7 @@ mx_scroll_view_set_enable_gestures (MxScrollView *scroll,
 
   g_return_if_fail (MX_IS_SCROLL_VIEW (scroll));
 
-  priv = MX_SCROLL_VIEW (scroll)->priv;
+  priv = scroll->priv;
 
   if (priv->enable_gestures != enabled)
     {
