@@ -649,11 +649,14 @@ mx_stack_allocate (ClutterActor           *actor,
 }
 
 static void
-mx_stack_paint_children (ClutterActor *actor)
+mx_stack_paint (ClutterActor *actor)
 {
+  GList *c;
+
   MxStackPrivate *priv = MX_STACK (actor)->priv;
 
-  GList *c;
+  /* allow MxWidget to paint the background */
+  CLUTTER_ACTOR_CLASS (mx_stack_parent_class)->paint (actor);
 
   for (c = priv->children; c; c = c->next)
     {
@@ -683,23 +686,11 @@ mx_stack_paint_children (ClutterActor *actor)
     }
 }
 
-static void
-mx_stack_paint (ClutterActor *actor)
-{
-  /* allow MxWidget to paint the background */
-  CLUTTER_ACTOR_CLASS (mx_stack_parent_class)->paint (actor);
-
-
-  mx_stack_paint_children (actor);
-}
-
 static void
 mx_stack_pick (ClutterActor       *actor,
                const ClutterColor *color)
 {
-  CLUTTER_ACTOR_CLASS (mx_stack_parent_class)->pick (actor, color);
-
-  mx_stack_paint_children (actor);
+  mx_stack_paint (actor);
 }
 
 static void
