@@ -31,16 +31,16 @@
  * with rows and columns. #MxTable adds several child properties to its
  * children which control their position and size in the table.
  *
- * While other layouts (like #MxGrid) allow you to achieve 
- * table-like effects, #MxTable is the only layout which allows 
+ * While other layouts (like #MxGrid) allow you to achieve
+ * table-like effects, #MxTable is the only layout which allows
  * you to precisely (and easily) place elements at particular grid coordinates,
  * via mx_table_add_actor().
  *
  * <figure id="mx-table">
  *   <title>#MxTable, 3 rows by 3 columns</title>
- *   <para>Notice how rectangles have only been placed in a few of 
- *   the cells inside the table: this would be very difficult to do 
- *   with any other layout, without having layouts within layouts within 
+ *   <para>Notice how rectangles have only been placed in a few of
+ *   the cells inside the table: this would be very difficult to do
+ *   with any other layout, without having layouts within layouts within
  *   layouts...</para>
  *   <graphic fileref="MxTable.png" format="PNG"/>
  * </figure>
@@ -82,7 +82,6 @@ enum
 typedef struct
 {
   guint expand : 1;
-  guint shrink : 1;
   guint is_visible : 1;
 
   gfloat min_size;
@@ -575,7 +574,7 @@ mx_table_depth_sort_cb (gconstpointer a,
                         gconstpointer b)
 {
   gfloat depth_a = clutter_actor_get_depth ((ClutterActor *)a);
-  gfloat depth_b = clutter_actor_get_depth ((ClutterActor *)b);
+  gfloat depth_b = clutter_actor_get_depth ((ClutterActor *)a);
 
   if (depth_a < depth_b)
     return -1;
@@ -719,8 +718,6 @@ mx_table_dispose (GObject *gobject)
   G_OBJECT_CLASS (mx_table_parent_class)->dispose (gobject);
 }
 
-#define CLAMP_TO_PIXEL(x) ((float)((int)(x)))
-
 static void
 mx_table_calculate_col_widths (MxTable *table,
                                gint     for_width)
@@ -786,7 +783,6 @@ mx_table_calculate_col_widths (MxTable *table,
     {
       MxTableChild *meta;
       ClutterActor *child;
-      DimensionData *col;
       gfloat c_min, c_pref;
       gfloat min_width, pref_width;
       gint start_col, end_col;
@@ -803,7 +799,6 @@ mx_table_calculate_col_widths (MxTable *table,
       if (meta->col_span < 2)
         continue;
 
-      col = &columns[meta->col];
       start_col = meta->col;
       end_col = meta->col + meta->col_span - 1;
 
@@ -826,11 +821,12 @@ mx_table_calculate_col_widths (MxTable *table,
 
           /* If this child is visible, then the columns it spans
              are also visible */
-          if (!col->is_visible)
+          if (!columns[i].is_visible)
             {
-              col->is_visible = TRUE;
+              columns[i].is_visible = TRUE;
               priv->visible_cols++;
             }
+          columns[i].expand = MAX (columns[i].expand, meta->x_expand);
         }
       min_width += priv->col_spacing * (meta->col_span - 1);
       pref_width += priv->col_spacing * (meta->col_span - 1);
@@ -1120,6 +1116,7 @@ mx_table_calculate_row_heights (MxTable *table,
               rows[i].is_visible = TRUE;
               priv->visible_rows++;
             }
+          rows[i].expand = MAX (rows[i].expand, meta->y_expand);
         }
       min_height += priv->row_spacing * (meta->row_span - 1);
       pref_height += priv->row_spacing * (meta->row_span - 1);
@@ -1485,8 +1482,6 @@ mx_table_get_preferred_width (ClutterActor *self,
   MxPadding padding;
   DimensionData *columns;
 
-  mx_widget_get_padding (MX_WIDGET (self), &padding);
-
   if (priv->n_cols < 1)
     {
       *min_width_p = 0;
@@ -1494,12 +1489,16 @@ mx_table_get_preferred_width (ClutterActor *self,
       return;
     }
 
+  /* use min_widths to help allocation of height-for-width widgets */
   mx_table_calculate_dimensions (MX_TABLE (self), -1, for_height);
 
   columns = &g_array_index (priv->columns, DimensionData, 0);
 
-  total_min_width = padding.left + padding.right
-                    + (priv->visible_cols - 1) * (float) priv->col_spacing;
+  mx_widget_get_padding (MX_WIDGET (self), &padding);
+
+  /* start off with padding plus row spacing */
+  total_min_width = padding.left + padding.right + (priv->visible_cols - 1) *
+                    (float)(priv->col_spacing);
   total_pref_width = total_min_width;
 
   for (i = 0; i < priv->n_cols; i++)
@@ -1520,8 +1519,9 @@ mx_table_get_preferred_height (ClutterActor *self,
                                gfloat       *min_height_p,
                                gfloat       *natural_height_p)
 {
+  gfloat total_min_height, total_pref_height;
   MxTablePrivate *priv = MX_TABLE (self)->priv;
-  gint i, total_min_height, total_pref_height;
+  gint i;
   MxPadding padding;
   DimensionData *rows;
 
@@ -1657,8 +1657,6 @@ mx_table_class_init (MxTableClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
 
-  /* MxWidgetClass *mx_widget_class = MX_WIDGET_CLASS (klass); */
-
   g_type_class_add_private (klass, sizeof (MxTablePrivate));
 
   gobject_class->set_property = mx_table_set_property;
