@@ -1,8 +1,9 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-texture-frame.h: Expandible texture actor
  *
  * Copyright 2007 OpenedHand
- * Copyright 2009 Intel Corporation.
+ * Copyright 2009, 2012 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -129,17 +130,134 @@ mx_texture_frame_get_preferred_height (ClutterActor *self,
     }
 }
 
+void
+mx_texture_frame_paint_texture (CoglHandle *texture,
+                                guint8      opacity,
+                                gfloat      top,
+                                gfloat      right,
+                                gfloat      bottom,
+                                gfloat      left,
+                                gfloat      width,
+                                gfloat      height)
+{
+  gfloat tex_width, tex_height;
+  gfloat ex, ey;
+  gfloat tx1, ty1, tx2, ty2;
+  static CoglHandle template_material;
+  CoglHandle material;
+
+
+  /* setup the template material */
+  if (!template_material)
+    template_material = cogl_material_new ();
+
+  /* create the material and apply opacity */
+  material = cogl_material_copy (template_material);
+  cogl_material_set_color4ub (material, opacity, opacity, opacity, opacity);
+
+  /* add the texture */
+  cogl_material_set_layer (material, 0, texture);
+
+  /* set the source */
+  cogl_set_source (material);
+
+
+
+  tex_width  = cogl_texture_get_width (texture);
+  tex_height = cogl_texture_get_height (texture);
+
+  /* simple stretch */
+  if (left == 0 && right == 0 && top == 0
+      && bottom == 0)
+    {
+      cogl_rectangle (0, 0, width, height);
+      return;
+    }
+
+  tx1 = left / tex_width;
+  tx2 = (tex_width - right) / tex_width;
+  ty1 = top / tex_height;
+  ty2 = (tex_height - bottom) / tex_height;
+
+  ex = width - right;
+  if (ex < left)
+    ex = left;
+
+  ey = height - bottom;
+  if (ey < top)
+    ey = top;
+
+
+  {
+    GLfloat rectangles[] =
+    {
+      /* top left corner */
+      0, 0,
+      left, top,
+      0.0, 0.0,
+      tx1, ty1,
+
+      /* top middle */
+      left, 0,
+      MAX (left, ex), top,
+      tx1, 0.0,
+      tx2, ty1,
+
+      /* top right */
+      ex, 0,
+      MAX (ex + right, width), top,
+      tx2, 0.0,
+      1.0, ty1,
+
+      /* mid left */
+      0, top,
+      left,  ey,
+      0.0, ty1,
+      tx1, ty2,
+
+      /* center */
+      left, top,
+      ex, ey,
+      tx1, ty1,
+      tx2, ty2,
+
+      /* mid right */
+      ex, top,
+      MAX (ex + right, width), ey,
+      tx2, ty1,
+      1.0, ty2,
+
+      /* bottom left */
+      0, ey,
+      left, MAX (ey + bottom, height),
+      0.0, ty2,
+      tx1, 1.0,
+
+      /* bottom center */
+      left, ey,
+      ex, MAX (ey + bottom, height),
+      tx1, ty2,
+      tx2, 1.0,
+
+      /* bottom right */
+      ex, ey,
+      MAX (ex + right, width), MAX (ey + bottom, height),
+      tx2, ty2,
+      1.0, 1.0
+    };
+
+    cogl_rectangles_with_texture_coords (rectangles, 9);
+  }
+}
+
 static void
 mx_texture_frame_paint (ClutterActor *self)
 {
   MxTextureFramePrivate *priv = MX_TEXTURE_FRAME (self)->priv;
-  CoglHandle cogl_texture = COGL_INVALID_HANDLE;
   CoglHandle cogl_material = COGL_INVALID_HANDLE;
+  CoglHandle cogl_texture = COGL_INVALID_HANDLE;
   ClutterActorBox box = { 0, };
   gfloat width, height;
-  gfloat tex_width, tex_height;
-  gfloat ex, ey;
-  gfloat tx1, ty1, tx2, ty2;
   guint8 opacity;
 
   /* no need to paint stuff if we don't have a texture */
@@ -155,90 +273,21 @@ mx_texture_frame_paint (ClutterActor *self)
   cogl_texture = clutter_texture_get_cogl_texture (priv->parent_texture);
   if (cogl_texture == COGL_INVALID_HANDLE)
     return;
+
   cogl_material = clutter_texture_get_cogl_material (priv->parent_texture);
   if (cogl_material == COGL_INVALID_HANDLE)
     return;
 
-  tex_width  = cogl_texture_get_width (cogl_texture);
-  tex_height = cogl_texture_get_height (cogl_texture);
 
   clutter_actor_get_allocation_box (self, &box);
   width = box.x2 - box.x1;
   height = box.y2 - box.y1;
 
-  tx1 = priv->left / tex_width;
-  tx2 = (tex_width - priv->right) / tex_width;
-  ty1 = priv->top / tex_height;
-  ty2 = (tex_height - priv->bottom) / tex_height;
-
-  ex = width - priv->right;
-  if (ex < 0)
-    ex = priv->right;           /* FIXME ? */
-
-  ey = height - priv->bottom;
-  if (ey < 0)
-    ey = priv->bottom;          /* FIXME ? */
 
   opacity = clutter_actor_get_paint_opacity (self);
 
-  /* Paint using the parent texture's material. It should already have
-     the cogl texture set as the first layer */
-  /* NB: for correct blending we need set a preumultiplied color here: */
-  cogl_material_set_color4ub (cogl_material,
-                              opacity, opacity, opacity, opacity);
-  cogl_set_source (cogl_material);
-
-  {
-    GLfloat rectangles[] =
-    {
-      /* top left corner */
-      0, 0, priv->left, priv->top,
-      0.0, 0.0,
-      tx1, ty1,
-
-      /* top middle */
-      priv->left, 0, ex, priv->top,
-      tx1, 0.0,
-      tx2, ty1,
-
-      /* top right */
-      ex, 0, width, priv->top,
-      tx2, 0.0,
-      1.0, ty1,
-
-      /* mid left */
-      0, priv->top, priv->left, ey,
-      0.0, ty1,
-      tx1, ty2,
-
-      /* center */
-      priv->left, priv->top, ex, ey,
-      tx1, ty1,
-      tx2, ty2,
-
-      /* mid right */
-      ex, priv->top, width, ey,
-      tx2, ty1,
-      1.0, ty2,
-
-      /* bottom left */
-      0, ey, priv->left, height,
-      0.0, ty2,
-      tx1, 1.0,
-
-      /* bottom center */
-      priv->left, ey, ex, height,
-      tx1, ty2,
-      tx2, 1.0,
-
-      /* bottom right */
-      ex, ey, width, height,
-      tx2, ty2,
-      1.0, 1.0
-    };
-
-    cogl_rectangles_with_texture_coords (rectangles, 9);
-  }
+  mx_texture_frame_paint_texture (cogl_texture, opacity, priv->top, priv->right,
+                                  priv->bottom, priv->left, width, height);
 }
 
 static inline void
@@ -392,6 +441,16 @@ mx_texture_frame_dispose (GObject *gobject)
   G_OBJECT_CLASS (mx_texture_frame_parent_class)->dispose (gobject);
 }
 
+static gboolean
+mx_texture_frame_get_paint_volume (ClutterActor       *actor,
+                                   ClutterPaintVolume *volume)
+{
+  if (G_OBJECT_TYPE (actor) != mx_texture_frame_get_type ())
+    return FALSE;
+
+  return clutter_paint_volume_set_from_allocation (volume, actor);
+}
+
 static void
 mx_texture_frame_class_init (MxTextureFrameClass *klass)
 {
@@ -406,6 +465,7 @@ mx_texture_frame_class_init (MxTextureFrameClass *klass)
   actor_class->get_preferred_height =
     mx_texture_frame_get_preferred_height;
   actor_class->paint = mx_texture_frame_paint;
+  actor_class->get_paint_volume = mx_texture_frame_get_paint_volume;
 
   gobject_class->set_property = mx_texture_frame_set_property;
   gobject_class->get_property = mx_texture_frame_get_property;
@@ -424,7 +484,8 @@ mx_texture_frame_class_init (MxTextureFrameClass *klass)
                               "Left offset",
                               0, G_MAXFLOAT,
                               0,
-                              MX_PARAM_READWRITE);
+                              MX_PARAM_READWRITE |
+                              G_PARAM_CONSTRUCT);
   g_object_class_install_property (gobject_class, PROP_LEFT, pspec);
 
   pspec = g_param_spec_float ("top",
@@ -432,7 +493,8 @@ mx_texture_frame_class_init (MxTextureFrameClass *klass)
                               "Top offset",
                               0, G_MAXFLOAT,
                               0,
-                              MX_PARAM_READWRITE);
+                              MX_PARAM_READWRITE |
+                              G_PARAM_CONSTRUCT);
   g_object_class_install_property (gobject_class, PROP_TOP, pspec);
 
   pspec = g_param_spec_float ("bottom",
@@ -440,7 +502,8 @@ mx_texture_frame_class_init (MxTextureFrameClass *klass)
                               "Bottom offset",
                               0, G_MAXFLOAT,
                               0,
-                              MX_PARAM_READWRITE);
+                              MX_PARAM_READWRITE |
+                              G_PARAM_CONSTRUCT);
   g_object_class_install_property (gobject_class, PROP_BOTTOM, pspec);
 
   pspec = g_param_spec_float ("right",
@@ -448,16 +511,15 @@ mx_texture_frame_class_init (MxTextureFrameClass *klass)
                               "Right offset",
                               0, G_MAXFLOAT,
                               0,
-                              MX_PARAM_READWRITE);
+                              MX_PARAM_READWRITE |
+                              G_PARAM_CONSTRUCT);
   g_object_class_install_property (gobject_class, PROP_RIGHT, pspec);
 }
 
 static void
 mx_texture_frame_init (MxTextureFrame *self)
 {
-  MxTextureFramePrivate *priv;
-
-  self->priv = priv = MX_TEXTURE_FRAME_GET_PRIVATE (self);
+  self->priv = MX_TEXTURE_FRAME_GET_PRIVATE (self);
 }
 
 /**
@@ -501,7 +563,7 @@ mx_texture_frame_new (ClutterTexture *texture,
  *
  * Return the texture used by the #MxTextureFrame
  *
- * Returns: a #ClutterTexture owned by the #MxTextureFrame
+ * Returns: (transfer none): a #ClutterTexture owned by the #MxTextureFrame
  */
 ClutterTexture *
 mx_texture_frame_get_parent_texture (MxTextureFrame *frame)
@@ -547,10 +609,28 @@ mx_texture_frame_set_parent_texture (MxTextureFrame *frame,
 
   if (texture)
     {
+      CoglHandle cogl_material = COGL_INVALID_HANDLE;
+
       priv->parent_texture = g_object_ref_sink (texture);
 
       if (was_visible && CLUTTER_ACTOR_IS_VISIBLE (priv->parent_texture))
         clutter_actor_show (CLUTTER_ACTOR (frame));
+
+      /* set the wrap mode */
+      cogl_material = clutter_texture_get_cogl_material (priv->parent_texture);
+      cogl_material_set_layer_wrap_mode (cogl_material, 0,
+                                         COGL_MATERIAL_WRAP_MODE_REPEAT);
+
+
+      /* The default filter can pull from adjacent pixels which is not what we
+       * want.
+       */
+      cogl_material_set_layer_filters (cogl_material,
+                                       0,
+                                       COGL_MATERIAL_FILTER_NEAREST,
+                                       COGL_MATERIAL_FILTER_NEAREST);
+
+
     }
 
   clutter_actor_queue_relayout (CLUTTER_ACTOR (frame));
@@ -559,7 +639,7 @@ mx_texture_frame_set_parent_texture (MxTextureFrame *frame,
 }
 
 /**
- * mx_texture_frame_set_frame:
+ * mx_texture_frame_set_border_values:
  * @frame: A #MxTextureFrame
  * @top: width of the top slice
  * @right: width of the right slice
@@ -571,11 +651,11 @@ mx_texture_frame_set_parent_texture (MxTextureFrame *frame,
  *
  */
 void
-mx_texture_frame_set_frame (MxTextureFrame *frame,
-                            gfloat          top,
-                            gfloat          right,
-                            gfloat          bottom,
-                            gfloat          left)
+mx_texture_frame_set_border_values (MxTextureFrame *frame,
+                                    gfloat          top,
+                                    gfloat          right,
+                                    gfloat          bottom,
+                                    gfloat          left)
 {
   g_return_if_fail (MX_IS_TEXTURE_FRAME (frame));
 
@@ -583,7 +663,7 @@ mx_texture_frame_set_frame (MxTextureFrame *frame,
 }
 
 /**
- * mx_texture_frame_get_frame:
+ * mx_texture_frame_get_border_values:
  * @frame: A #MxTextureFrame
  * @top: width of the top slice
  * @right: width of the right slice
@@ -594,11 +674,11 @@ mx_texture_frame_set_frame (MxTextureFrame *frame,
  *
  */
 void
-mx_texture_frame_get_frame (MxTextureFrame *frame,
-                            gfloat         *top,
-                            gfloat         *right,
-                            gfloat         *bottom,
-                            gfloat         *left)
+mx_texture_frame_get_border_values (MxTextureFrame *frame,
+                                    gfloat         *top,
+                                    gfloat         *right,
+                                    gfloat         *bottom,
+                                    gfloat         *left)
 {
   MxTextureFramePrivate *priv;
 
