@@ -55,9 +55,7 @@ mx_toggle_get_preferred_width (ClutterActor *actor,
   ClutterActor *background;
   gfloat pref_w;
 
-  background = mx_widget_get_border_image (MX_WIDGET (actor));
-  if (!background)
-    background = mx_widget_get_background_image (MX_WIDGET (actor));
+  background = mx_widget_get_background_image (MX_WIDGET (actor));
 
   if (!background)
     {
@@ -87,9 +85,7 @@ mx_toggle_get_preferred_height (ClutterActor *actor,
   ClutterActor *background;
   gfloat pref_h;
 
-  background = mx_widget_get_border_image (MX_WIDGET (actor));
-  if (!background)
-    background = mx_widget_get_background_image (MX_WIDGET (actor));
+  background = mx_widget_get_background_image (MX_WIDGET (actor));
 
   if (!background)
     {
@@ -115,6 +111,10 @@ mx_toggle_get_preferred_height (ClutterActor *actor,
 static void
 mx_toggle_handle_class_init (MxToggleHandleClass *klass)
 {
+  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+
+  actor_class->get_preferred_height = mx_toggle_get_preferred_height;
+  actor_class->get_preferred_width = mx_toggle_get_preferred_width;
 }
 
 static void
@@ -125,10 +125,13 @@ mx_toggle_handle_init (MxToggleHandle *self)
 /* mx-toggle */
 
 static void mx_toggle_stylable_iface_init (MxStylableIface *iface);
+static void mx_toggle_focusable_iface_init (MxFocusableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxToggle, mx_toggle, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
-                                                mx_toggle_stylable_iface_init))
+                                                mx_toggle_stylable_iface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
+                                                mx_toggle_focusable_iface_init))
 
 #define TOGGLE_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), MX_TYPE_TOGGLE, MxTogglePrivate))
@@ -167,6 +170,43 @@ mx_toggle_stylable_iface_init (MxStylableIface *iface)
   mx_stylable_iface_install_property (iface, MX_TYPE_TOGGLE, pspec);
 }
 
+static MxFocusable*
+mx_toggle_accept_focus (MxFocusable *focusable, MxFocusHint hint)
+{
+  MxTogglePrivate *priv = MX_TOGGLE (focusable)->priv;
+
+  mx_stylable_style_pseudo_class_add (MX_STYLABLE (focusable), "focus");
+  mx_stylable_style_pseudo_class_add (MX_STYLABLE (priv->handle), "focus");
+
+  clutter_actor_grab_key_focus (CLUTTER_ACTOR (focusable));
+
+  return focusable;
+}
+
+static MxFocusable*
+mx_toggle_move_focus (MxFocusable      *focusable,
+                      MxFocusDirection  direction,
+                      MxFocusable      *from)
+{
+  /* check if focus is being moved from us */
+  if (focusable == from)
+    {
+      MxTogglePrivate *priv = MX_TOGGLE (focusable)->priv;
+
+      mx_stylable_style_pseudo_class_remove (MX_STYLABLE (focusable), "focus");
+      mx_stylable_style_pseudo_class_remove (MX_STYLABLE (priv->handle), "focus");
+    }
+
+  return NULL;
+}
+
+static void
+mx_toggle_focusable_iface_init (MxFocusableIface *iface)
+{
+  iface->accept_focus = mx_toggle_accept_focus;
+  iface->move_focus = mx_toggle_move_focus;
+}
+
 static void
 mx_toggle_get_property (GObject    *object,
                         guint       property_id,
@@ -257,8 +297,7 @@ mx_toggle_unmap (ClutterActor *actor)
 {
   MxTogglePrivate *priv = MX_TOGGLE (actor)->priv;
 
-  if (priv->handle)
-    clutter_actor_unmap (priv->handle);
+  clutter_actor_unmap (priv->handle);
 
   CLUTTER_ACTOR_CLASS (mx_toggle_parent_class)->unmap (actor);
 }
@@ -296,7 +335,7 @@ mx_toggle_allocate (ClutterActor          *actor,
       child_box.x2 = child_box.x1 + width;
     }
 
-  handle_w = (int)((child_box.x2 - child_box.x1) / 2.f);
+  clutter_actor_get_preferred_width (priv->handle, -1, NULL, &handle_w);
   toggle_pos = child_box.x2 - handle_w - child_box.x1;
   priv->slide_length = toggle_pos;
 
@@ -436,6 +475,25 @@ mx_toggle_enter_event (ClutterActor         *actor,
   return FALSE;
 }
 
+static gboolean
+mx_toggle_key_press (ClutterActor    *actor,
+                     ClutterKeyEvent *event)
+{
+  if (event->keyval == CLUTTER_KEY_Return ||
+      event->keyval == CLUTTER_KEY_KP_Enter ||
+      event->keyval == CLUTTER_KEY_ISO_Enter ||
+      event->keyval == CLUTTER_KEY_space)
+    {
+      MxToggle *toggle = (MxToggle *) actor;
+
+      mx_toggle_set_active (toggle, !toggle->priv->active);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 mx_toggle_apply_style (MxWidget *widget,
                        MxStyle  *style)
@@ -473,6 +531,7 @@ mx_toggle_class_init (MxToggleClass *klass)
   actor_class->button_press_event = mx_toggle_button_press_event;
   actor_class->leave_event = mx_toggle_leave_event;
   actor_class->enter_event = mx_toggle_enter_event;
+  actor_class->key_press_event = mx_toggle_key_press;
 
   widget_class->apply_style = mx_toggle_apply_style;
 
