@@ -66,6 +66,9 @@ struct _MxBorderImage
   gint left;
 };
 
+gboolean mx_border_image_equal (MxBorderImage *b1, MxBorderImage *b2);
+
+
 void mx_border_image_set_from_string (GValue *value,
                                       const gchar *str,
                                       const gchar *filename);
@@ -161,6 +164,8 @@ typedef enum /*< prefix=MX_ORIENTATION >*/
  * @MX_WINDOW_ROTATION_270: 270 degrees of rotation
  *
  * Defines the clock-wise rotation angle of a window.
+ *
+ * Since: 1.2
  */
 typedef enum /*< prefix=MX_WINDOW_ROTATION >*/
 {
@@ -178,6 +183,8 @@ typedef enum /*< prefix=MX_WINDOW_ROTATION >*/
  * @MX_POSITION_LEFT: The left position
  *
  * Defines the position of an interface element.
+ *
+ * Since: 1.2
  */
 typedef enum /*< prefix=MX_POSITION >*/
 {
@@ -206,6 +213,22 @@ typedef enum /*< prefix=MX_IMAGE_SCALE >*/
   MX_IMAGE_SCALE_CROP
 } MxImageScaleMode;
 
+
+/**
+ * MxTooltipAnimation:
+ * @MX_TOOLTIP_ANIMATION_BOUNCE: Bounce the tooltips when they appear
+ * @MX_TOOLTIP_ANIMATION_FADE: Fade the tooltips on show and hide
+ *
+ * Defines the animation when tooltips are shown and hidden.
+ *
+ * Since: 1.2
+ */
+typedef enum /*< prefix=MX_TOOLTIP_ANIMATION >*/
+{
+  MX_TOOLTIP_ANIMATION_BOUNCE,
+  MX_TOOLTIP_ANIMATION_FADE
+} MxTooltipAnimation;
+
 G_END_DECLS
 
 #endif /* __MX_TYPES_H__ */
