@@ -3,7 +3,7 @@
  * mx-widget.c: Base class for Mx actors
  *
  * Copyright 2007 OpenedHand
- * Copyright 2008, 2009, 2010 Intel Corporation.
+ * Copyright 2008, 2009, 2010, 2012 Intel Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms and conditions of the GNU Lesser General Public License,
@@ -58,12 +58,11 @@ struct _MxWidgetPrivate
   gchar         *style_class;
   MxBorderImage *mx_border_image;
 
-  ClutterActor *border_image;
-  ClutterActor *old_border_image;
+  CoglHandle    border_image;
+  CoglHandle    old_border_image;
   ClutterActor *background_image;
   ClutterColor *bg_color;
 
-  guint         is_hovered : 1;
   guint         is_disabled : 1;
   guint         parent_disabled : 1;
 
@@ -73,6 +72,7 @@ struct _MxWidgetPrivate
   guint         long_press_source;
 
   guint         tooltip_timeout;
+  guint         tooltip_delay;
 };
 
 /**
@@ -97,9 +97,15 @@ enum
   PROP_TOOLTIP_TEXT,
   PROP_MENU,
 
-  PROP_DISABLED
+  PROP_DISABLED,
+
+  PROP_TOOLTIP_DELAY,
+
+  LAST_PROP
 };
 
+static GParamSpec *widget_properties[LAST_PROP];
+
 enum
 {
   LONG_PRESS,
@@ -160,6 +166,10 @@ mx_widget_set_property (GObject      *gobject,
       mx_widget_set_disabled (actor, g_value_get_boolean (value));
       break;
 
+    case PROP_TOOLTIP_DELAY:
+      mx_widget_set_tooltip_delay (actor, g_value_get_int (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -201,6 +211,10 @@ mx_widget_get_property (GObject    *gobject,
       g_value_set_boolean (value, mx_widget_get_disabled (actor));
       break;
 
+    case PROP_TOOLTIP_DELAY:
+      g_value_set_int (value, mx_widget_get_tooltip_delay (actor));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -241,7 +255,7 @@ mx_widget_set_tooltip_timeout (MxWidget *widget)
   mx_widget_remove_tooltip_timeout (widget);
 
   priv->tooltip_timeout =
-    clutter_threads_add_timeout (MX_WIDGET_TOOLTIP_TIMEOUT,
+    clutter_threads_add_timeout (mx_widget_get_tooltip_delay (widget),
                                  mx_widget_tooltip_timeout_cb,
                                  widget);
 }
@@ -260,15 +274,13 @@ mx_widget_dispose (GObject *gobject)
 
   if (priv->border_image)
     {
-      clutter_actor_unparent (priv->border_image);
+      cogl_handle_unref (priv->border_image);
       priv->border_image = NULL;
     }
 
   if (priv->old_border_image)
     {
-      g_object_remove_weak_pointer (G_OBJECT (priv->old_border_image),
-                                    (gpointer)&priv->old_border_image);
-      clutter_actor_unparent (priv->old_border_image);
+      cogl_handle_unref (priv->old_border_image);
       priv->old_border_image = NULL;
     }
 
@@ -321,6 +333,7 @@ mx_widget_allocate (ClutterActor          *actor,
 {
   MxWidgetPrivate *priv = MX_WIDGET (actor)->priv;
   ClutterActorClass *klass;
+  ClutterActorBox frame_box = { 0, 0, box->x2 - box->x1, box->y2 - box->y1 };
 
   klass = CLUTTER_ACTOR_CLASS (mx_widget_parent_class);
   klass->allocate (actor, box, flags);
@@ -357,30 +370,8 @@ mx_widget_allocate (ClutterActor          *actor,
       mx_tooltip_set_tip_area (priv->tooltip, &area);
     }
 
-
-
-  if (priv->border_image)
-    {
-      ClutterActorBox frame_box = {
-        0,
-        0,
-        box->x2 - box->x1,
-        box->y2 - box->y1
-      };
-
-      clutter_actor_allocate (CLUTTER_ACTOR (priv->border_image),
-                              &frame_box,
-                              flags);
-
-      if (priv->old_border_image)
-        clutter_actor_allocate (priv->old_border_image, &frame_box, flags);
-    }
-
   if (priv->background_image)
     {
-      ClutterActorBox frame_box = {
-        0, 0, box->x2 - box->x1, box->y2 - box->y1
-      };
       gfloat w, h;
 
       clutter_actor_get_preferred_size (CLUTTER_ACTOR (priv->background_image),
@@ -443,52 +434,41 @@ mx_widget_allocate (ClutterActor          *actor,
 }
 
 static void
-mx_widget_real_paint_background (MxWidget           *self,
-                                 ClutterActor       *background,
-                                 const ClutterColor *color)
+mx_widget_paint (ClutterActor *actor)
 {
-  /* Default implementation just draws the background
-   * colour and the image on top
-   */
-  if (color && color->alpha != 0)
+  MxWidgetPrivate *priv = MX_WIDGET (actor)->priv;
+
+  ClutterActorBox allocation = { 0, };
+  gfloat width, height;
+
+  clutter_actor_get_allocation_box (actor, &allocation);
+
+
+  width = allocation.x2 - allocation.x1;
+  height = allocation.y2 - allocation.y1;
+
+  /* paint the background color first */
+  if (priv->bg_color && priv->bg_color->alpha != 0)
     {
-      ClutterActor *actor = CLUTTER_ACTOR (self);
-      ClutterActorBox allocation = { 0, };
-      ClutterColor bg_color = *color;
-      gfloat w, h;
+      priv->bg_color->alpha = clutter_actor_get_paint_opacity (actor) *
+        priv->bg_color->alpha / 255;
 
-      bg_color.alpha = clutter_actor_get_paint_opacity (actor)
-                       * bg_color.alpha
-                       / 255;
 
-      clutter_actor_get_allocation_box (actor, &allocation);
-
-      w = allocation.x2 - allocation.x1;
-      h = allocation.y2 - allocation.y1;
-
-      cogl_set_source_color4ub (bg_color.red,
-                                bg_color.green,
-                                bg_color.blue,
-                                bg_color.alpha);
-      cogl_rectangle (0, 0, w, h);
+      cogl_set_source_color4ub (priv->bg_color->red,
+                                priv->bg_color->green,
+                                priv->bg_color->blue,
+                                priv->bg_color->alpha);
+      cogl_rectangle (0, 0, width, height);
     }
 
-  if (background)
-    clutter_actor_paint (background);
-
-  if (self->priv->old_border_image)
-    clutter_actor_paint (self->priv->old_border_image);
-}
-
-static void
-mx_widget_paint (ClutterActor *self)
-{
-  MxWidgetPrivate *priv = MX_WIDGET (self)->priv;
-  MxWidgetClass *klass = MX_WIDGET_GET_CLASS (self);
-
-  klass->paint_background (MX_WIDGET (self),
-                           priv->border_image,
-                           priv->bg_color);
+  if (priv->border_image)
+    mx_texture_frame_paint_texture (priv->border_image,
+                                    clutter_actor_get_paint_opacity (actor),
+                                    priv->mx_border_image->top,
+                                    priv->mx_border_image->right,
+                                    priv->mx_border_image->bottom,
+                                    priv->mx_border_image->left,
+                                    width, height);
 
   if (priv->background_image != NULL)
     clutter_actor_paint (priv->background_image);
@@ -518,12 +498,6 @@ mx_widget_map (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_widget_parent_class)->map (actor);
 
-  if (priv->border_image)
-    clutter_actor_map (priv->border_image);
-
-  if (priv->old_border_image)
-    clutter_actor_map (priv->old_border_image);
-
   if (priv->background_image)
     clutter_actor_map (priv->background_image);
 
@@ -541,12 +515,6 @@ mx_widget_unmap (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_widget_parent_class)->unmap (actor);
 
-  if (priv->border_image)
-    clutter_actor_unmap (priv->border_image);
-
-  if (priv->old_border_image)
-    clutter_actor_unmap (CLUTTER_ACTOR (priv->old_border_image));
-
   if (priv->background_image)
     clutter_actor_unmap (priv->background_image);
 
@@ -557,51 +525,12 @@ mx_widget_unmap (ClutterActor *actor)
     clutter_actor_unmap (CLUTTER_ACTOR (priv->menu));
 }
 
-static void
-old_background_faded_cb (ClutterAnimation *animation, ClutterActor *self)
-{
-  clutter_actor_unparent (self);
-}
-
-/* TODO: move to mx-types.c */
-static gboolean
-mx_border_image_equal (MxBorderImage *v1,
-                       MxBorderImage *v2)
-{
-  if (v1 == v2)
-    return FALSE;
-
-  if (!v1 && v2)
-    return TRUE;
-
-  if (!v2 && v1)
-    return TRUE;
-
-  if (g_strcmp0 (v1->uri, v2->uri))
-    return TRUE;
-
-  if (v1->top != v2->top)
-    return TRUE;
-
-  if (v1->right != v2->right)
-    return TRUE;
-
-  if (v1->bottom != v2->bottom)
-    return TRUE;
-
-  if (v1->left != v2->left)
-    return TRUE;
-
-  return FALSE;
-}
-
 static void
 mx_widget_style_changed (MxStylable *self, MxStyleChangedFlags flags)
 {
   MxWidgetPrivate *priv = MX_WIDGET (self)->priv;
   MxBorderImage *border_image = NULL, *background_image = NULL;
   MxTextureCache *texture_cache;
-  ClutterTexture *texture;
   gchar *bg_file;
   MxPadding *padding = NULL;
   gboolean relayout_needed = FALSE;
@@ -663,36 +592,15 @@ mx_widget_style_changed (MxStylable *self, MxStyleChangedFlags flags)
   /* border-image property */
 
   /* check whether the border-image has changed */
-  border_image_changed = mx_border_image_equal (priv->mx_border_image,
-                                                border_image);
+  border_image_changed = !mx_border_image_equal (priv->mx_border_image,
+                                                 border_image);
 
   /* remove the old border-image if it has changed */
   if (border_image_changed && priv->border_image)
     {
       if (duration == 0)
         {
-          clutter_actor_unparent (priv->border_image);
-        }
-      else
-        {
-          if (priv->old_border_image)
-            {
-              g_object_remove_weak_pointer (G_OBJECT (priv->old_border_image),
-                                            (gpointer)&priv->old_border_image);
-              clutter_actor_unparent (priv->old_border_image);
-            }
-
-          priv->old_border_image = priv->border_image;
-          g_object_add_weak_pointer (G_OBJECT (priv->old_border_image),
-                                     (gpointer)&priv->old_border_image);
-          clutter_actor_animate (priv->old_border_image,
-                                 CLUTTER_LINEAR,
-                                 duration,
-                                 "opacity", 0x0,
-                                 "signal-after::completed",
-                                   old_background_faded_cb,
-                                   priv->old_border_image,
-                                 NULL);
+          cogl_handle_unref (priv->border_image);
         }
 
       priv->border_image = NULL;
@@ -702,26 +610,8 @@ mx_widget_style_changed (MxStylable *self, MxStyleChangedFlags flags)
   /* apply the new border-image, as long as there is a valid URI */
   if (border_image_changed && border_image && border_image->uri)
     {
-      gint border_left, border_right, border_top, border_bottom;
-      gint width, height;
-
-      texture = mx_texture_cache_get_texture (texture_cache,
-                                              border_image->uri);
-
-      clutter_texture_get_base_size (CLUTTER_TEXTURE (texture),
-                                     &width, &height);
-
-      border_left = border_image->left;
-      border_top = border_image->top;
-      border_right = border_image->right;
-      border_bottom = border_image->bottom;
-
-      priv->border_image = mx_texture_frame_new (texture,
-                                                 border_top,
-                                                 border_right,
-                                                 border_bottom,
-                                                 border_left);
-      clutter_actor_set_parent (priv->border_image, CLUTTER_ACTOR (self));
+      priv->border_image = mx_texture_cache_get_cogl_texture (texture_cache,
+                                                              border_image->uri);
 
       has_changed = TRUE;
       relayout_needed = TRUE;
@@ -755,8 +645,8 @@ mx_widget_style_changed (MxStylable *self, MxStyleChangedFlags flags)
       if (bg_file != NULL &&
           strcmp (bg_file, "none"))
         {
-          texture = mx_texture_cache_get_texture (texture_cache, bg_file);
-          priv->background_image = (ClutterActor*) texture;
+          priv->background_image =
+            (ClutterActor*) mx_texture_cache_get_texture (texture_cache, bg_file);
 
           if (priv->background_image != NULL)
             {
@@ -789,13 +679,11 @@ mx_widget_enter (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
 {
   MxWidget *widget = MX_WIDGET (actor);
-  MxWidgetPrivate *priv = widget->priv;
 
   /* This is also expected to handle enter events from child actors
      because they will bubble up */
 
-  mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
-  priv->is_hovered = TRUE;
+  mx_stylable_style_pseudo_class_add (MX_STYLABLE (widget), "hover");
 
   return FALSE;
 }
@@ -805,7 +693,6 @@ mx_widget_leave (ClutterActor         *actor,
                  ClutterCrossingEvent *event)
 {
   MxWidget *widget = MX_WIDGET (actor);
-  MxWidgetPrivate *priv = widget->priv;
 
   /* If the leave event is simply moving to a child actor then we'll
      ignore it so that the actor will retain its hover state until the
@@ -816,8 +703,9 @@ mx_widget_leave (ClutterActor         *actor,
 
   mx_widget_hide_tooltip (widget);
 
-  mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "");
-  priv->is_hovered = FALSE;
+  mx_widget_long_press_cancel (widget);
+  mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "active");
+  mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "hover");
 
   return FALSE;
 }
@@ -938,11 +826,11 @@ mx_widget_button_press (ClutterActor       *actor,
       return TRUE;
 
   if (event->button == 1)
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "active");
+    mx_stylable_style_pseudo_class_add (MX_STYLABLE (widget), "active");
 
   mx_widget_long_press_query (widget, event);
 
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -955,16 +843,11 @@ mx_widget_button_release (ClutterActor       *actor,
       return TRUE;
 
   if (event->button == 1)
-    {
-      if (widget->priv->is_hovered)
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
-      else
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "");
-    }
+    mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "active");
 
   mx_widget_long_press_cancel (widget);
 
-  return TRUE;
+  return FALSE;
 }
 
 static void
@@ -997,6 +880,18 @@ _mx_widget_propagate_disabled (ClutterContainer *container,
 
           child_priv->parent_disabled = disabled;
 
+          /* emit the "notify" signal for the "disabled" property */
+          g_object_notify_by_pspec (G_OBJECT (container),
+                                    widget_properties[PROP_DISABLED]);
+
+
+          if (disabled)
+            mx_stylable_style_pseudo_class_add (MX_STYLABLE (child),
+                                                "disabled");
+          else
+            mx_stylable_style_pseudo_class_remove (MX_STYLABLE (child),
+                                                   "disabled");
+
           /* If this child has already been disabled explicitly,
            * we don't need to recurse through its children to set
            * the 'parent_disabled' flag, as it'll already be set.
@@ -1008,6 +903,8 @@ _mx_widget_propagate_disabled (ClutterContainer *container,
       if (CLUTTER_IS_CONTAINER (child))
         _mx_widget_propagate_disabled ((ClutterContainer *) child, disabled);
     }
+
+  g_list_free (children);
 }
 
 static void
@@ -1045,13 +942,24 @@ mx_widget_parent_set (ClutterActor *actor,
     }
 }
 
+static gboolean
+mx_widget_get_paint_volume (ClutterActor       *actor,
+                            ClutterPaintVolume *volume)
+{
+  /* the allocation of scrollable widgets cannot be used as the paint volume
+   * because it does not account for any transformations applied during
+   * scrolling */
+  if (MX_IS_SCROLLABLE (actor))
+    return FALSE;
+
+  return clutter_paint_volume_set_from_allocation (volume, actor);
+}
 
 static void
 mx_widget_class_init (MxWidgetClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (MxWidgetPrivate));
 
@@ -1075,49 +983,70 @@ mx_widget_class_init (MxWidgetClass *klass)
   actor_class->hide = mx_widget_hide;
   actor_class->parent_set = mx_widget_parent_set;
 
-  klass->paint_background = mx_widget_real_paint_background;
+  actor_class->get_paint_volume = mx_widget_get_paint_volume;
 
   /* stylable interface properties */
   g_object_class_override_property (gobject_class, PROP_STYLE, "style");
+  widget_properties[PROP_STYLE] = g_object_class_find_property (gobject_class,
+                                                                "style");
   g_object_class_override_property (gobject_class, PROP_STYLE_CLASS,
                                     "style-class");
+  widget_properties[PROP_STYLE_CLASS] =
+    g_object_class_find_property (gobject_class, "style-class");
+
   g_object_class_override_property (gobject_class, PROP_STYLE_PSEUDO_CLASS,
                                     "style-pseudo-class");
+  widget_properties[PROP_STYLE_PSEUDO_CLASS] =
+    g_object_class_find_property (gobject_class, "style-pseudo-class");
 
   /**
    * MxWidget:tooltip-text:
    *
    * text displayed on the tooltip
    */
-  pspec = g_param_spec_string ("tooltip-text",
-                               "Tooltip Text",
-                               "Text displayed on the tooltip",
-                               "",
-                               MX_PARAM_READWRITE | MX_PARAM_TRANSLATEABLE);
-  g_object_class_install_property (gobject_class, PROP_TOOLTIP_TEXT, pspec);
+  widget_properties[PROP_TOOLTIP_TEXT] =
+    g_param_spec_string ("tooltip-text",
+                         "Tooltip Text",
+                         "Text displayed on the tooltip",
+                         "",
+                         MX_PARAM_READWRITE | MX_PARAM_TRANSLATEABLE);
+  g_object_class_install_property (gobject_class, PROP_TOOLTIP_TEXT,
+                                   widget_properties[PROP_TOOLTIP_TEXT]);
 
   /**
    * MxWidget:menu:
    *
    * #MxMenu associated with the widget.
-   * See mx_widget_show_menu().
    */
-  pspec = g_param_spec_object ("menu",
-                               "Menu",
-                               "The MxMenu associated with the widget",
-                               MX_TYPE_MENU,
-                               MX_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_MENU, pspec);
-
-  pspec = g_param_spec_boolean ("disabled",
-                               "Disabled",
-                               "Whether disabled styling should be applied and"
-                               " the widget made unreactive.",
-                               FALSE,
-                               MX_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_DISABLED, pspec);
+  widget_properties[PROP_MENU] =
+    g_param_spec_object ("menu",
+                         "Menu",
+                         "The MxMenu associated with the widget",
+                         MX_TYPE_MENU,
+                         MX_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_MENU,
+                                   widget_properties[PROP_MENU]);
 
+  widget_properties[PROP_DISABLED] = g_param_spec_boolean ("disabled",
+                                                           "Disabled",
+                                                           "Whether disabled"
+                                                           " styling should be"
+                                                           " applied and the"
+                                                           " widget made"
+                                                           " unreactive.",
+                                                           FALSE,
+                                                           MX_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_DISABLED,
+                                   widget_properties[PROP_DISABLED]);
 
+  widget_properties[PROP_TOOLTIP_DELAY] =
+    g_param_spec_int ("tooltip-delay",
+                      "Tooltip delay",
+                      "Delay time before showing the tooltip",
+                      0, G_MAXINT, MX_WIDGET_TOOLTIP_TIMEOUT,
+                      MX_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+  g_object_class_install_property (gobject_class, PROP_TOOLTIP_DELAY,
+                                   widget_properties[PROP_TOOLTIP_DELAY]);
 
   /**
    * MxWidget::long-press:
@@ -1175,7 +1104,8 @@ _mx_widget_set_style_class (MxStylable  *actor,
       g_free (priv->style_class);
       priv->style_class = g_strdup (style_class);
 
-      g_object_notify (G_OBJECT (actor), "style-class");
+      g_object_notify_by_pspec (G_OBJECT (actor),
+                                widget_properties[PROP_STYLE_CLASS]);
     }
 }
 
@@ -1194,17 +1124,15 @@ _mx_stylable_set_style_pseudo_class (MxStylable  *actor,
       g_free (priv->pseudo_class);
       priv->pseudo_class = g_strdup (pseudo_class);
 
-      g_object_notify (G_OBJECT (actor), "style-pseudo-class");
+      g_object_notify_by_pspec (G_OBJECT (actor),
+                                widget_properties[PROP_STYLE_PSEUDO_CLASS]);
     }
 }
 
 static const gchar*
 _mx_stylable_get_style_pseudo_class (MxStylable *actor)
 {
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
-    return "disabled";
-  else
-    return ((MxWidget *) actor)->priv->pseudo_class;
+  return ((MxWidget *) actor)->priv->pseudo_class;
 }
 
 
@@ -1325,9 +1253,7 @@ mx_stylable_iface_init (MxStylableIface *iface)
 static void
 mx_widget_init (MxWidget *actor)
 {
-  MxWidgetPrivate *priv;
-
-  actor->priv = priv = MX_WIDGET_GET_PRIVATE (actor);
+  actor->priv = MX_WIDGET_GET_PRIVATE (actor);
 
   /* set the default style */
   mx_stylable_set_style (MX_STYLABLE (actor), mx_style_get_default ());
@@ -1336,24 +1262,6 @@ mx_widget_init (MxWidget *actor)
   mx_stylable_connect_change_notifiers (MX_STYLABLE (actor));
 }
 
-
-/**
- * mx_widget_get_border_image:
- * @actor: A #MxWidget
- *
- * Get the texture used as the border image. This is set using the
- * "border-image" CSS property. This function should normally only be used
- * by subclasses.
- *
- * Returns: (transfer none): #ClutterActor
- */
-ClutterActor *
-mx_widget_get_border_image (MxWidget *actor)
-{
-  MxWidgetPrivate *priv = MX_WIDGET (actor)->priv;
-  return priv->border_image;
-}
-
 /**
  * mx_widget_get_background_image:
  * @actor: A #MxWidget
@@ -1371,6 +1279,23 @@ mx_widget_get_background_image (MxWidget *actor)
   return priv->background_image;
 }
 
+/**
+ * mx_widget_get_background_color:
+ * @actor: A #MxWidget
+ *
+ * Get the color used as the background. This is set using the
+ * "background-color" CSS property. This function should normally only
+ * be used by subclasses.
+ *
+ * Returns: (transfer none): a #ClutterColor
+ */
+ClutterColor *
+mx_widget_get_background_color (MxWidget *actor)
+{
+  MxWidgetPrivate *priv = MX_WIDGET (actor)->priv;
+  return priv->bg_color;
+}
+
 /**
  * mx_widget_get_padding:
  * @widget: A #MxWidget
@@ -1411,6 +1336,8 @@ mx_widget_set_has_tooltip (MxWidget *widget,
         {
           priv->tooltip = g_object_new (MX_TYPE_TOOLTIP, NULL);
           clutter_actor_set_parent (CLUTTER_ACTOR (priv->tooltip), actor);
+          if (mx_stylable_style_pseudo_class_contains (MX_STYLABLE (widget), "hover"))
+            mx_widget_show_tooltip (widget);
         }
     }
   else
@@ -1465,7 +1392,8 @@ mx_widget_set_tooltip_text (MxWidget    *widget,
   if (priv->tooltip)
     mx_tooltip_set_text (priv->tooltip, text);
 
-  g_object_notify (G_OBJECT (widget), "tooltip-text");
+  g_object_notify_by_pspec (G_OBJECT (widget),
+                            widget_properties[PROP_TOOLTIP_TEXT]);
 }
 
 /**
@@ -1561,33 +1489,6 @@ mx_widget_hide_tooltip (MxWidget *widget)
     mx_tooltip_hide (widget->priv->tooltip);
 }
 
-/**
- * mx_widget_paint_background:
- * @widget: a #MxWidget
- *
- * Invokes #MxWidget::paint_background() using the default background
- * image and/or color from the @widget style
- *
- * This function should be used by subclasses of #MxWidget that override
- * the paint() virtual function and cannot chain up
- */
-void
-mx_widget_paint_background (MxWidget *self)
-{
-  MxWidgetPrivate *priv;
-  MxWidgetClass *klass;
-
-  g_return_if_fail (MX_IS_WIDGET (self));
-
-  priv = self->priv;
-
-  klass = MX_WIDGET_GET_CLASS (self);
-  klass->paint_background (MX_WIDGET (self),
-                          priv->border_image,
-                          priv->bg_color);
-
-}
-
 /**
  * mx_widget_get_available_area:
  * @widget: A #MxWidget
@@ -1648,7 +1549,7 @@ mx_widget_set_menu (MxWidget *widget,
  *
  * Get the object in the #MxWidget:menu property.
  *
- * Returns: The current object in the "menu" property.
+ * Returns: (transfer none): The current object in the "menu" property.
  */
 MxMenu *
 mx_widget_get_menu (MxWidget *widget)
@@ -1678,6 +1579,11 @@ mx_widget_set_disabled (MxWidget *widget,
     {
       priv->is_disabled = disabled;
 
+      if (disabled)
+        mx_stylable_style_pseudo_class_add (MX_STYLABLE (widget), "disabled");
+      else
+        mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "disabled");
+
       /* Propagate the disabled state to our children, if necessary */
       if (!priv->parent_disabled && CLUTTER_IS_CONTAINER (widget))
         _mx_widget_propagate_disabled ((ClutterContainer *) widget, disabled);
@@ -1689,7 +1595,8 @@ mx_widget_set_disabled (MxWidget *widget,
 
       mx_stylable_style_changed (MX_STYLABLE (widget), 0);
 
-      g_object_notify (G_OBJECT (widget), "disabled");
+      g_object_notify_by_pspec (G_OBJECT (widget),
+                                widget_properties[PROP_DISABLED]);
     }
 }
 
@@ -1706,6 +1613,43 @@ mx_widget_get_disabled (MxWidget *widget)
   return widget->priv->is_disabled || widget->priv->parent_disabled;
 }
 
+/**
+ * mx_widget_set_tooltip_delay:
+ * @widget: an #MxWidget
+ *
+ * Set the value, in milliseconds, of the "tooltip-delay" property.
+ * This is initially set to MX_WIDGET_TOOLTIP_TIMEOUT.
+ */
+void
+mx_widget_set_tooltip_delay (MxWidget *widget,
+                             guint delay)
+{
+  g_return_if_fail (MX_IS_WIDGET (widget));
+
+  if (widget->priv->tooltip_delay != delay)
+    {
+      widget->priv->tooltip_delay = delay;
+      g_object_notify_by_pspec (G_OBJECT (widget),
+                                widget_properties[PROP_TOOLTIP_DELAY]);
+    }
+}
+
+/**
+ * mx_widget_get_tooltip_delay:
+ * @widget: an #MxWidget
+ *
+ * Get the value of the "tooltip-delay" property.
+ *
+ * Returns: the current delay value in milliseconds
+ */
+guint
+mx_widget_get_tooltip_delay (MxWidget *widget)
+{
+  g_return_val_if_fail (MX_IS_WIDGET (widget), 0);
+
+  return widget->priv->tooltip_delay;
+}
+
 /* Support translateable strings from JSON */
 static void
 widget_scriptable_set_custom_property (ClutterScriptable *scriptable,
@@ -1717,7 +1661,7 @@ widget_scriptable_set_custom_property (ClutterScriptable *scriptable,
 
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (scriptable), name);
 
-  if (pspec->flags & MX_PARAM_TRANSLATEABLE &&
+  if (pspec && pspec->flags & MX_PARAM_TRANSLATEABLE &&
       pspec->value_type == G_TYPE_STRING)
     {
       g_object_set (scriptable, name,
