@@ -130,7 +130,8 @@ main (int argc, char *argv[])
       return 1;
     }
 
-  clutter_init (&argc, &argv);
+  if (clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS)
+    return 1;
 
   stage = clutter_stage_get_default ();
   clutter_stage_set_user_resizable ((ClutterStage*) stage, TRUE);
